;// ____________________________________________________________________
;//
;// Copyright (c) 2003, Andrew N. Sloss, Dominic Symes, Chris Wright
;// All rights reserved.
;// ____________________________________________________________________
;// 
;// NON-COMMERCIAL USE License
;// 
;// Redistribution and use in source and binary forms, with or without 
;// modification, are permitted provided that the following conditions 
;// are met: 
;//
;// 1. For NON-COMMERCIAL USE only.
;// 
;// 2. Redistributions of source code must retain the above copyright 
;//    notice, this list of conditions and the following disclaimer. 
;//
;// 3. Redistributions in binary form must reproduce the above 
;//    copyright notice, this list of conditions and the following 
;//    disclaimer in the documentation and/or other materials provided 
;//    with the distribution. 
;//
;// 4. All advertising materials mentioning features or use of this 
;//    software must display the following acknowledgement:
;//
;//    This product includes software developed by Andrew N. Sloss,
;//    Chris Wright and Dominic Symes. 
;//
;//  THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY 
;//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
;//  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
;//  PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE CONTRIBUTORS BE 
;//  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
;//  OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
;//  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
;//  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
;//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
;//  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
;//  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
;//  OF SUCH DAMAGE. 
;//
;// If you have questions about this license or would like a different
;// license please email : andrew@sloss.net
;//
;// Section 8.5: FFT

        AREA    ch08_5, CODE, READONLY

        EXPORT  fft_16_arm9m
        EXPORT  fft_16_arm9e

;// Example 8.17
        
        ; complex load, x=[a], a+=offset
        MACRO
        C_LDR   $x, $a, $offset
        LDRSH   $x._i, [$a, #2]
        LDRSH   $x._r, [$a], $offset
        MEND
        
        ; complex store, [a]=x, a+=offset
        MACRO
        C_STR   $x, $a, $offset
        STRH    $x._i, [$a, #2]
        STRH    $x._r, [$a], $offset
        MEND        
        
        ; Four point complex Fast Fourier Transform
        ;
        ; (x0,x1,x2,y3)=DFT4(x0,x2>>s,x1>>s,x3>>s)/4
        ;
        ; x0 = (x0 +   (x2>>s) + (x1>>s) +   (x3>>s))/4
        ; x1 = (x0 - i*(x2>>s) - (x1>>s) + i*(x3>>s))/4
        ; x2 = (x0 -   (x2>>s) + (x1>>s) -   (x3>>s))/4
        ; y3 = (x0 + i*(x2>>s) - (x1>>s) - i*(x3>>s))/4
        ;
        MACRO
        C_FFT4  $s
        ; (x2,x3) = (x2+x3, x2-x3)
        ADD     x2_r, x2_r, x3_r
        ADD     x2_i, x2_i, x3_i
        SUB     x3_r, x2_r, x3_r, LSL#1
        SUB     x3_i, x2_i, x3_i, LSL#1
        ; (x0,x1) = (x0+(x1>>s), x0-(x1>>s))/4
        MOV     x0_r, x0_r, ASR#2
        MOV     x0_i, x0_i, ASR#2
        ADD     x0_r, x0_r, x1_r, ASR#(2+$s)
        ADD     x0_i, x0_i, x1_i, ASR#(2+$s)
        SUB     x1_r, x0_r, x1_r, ASR#(1+$s)
        SUB     x1_i, x0_i, x1_i, ASR#(1+$s)
        ; (x0,x2) = (x0+(x2>>s)/4, x0-(x2>>s)/4)
        ADD     x0_r, x0_r, x2_r, ASR#(2+$s)
        ADD     x0_i, x0_i, x2_i, ASR#(2+$s)
        SUB     x2_r, x0_r, x2_r, ASR#(1+$s)
        SUB     x2_i, x0_i, x2_i, ASR#(1+$s)
        ; (x1,y3) = (x1-i*(x3>>s)/4, x1+i*(x3>>s)/4)
        ADD     x1_r, x1_r, x3_i, ASR#(2+$s)
        SUB     x1_i, x1_i, x3_r, ASR#(2+$s)
        SUB     y3_r, x1_r, x3_i, ASR#(1+$s)
        ADD     y3_i, x1_i, x3_r, ASR#(1+$s)
        MEND
        
        ; Complex conjugate multiply a=(xr+i*xi)*(cr-i*ci)
        ;  x = xr + i*xi
        ;  w = (cr-ci) + i*ci
        MACRO
        C_MUL9m $a, $x, $w
        SUB     t1, $x._i, $x._r        ; (xi-xr)
        MUL     t0, t1, $w._i           ; (xi-xr)*ci
        ADD     t1, $w._r, $w._i, LSL#1 ; (cr+ci)
        MLA     $a._i, $x._i, $w._r, t0 ; xi*cr-xr*ci
        MLA     $a._r, $x._r, t1, t0    ; xr*cr+xi*ci
        MEND
        
y       RN 0    ; output complex array y[]
c       RN 0    ; coefficient array
x       RN 1    ; input complex array x[]
N       RN 2    ; number of samples (a power of 2)
S       RN 2    ; the number of blocks
R       RN 3    ; the number of samples in each block
x0_r    RN 4    ; data register (real part)
x0_i    RN 5    ; data register (imaginary part)
x1_r    RN 6
x1_i    RN 7
x2_r    RN 8
x2_i    RN 9
x3_r    RN 10
x3_i    RN 11
y3_r    RN x3_i
y3_i    RN x3_r
t0      RN 12   ; scratch register
t1      RN 14

        ; void fft_16_arm9m(short *y, short *x, unsigned int N)
fft_16_arm9m
        STMFD   sp!, {r4-r11, lr}
        MOV     t0, #0            ; bit reversed counter
first_stage_arm9m
        ; first stage load and bit reverse
        ADD     t1,  x, t0, LSL#2
        C_LDR   x0, t1, N
        C_LDR   x2, t1, N
        C_LDR   x1, t1, N
        C_LDR   x3, t1, N
        C_FFT4  0
        C_STR   x0, y, #4
        C_STR   x1, y, #4
        C_STR   x2, y, #4
        C_STR   y3, y, #4
        EOR     t0, t0, N, LSR#3  ; increment third bit
        TST     t0, N, LSR#3      ; from the top
        BNE     first_stage_arm9m
        EOR     t0, t0, N, LSR#4  ; increment fourth bit
        TST     t0, N, LSR#4      ; from the top
        BNE     first_stage_arm9m
        MOV     t1, N, LSR#5      ; increment fifth
bit_reversed_count_arm9m          ; bits downward
        EOR     t0, t0, t1
        TST     t0, t1
        BNE     first_stage_arm9m
        MOVS    t1, t1, LSR#1
        BNE     bit_reversed_count_arm9m
        ; finished the first stage
        SUB     x, y, N, LSL#2    ; x = working buffer
        MOV     R, #16
        MOVS    S, N, LSR#4
        LDMEQFD sp!, {r4-r11, pc}
        ADR     c, fft_table_arm9m
next_stage_arm9m
        ; S = the number of blocks
        ; R = the number of samples in each block
        STMFD   sp!, {x, S}
        ADD     t0, R, R, LSL#1
        ADD     x, x, t0
        SUB     S, S, #1<<16
next_block_arm9m
        ADD     S, S, R, LSL#(16-2)
next_butterfly_arm9m
        ; S=((number butterflies left-1)<<16)
        ;   + (number of blocks left)
        C_LDR   x0, x, -R
        C_LDR   x3, c, #4
        C_MUL9m x3, x0, x3
        C_LDR   x0, x, -R
        C_LDR   x2, c, #4
        C_MUL9m x2, x0, x2
        C_LDR   x0, x, -R
        C_LDR   x1, c, #4
        C_MUL9m x1, x0, x1
        C_LDR   x0, x, #0
        C_FFT4  14              ; coefficients are Q14
        C_STR   x0, x, R
        C_STR   x1, x, R
        C_STR   x2, x, R
        C_STR   y3, x, #4
        SUBS    S, S, #1<<16
        BGE     next_butterfly_arm9m
        ADD     t0, R, R, LSL#1
        ADD     x, x, t0
        SUB     S, S, #1
        MOVS    t1, S, LSL#16
        SUBNE   c, c, t0
        BNE     next_block_arm9m
        LDMFD   sp!, {x, S}
        MOV     R, R, LSL#2     ; quadrouple block size
        MOVS    S, S, LSR#2     ; quarter number of blocks
        BNE     next_stage_arm9m
        LDMFD   sp!, {r4-r11, pc}
        
fft_table_arm9m
        ; FFT twiddle table of triplets E(3t), E(t), E(2t)
        ; Where E(t)=(cos(t)-sin(t))+i*sin(t) at Q14
        ; N=16 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x4000,0x0000, 0x4000,0x0000, 0x4000,0x0000
        DCW 0xdd5d,0x3b21, 0x22a3,0x187e, 0x0000,0x2d41
        DCW 0xa57e,0x2d41, 0x0000,0x2d41, 0xc000,0x4000
        DCW 0xdd5d,0xe782, 0xdd5d,0x3b21, 0xa57e,0x2d41
        ; N=64 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x4000,0x0000, 0x4000,0x0000, 0x4000,0x0000
        DCW 0x2aaa,0x1294, 0x396b,0x0646, 0x3249,0x0c7c
        DCW 0x11a8,0x238e, 0x3249,0x0c7c, 0x22a3,0x187e
        DCW 0xf721,0x3179, 0x2aaa,0x1294, 0x11a8,0x238e
        DCW 0xdd5d,0x3b21, 0x22a3,0x187e, 0x0000,0x2d41
        DCW 0xc695,0x3fb1, 0x1a46,0x1e2b, 0xee58,0x3537
        DCW 0xb4be,0x3ec5, 0x11a8,0x238e, 0xdd5d,0x3b21
        DCW 0xa963,0x3871, 0x08df,0x289a, 0xcdb7,0x3ec5
        DCW 0xa57e,0x2d41, 0x0000,0x2d41, 0xc000,0x4000
        DCW 0xa963,0x1e2b, 0xf721,0x3179, 0xb4be,0x3ec5
        DCW 0xb4be,0x0c7c, 0xee58,0x3537, 0xac61,0x3b21
        DCW 0xc695,0xf9ba, 0xe5ba,0x3871, 0xa73b,0x3537
        DCW 0xdd5d,0xe782, 0xdd5d,0x3b21, 0xa57e,0x2d41
        DCW 0xf721,0xd766, 0xd556,0x3d3f, 0xa73b,0x238e
        DCW 0x11a8,0xcac9, 0xcdb7,0x3ec5, 0xac61,0x187e
        DCW 0x2aaa,0xc2c1, 0xc695,0x3fb1, 0xb4be,0x0c7c
        ; N=256 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x4000,0x0000, 0x4000,0x0000, 0x4000,0x0000
        DCW 0x3b1e,0x04b5, 0x3e69,0x0192, 0x3cc8,0x0324
        DCW 0x35eb,0x0964, 0x3cc8,0x0324, 0x396b,0x0646
        DCW 0x306c,0x0e06, 0x3b1e,0x04b5, 0x35eb,0x0964
        DCW 0x2aaa,0x1294, 0x396b,0x0646, 0x3249,0x0c7c
        DCW 0x24ae,0x1709, 0x37af,0x07d6, 0x2e88,0x0f8d
        DCW 0x1e7e,0x1b5d, 0x35eb,0x0964, 0x2aaa,0x1294
        DCW 0x1824,0x1f8c, 0x341e,0x0af1, 0x26b3,0x1590
        DCW 0x11a8,0x238e, 0x3249,0x0c7c, 0x22a3,0x187e
        DCW 0x0b14,0x2760, 0x306c,0x0e06, 0x1e7e,0x1b5d
        DCW 0x0471,0x2afb, 0x2e88,0x0f8d, 0x1a46,0x1e2b
        DCW 0xfdc7,0x2e5a, 0x2c9d,0x1112, 0x15fe,0x20e7
        DCW 0xf721,0x3179, 0x2aaa,0x1294, 0x11a8,0x238e
        DCW 0xf087,0x3453, 0x28b2,0x1413, 0x0d48,0x2620
        DCW 0xea02,0x36e5, 0x26b3,0x1590, 0x08df,0x289a
        DCW 0xe39c,0x392b, 0x24ae,0x1709, 0x0471,0x2afb
        DCW 0xdd5d,0x3b21, 0x22a3,0x187e, 0x0000,0x2d41
        DCW 0xd74e,0x3cc5, 0x2093,0x19ef, 0xfb8f,0x2f6c
        DCW 0xd178,0x3e15, 0x1e7e,0x1b5d, 0xf721,0x3179
        DCW 0xcbe2,0x3f0f, 0x1c64,0x1cc6, 0xf2b8,0x3368
        DCW 0xc695,0x3fb1, 0x1a46,0x1e2b, 0xee58,0x3537
        DCW 0xc197,0x3ffb, 0x1824,0x1f8c, 0xea02,0x36e5
        DCW 0xbcf0,0x3fec, 0x15fe,0x20e7, 0xe5ba,0x3871
        DCW 0xb8a6,0x3f85, 0x13d5,0x223d, 0xe182,0x39db
        DCW 0xb4be,0x3ec5, 0x11a8,0x238e, 0xdd5d,0x3b21
        DCW 0xb140,0x3daf, 0x0f79,0x24da, 0xd94d,0x3c42
        DCW 0xae2e,0x3c42, 0x0d48,0x2620, 0xd556,0x3d3f
        DCW 0xab8e,0x3a82, 0x0b14,0x2760, 0xd178,0x3e15
        DCW 0xa963,0x3871, 0x08df,0x289a, 0xcdb7,0x3ec5
        DCW 0xa7b1,0x3612, 0x06a9,0x29ce, 0xca15,0x3f4f
        DCW 0xa678,0x3368, 0x0471,0x2afb, 0xc695,0x3fb1
        DCW 0xa5bc,0x3076, 0x0239,0x2c21, 0xc338,0x3fec
        DCW 0xa57e,0x2d41, 0x0000,0x2d41, 0xc000,0x4000
        DCW 0xa5bc,0x29ce, 0xfdc7,0x2e5a, 0xbcf0,0x3fec
        DCW 0xa678,0x2620, 0xfb8f,0x2f6c, 0xba09,0x3fb1
        DCW 0xa7b1,0x223d, 0xf957,0x3076, 0xb74d,0x3f4f
        DCW 0xa963,0x1e2b, 0xf721,0x3179, 0xb4be,0x3ec5
        DCW 0xab8e,0x19ef, 0xf4ec,0x3274, 0xb25e,0x3e15
        DCW 0xae2e,0x1590, 0xf2b8,0x3368, 0xb02d,0x3d3f
        DCW 0xb140,0x1112, 0xf087,0x3453, 0xae2e,0x3c42
        DCW 0xb4be,0x0c7c, 0xee58,0x3537, 0xac61,0x3b21
        DCW 0xb8a6,0x07d6, 0xec2b,0x3612, 0xaac8,0x39db
        DCW 0xbcf0,0x0324, 0xea02,0x36e5, 0xa963,0x3871
        DCW 0xc197,0xfe6e, 0xe7dc,0x37b0, 0xa834,0x36e5
        DCW 0xc695,0xf9ba, 0xe5ba,0x3871, 0xa73b,0x3537
        DCW 0xcbe2,0xf50f, 0xe39c,0x392b, 0xa678,0x3368
        DCW 0xd178,0xf073, 0xe182,0x39db, 0xa5ed,0x3179
        DCW 0xd74e,0xebed, 0xdf6d,0x3a82, 0xa599,0x2f6c
        DCW 0xdd5d,0xe782, 0xdd5d,0x3b21, 0xa57e,0x2d41
        DCW 0xe39c,0xe33a, 0xdb52,0x3bb6, 0xa599,0x2afb
        DCW 0xea02,0xdf19, 0xd94d,0x3c42, 0xa5ed,0x289a
        DCW 0xf087,0xdb26, 0xd74e,0x3cc5, 0xa678,0x2620
        DCW 0xf721,0xd766, 0xd556,0x3d3f, 0xa73b,0x238e
        DCW 0xfdc7,0xd3df, 0xd363,0x3daf, 0xa834,0x20e7
        DCW 0x0471,0xd094, 0xd178,0x3e15, 0xa963,0x1e2b
        DCW 0x0b14,0xcd8c, 0xcf94,0x3e72, 0xaac8,0x1b5d
        DCW 0x11a8,0xcac9, 0xcdb7,0x3ec5, 0xac61,0x187e
        DCW 0x1824,0xc850, 0xcbe2,0x3f0f, 0xae2e,0x1590
        DCW 0x1e7e,0xc625, 0xca15,0x3f4f, 0xb02d,0x1294
        DCW 0x24ae,0xc44a, 0xc851,0x3f85, 0xb25e,0x0f8d
        DCW 0x2aaa,0xc2c1, 0xc695,0x3fb1, 0xb4be,0x0c7c
        DCW 0x306c,0xc18e, 0xc4e2,0x3fd4, 0xb74d,0x0964
        DCW 0x35eb,0xc0b1, 0xc338,0x3fec, 0xba09,0x0646
        DCW 0x3b1e,0xc02c, 0xc197,0x3ffb, 0xbcf0,0x0324
        ; N=1024 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x4000,0x0000, 0x4000,0x0000, 0x4000,0x0000
        DCW 0x3ed0,0x012e, 0x3f9b,0x0065, 0x3f36,0x00c9
        DCW 0x3d9a,0x025b, 0x3f36,0x00c9, 0x3e69,0x0192
        DCW 0x3c5f,0x0388, 0x3ed0,0x012e, 0x3d9a,0x025b
        DCW 0x3b1e,0x04b5, 0x3e69,0x0192, 0x3cc8,0x0324
        DCW 0x39d9,0x05e2, 0x3e02,0x01f7, 0x3bf4,0x03ed
        DCW 0x388e,0x070e, 0x3d9a,0x025b, 0x3b1e,0x04b5
        DCW 0x373f,0x0839, 0x3d31,0x02c0, 0x3a46,0x057e
        DCW 0x35eb,0x0964, 0x3cc8,0x0324, 0x396b,0x0646
        DCW 0x3492,0x0a8e, 0x3c5f,0x0388, 0x388e,0x070e
        DCW 0x3334,0x0bb7, 0x3bf4,0x03ed, 0x37af,0x07d6
        DCW 0x31d2,0x0cdf, 0x3b8a,0x0451, 0x36ce,0x089d
        DCW 0x306c,0x0e06, 0x3b1e,0x04b5, 0x35eb,0x0964
        DCW 0x2f02,0x0f2b, 0x3ab2,0x051a, 0x3505,0x0a2b
        DCW 0x2d93,0x1050, 0x3a46,0x057e, 0x341e,0x0af1
        DCW 0x2c21,0x1173, 0x39d9,0x05e2, 0x3334,0x0bb7
        DCW 0x2aaa,0x1294, 0x396b,0x0646, 0x3249,0x0c7c
        DCW 0x2931,0x13b4, 0x38fd,0x06aa, 0x315b,0x0d41
        DCW 0x27b3,0x14d2, 0x388e,0x070e, 0x306c,0x0e06
        DCW 0x2632,0x15ee, 0x381f,0x0772, 0x2f7b,0x0eca
        DCW 0x24ae,0x1709, 0x37af,0x07d6, 0x2e88,0x0f8d
        DCW 0x2326,0x1821, 0x373f,0x0839, 0x2d93,0x1050
        DCW 0x219c,0x1937, 0x36ce,0x089d, 0x2c9d,0x1112
        DCW 0x200e,0x1a4b, 0x365d,0x0901, 0x2ba4,0x11d3
        DCW 0x1e7e,0x1b5d, 0x35eb,0x0964, 0x2aaa,0x1294
        DCW 0x1ceb,0x1c6c, 0x3578,0x09c7, 0x29af,0x1354
        DCW 0x1b56,0x1d79, 0x3505,0x0a2b, 0x28b2,0x1413
        DCW 0x19be,0x1e84, 0x3492,0x0a8e, 0x27b3,0x14d2
        DCW 0x1824,0x1f8c, 0x341e,0x0af1, 0x26b3,0x1590
        DCW 0x1688,0x2091, 0x33a9,0x0b54, 0x25b1,0x164c
        DCW 0x14ea,0x2193, 0x3334,0x0bb7, 0x24ae,0x1709
        DCW 0x134a,0x2292, 0x32bf,0x0c1a, 0x23a9,0x17c4
        DCW 0x11a8,0x238e, 0x3249,0x0c7c, 0x22a3,0x187e
        DCW 0x1005,0x2488, 0x31d2,0x0cdf, 0x219c,0x1937
        DCW 0x0e61,0x257e, 0x315b,0x0d41, 0x2093,0x19ef
        DCW 0x0cbb,0x2671, 0x30e4,0x0da4, 0x1f89,0x1aa7
        DCW 0x0b14,0x2760, 0x306c,0x0e06, 0x1e7e,0x1b5d
        DCW 0x096d,0x284c, 0x2ff4,0x0e68, 0x1d72,0x1c12
        DCW 0x07c4,0x2935, 0x2f7b,0x0eca, 0x1c64,0x1cc6
        DCW 0x061b,0x2a1a, 0x2f02,0x0f2b, 0x1b56,0x1d79
        DCW 0x0471,0x2afb, 0x2e88,0x0f8d, 0x1a46,0x1e2b
        DCW 0x02c7,0x2bd8, 0x2e0e,0x0fee, 0x1935,0x1edc
        DCW 0x011c,0x2cb2, 0x2d93,0x1050, 0x1824,0x1f8c
        DCW 0xff72,0x2d88, 0x2d18,0x10b1, 0x1711,0x203a
        DCW 0xfdc7,0x2e5a, 0x2c9d,0x1112, 0x15fe,0x20e7
        DCW 0xfc1d,0x2f28, 0x2c21,0x1173, 0x14ea,0x2193
        DCW 0xfa73,0x2ff2, 0x2ba4,0x11d3, 0x13d5,0x223d
        DCW 0xf8ca,0x30b8, 0x2b28,0x1234, 0x12bf,0x22e7
        DCW 0xf721,0x3179, 0x2aaa,0x1294, 0x11a8,0x238e
        DCW 0xf579,0x3236, 0x2a2d,0x12f4, 0x1091,0x2435
        DCW 0xf3d2,0x32ef, 0x29af,0x1354, 0x0f79,0x24da
        DCW 0xf22c,0x33a3, 0x2931,0x13b4, 0x0e61,0x257e
        DCW 0xf087,0x3453, 0x28b2,0x1413, 0x0d48,0x2620
        DCW 0xeee3,0x34ff, 0x2833,0x1473, 0x0c2e,0x26c1
        DCW 0xed41,0x35a5, 0x27b3,0x14d2, 0x0b14,0x2760
        DCW 0xeba1,0x3648, 0x2733,0x1531, 0x09fa,0x27fe
        DCW 0xea02,0x36e5, 0x26b3,0x1590, 0x08df,0x289a
        DCW 0xe865,0x377e, 0x2632,0x15ee, 0x07c4,0x2935
        DCW 0xe6cb,0x3812, 0x25b1,0x164c, 0x06a9,0x29ce
        DCW 0xe532,0x38a1, 0x252f,0x16ab, 0x058d,0x2a65
        DCW 0xe39c,0x392b, 0x24ae,0x1709, 0x0471,0x2afb
        DCW 0xe208,0x39b0, 0x242b,0x1766, 0x0355,0x2b8f
        DCW 0xe077,0x3a30, 0x23a9,0x17c4, 0x0239,0x2c21
        DCW 0xdee9,0x3aab, 0x2326,0x1821, 0x011c,0x2cb2
        DCW 0xdd5d,0x3b21, 0x22a3,0x187e, 0x0000,0x2d41
        DCW 0xdbd5,0x3b92, 0x221f,0x18db, 0xfee4,0x2dcf
        DCW 0xda4f,0x3bfd, 0x219c,0x1937, 0xfdc7,0x2e5a
        DCW 0xd8cd,0x3c64, 0x2117,0x1993, 0xfcab,0x2ee4
        DCW 0xd74e,0x3cc5, 0x2093,0x19ef, 0xfb8f,0x2f6c
        DCW 0xd5d3,0x3d21, 0x200e,0x1a4b, 0xfa73,0x2ff2
        DCW 0xd45c,0x3d78, 0x1f89,0x1aa7, 0xf957,0x3076
        DCW 0xd2e8,0x3dc9, 0x1f04,0x1b02, 0xf83c,0x30f9
        DCW 0xd178,0x3e15, 0x1e7e,0x1b5d, 0xf721,0x3179
        DCW 0xd00c,0x3e5c, 0x1df8,0x1bb8, 0xf606,0x31f8
        DCW 0xcea5,0x3e9d, 0x1d72,0x1c12, 0xf4ec,0x3274
        DCW 0xcd41,0x3ed8, 0x1ceb,0x1c6c, 0xf3d2,0x32ef
        DCW 0xcbe2,0x3f0f, 0x1c64,0x1cc6, 0xf2b8,0x3368
        DCW 0xca88,0x3f40, 0x1bdd,0x1d20, 0xf19f,0x33df
        DCW 0xc932,0x3f6b, 0x1b56,0x1d79, 0xf087,0x3453
        DCW 0xc7e1,0x3f91, 0x1ace,0x1dd3, 0xef6f,0x34c6
        DCW 0xc695,0x3fb1, 0x1a46,0x1e2b, 0xee58,0x3537
        DCW 0xc54e,0x3fcc, 0x19be,0x1e84, 0xed41,0x35a5
        DCW 0xc40c,0x3fe1, 0x1935,0x1edc, 0xec2b,0x3612
        DCW 0xc2cf,0x3ff1, 0x18ad,0x1f34, 0xeb16,0x367d
        DCW 0xc197,0x3ffb, 0x1824,0x1f8c, 0xea02,0x36e5
        DCW 0xc065,0x4000, 0x179b,0x1fe3, 0xe8ef,0x374b
        DCW 0xbf38,0x3fff, 0x1711,0x203a, 0xe7dc,0x37b0
        DCW 0xbe11,0x3ff8, 0x1688,0x2091, 0xe6cb,0x3812
        DCW 0xbcf0,0x3fec, 0x15fe,0x20e7, 0xe5ba,0x3871
        DCW 0xbbd4,0x3fdb, 0x1574,0x213d, 0xe4aa,0x38cf
        DCW 0xbabf,0x3fc4, 0x14ea,0x2193, 0xe39c,0x392b
        DCW 0xb9af,0x3fa7, 0x145f,0x21e8, 0xe28e,0x3984
        DCW 0xb8a6,0x3f85, 0x13d5,0x223d, 0xe182,0x39db
        DCW 0xb7a2,0x3f5d, 0x134a,0x2292, 0xe077,0x3a30
        DCW 0xb6a5,0x3f30, 0x12bf,0x22e7, 0xdf6d,0x3a82
        DCW 0xb5af,0x3efd, 0x1234,0x233b, 0xde64,0x3ad3
        DCW 0xb4be,0x3ec5, 0x11a8,0x238e, 0xdd5d,0x3b21
        DCW 0xb3d5,0x3e88, 0x111d,0x23e2, 0xdc57,0x3b6d
        DCW 0xb2f2,0x3e45, 0x1091,0x2435, 0xdb52,0x3bb6
        DCW 0xb215,0x3dfc, 0x1005,0x2488, 0xda4f,0x3bfd
        DCW 0xb140,0x3daf, 0x0f79,0x24da, 0xd94d,0x3c42
        DCW 0xb071,0x3d5b, 0x0eed,0x252c, 0xd84d,0x3c85
        DCW 0xafa9,0x3d03, 0x0e61,0x257e, 0xd74e,0x3cc5
        DCW 0xaee8,0x3ca5, 0x0dd4,0x25cf, 0xd651,0x3d03
        DCW 0xae2e,0x3c42, 0x0d48,0x2620, 0xd556,0x3d3f
        DCW 0xad7b,0x3bda, 0x0cbb,0x2671, 0xd45c,0x3d78
        DCW 0xacd0,0x3b6d, 0x0c2e,0x26c1, 0xd363,0x3daf
        DCW 0xac2b,0x3afa, 0x0ba1,0x2711, 0xd26d,0x3de3
        DCW 0xab8e,0x3a82, 0x0b14,0x2760, 0xd178,0x3e15
        DCW 0xaaf8,0x3a06, 0x0a87,0x27af, 0xd085,0x3e45
        DCW 0xaa6a,0x3984, 0x09fa,0x27fe, 0xcf94,0x3e72
        DCW 0xa9e3,0x38fd, 0x096d,0x284c, 0xcea5,0x3e9d
        DCW 0xa963,0x3871, 0x08df,0x289a, 0xcdb7,0x3ec5
        DCW 0xa8eb,0x37e1, 0x0852,0x28e7, 0xcccc,0x3eeb
        DCW 0xa87b,0x374b, 0x07c4,0x2935, 0xcbe2,0x3f0f
        DCW 0xa812,0x36b1, 0x0736,0x2981, 0xcafb,0x3f30
        DCW 0xa7b1,0x3612, 0x06a9,0x29ce, 0xca15,0x3f4f
        DCW 0xa757,0x356e, 0x061b,0x2a1a, 0xc932,0x3f6b
        DCW 0xa705,0x34c6, 0x058d,0x2a65, 0xc851,0x3f85
        DCW 0xa6bb,0x3419, 0x04ff,0x2ab0, 0xc772,0x3f9c
        DCW 0xa678,0x3368, 0x0471,0x2afb, 0xc695,0x3fb1
        DCW 0xa63e,0x32b2, 0x03e3,0x2b45, 0xc5ba,0x3fc4
        DCW 0xa60b,0x31f8, 0x0355,0x2b8f, 0xc4e2,0x3fd4
        DCW 0xa5e0,0x3139, 0x02c7,0x2bd8, 0xc40c,0x3fe1
        DCW 0xa5bc,0x3076, 0x0239,0x2c21, 0xc338,0x3fec
        DCW 0xa5a1,0x2faf, 0x01aa,0x2c6a, 0xc266,0x3ff5
        DCW 0xa58d,0x2ee4, 0x011c,0x2cb2, 0xc197,0x3ffb
        DCW 0xa581,0x2e15, 0x008e,0x2cfa, 0xc0ca,0x3fff
        DCW 0xa57e,0x2d41, 0x0000,0x2d41, 0xc000,0x4000
        DCW 0xa581,0x2c6a, 0xff72,0x2d88, 0xbf38,0x3fff
        DCW 0xa58d,0x2b8f, 0xfee4,0x2dcf, 0xbe73,0x3ffb
        DCW 0xa5a1,0x2ab0, 0xfe56,0x2e15, 0xbdb0,0x3ff5
        DCW 0xa5bc,0x29ce, 0xfdc7,0x2e5a, 0xbcf0,0x3fec
        DCW 0xa5e0,0x28e7, 0xfd39,0x2e9f, 0xbc32,0x3fe1
        DCW 0xa60b,0x27fe, 0xfcab,0x2ee4, 0xbb77,0x3fd4
        DCW 0xa63e,0x2711, 0xfc1d,0x2f28, 0xbabf,0x3fc4
        DCW 0xa678,0x2620, 0xfb8f,0x2f6c, 0xba09,0x3fb1
        DCW 0xa6bb,0x252c, 0xfb01,0x2faf, 0xb956,0x3f9c
        DCW 0xa705,0x2435, 0xfa73,0x2ff2, 0xb8a6,0x3f85
        DCW 0xa757,0x233b, 0xf9e5,0x3034, 0xb7f8,0x3f6b
        DCW 0xa7b1,0x223d, 0xf957,0x3076, 0xb74d,0x3f4f
        DCW 0xa812,0x213d, 0xf8ca,0x30b8, 0xb6a5,0x3f30
        DCW 0xa87b,0x203a, 0xf83c,0x30f9, 0xb600,0x3f0f
        DCW 0xa8eb,0x1f34, 0xf7ae,0x3139, 0xb55e,0x3eeb
        DCW 0xa963,0x1e2b, 0xf721,0x3179, 0xb4be,0x3ec5
        DCW 0xa9e3,0x1d20, 0xf693,0x31b9, 0xb422,0x3e9d
        DCW 0xaa6a,0x1c12, 0xf606,0x31f8, 0xb388,0x3e72
        DCW 0xaaf8,0x1b02, 0xf579,0x3236, 0xb2f2,0x3e45
        DCW 0xab8e,0x19ef, 0xf4ec,0x3274, 0xb25e,0x3e15
        DCW 0xac2b,0x18db, 0xf45f,0x32b2, 0xb1cd,0x3de3
        DCW 0xacd0,0x17c4, 0xf3d2,0x32ef, 0xb140,0x3daf
        DCW 0xad7b,0x16ab, 0xf345,0x332c, 0xb0b5,0x3d78
        DCW 0xae2e,0x1590, 0xf2b8,0x3368, 0xb02d,0x3d3f
        DCW 0xaee8,0x1473, 0xf22c,0x33a3, 0xafa9,0x3d03
        DCW 0xafa9,0x1354, 0xf19f,0x33df, 0xaf28,0x3cc5
        DCW 0xb071,0x1234, 0xf113,0x3419, 0xaea9,0x3c85
        DCW 0xb140,0x1112, 0xf087,0x3453, 0xae2e,0x3c42
        DCW 0xb215,0x0fee, 0xeffb,0x348d, 0xadb6,0x3bfd
        DCW 0xb2f2,0x0eca, 0xef6f,0x34c6, 0xad41,0x3bb6
        DCW 0xb3d5,0x0da4, 0xeee3,0x34ff, 0xacd0,0x3b6d
        DCW 0xb4be,0x0c7c, 0xee58,0x3537, 0xac61,0x3b21
        DCW 0xb5af,0x0b54, 0xedcc,0x356e, 0xabf6,0x3ad3
        DCW 0xb6a5,0x0a2b, 0xed41,0x35a5, 0xab8e,0x3a82
        DCW 0xb7a2,0x0901, 0xecb6,0x35dc, 0xab29,0x3a30
        DCW 0xb8a6,0x07d6, 0xec2b,0x3612, 0xaac8,0x39db
        DCW 0xb9af,0x06aa, 0xeba1,0x3648, 0xaa6a,0x3984
        DCW 0xbabf,0x057e, 0xeb16,0x367d, 0xaa0f,0x392b
        DCW 0xbbd4,0x0451, 0xea8c,0x36b1, 0xa9b7,0x38cf
        DCW 0xbcf0,0x0324, 0xea02,0x36e5, 0xa963,0x3871
        DCW 0xbe11,0x01f7, 0xe978,0x3718, 0xa912,0x3812
        DCW 0xbf38,0x00c9, 0xe8ef,0x374b, 0xa8c5,0x37b0
        DCW 0xc065,0xff9b, 0xe865,0x377e, 0xa87b,0x374b
        DCW 0xc197,0xfe6e, 0xe7dc,0x37b0, 0xa834,0x36e5
        DCW 0xc2cf,0xfd40, 0xe753,0x37e1, 0xa7f1,0x367d
        DCW 0xc40c,0xfc13, 0xe6cb,0x3812, 0xa7b1,0x3612
        DCW 0xc54e,0xfae6, 0xe642,0x3842, 0xa774,0x35a5
        DCW 0xc695,0xf9ba, 0xe5ba,0x3871, 0xa73b,0x3537
        DCW 0xc7e1,0xf88e, 0xe532,0x38a1, 0xa705,0x34c6
        DCW 0xc932,0xf763, 0xe4aa,0x38cf, 0xa6d3,0x3453
        DCW 0xca88,0xf639, 0xe423,0x38fd, 0xa6a4,0x33df
        DCW 0xcbe2,0xf50f, 0xe39c,0x392b, 0xa678,0x3368
        DCW 0xcd41,0xf3e6, 0xe315,0x3958, 0xa650,0x32ef
        DCW 0xcea5,0xf2bf, 0xe28e,0x3984, 0xa62c,0x3274
        DCW 0xd00c,0xf198, 0xe208,0x39b0, 0xa60b,0x31f8
        DCW 0xd178,0xf073, 0xe182,0x39db, 0xa5ed,0x3179
        DCW 0xd2e8,0xef4f, 0xe0fc,0x3a06, 0xa5d3,0x30f9
        DCW 0xd45c,0xee2d, 0xe077,0x3a30, 0xa5bc,0x3076
        DCW 0xd5d3,0xed0c, 0xdff2,0x3a59, 0xa5a9,0x2ff2
        DCW 0xd74e,0xebed, 0xdf6d,0x3a82, 0xa599,0x2f6c
        DCW 0xd8cd,0xeacf, 0xdee9,0x3aab, 0xa58d,0x2ee4
        DCW 0xda4f,0xe9b4, 0xde64,0x3ad3, 0xa585,0x2e5a
        DCW 0xdbd5,0xe89a, 0xdde1,0x3afa, 0xa57f,0x2dcf
        DCW 0xdd5d,0xe782, 0xdd5d,0x3b21, 0xa57e,0x2d41
        DCW 0xdee9,0xe66d, 0xdcda,0x3b47, 0xa57f,0x2cb2
        DCW 0xe077,0xe559, 0xdc57,0x3b6d, 0xa585,0x2c21
        DCW 0xe208,0xe448, 0xdbd5,0x3b92, 0xa58d,0x2b8f
        DCW 0xe39c,0xe33a, 0xdb52,0x3bb6, 0xa599,0x2afb
        DCW 0xe532,0xe22d, 0xdad1,0x3bda, 0xa5a9,0x2a65
        DCW 0xe6cb,0xe124, 0xda4f,0x3bfd, 0xa5bc,0x29ce
        DCW 0xe865,0xe01d, 0xd9ce,0x3c20, 0xa5d3,0x2935
        DCW 0xea02,0xdf19, 0xd94d,0x3c42, 0xa5ed,0x289a
        DCW 0xeba1,0xde18, 0xd8cd,0x3c64, 0xa60b,0x27fe
        DCW 0xed41,0xdd19, 0xd84d,0x3c85, 0xa62c,0x2760
        DCW 0xeee3,0xdc1e, 0xd7cd,0x3ca5, 0xa650,0x26c1
        DCW 0xf087,0xdb26, 0xd74e,0x3cc5, 0xa678,0x2620
        DCW 0xf22c,0xda31, 0xd6cf,0x3ce4, 0xa6a4,0x257e
        DCW 0xf3d2,0xd93f, 0xd651,0x3d03, 0xa6d3,0x24da
        DCW 0xf579,0xd851, 0xd5d3,0x3d21, 0xa705,0x2435
        DCW 0xf721,0xd766, 0xd556,0x3d3f, 0xa73b,0x238e
        DCW 0xf8ca,0xd67f, 0xd4d8,0x3d5b, 0xa774,0x22e7
        DCW 0xfa73,0xd59b, 0xd45c,0x3d78, 0xa7b1,0x223d
        DCW 0xfc1d,0xd4bb, 0xd3df,0x3d93, 0xa7f1,0x2193
        DCW 0xfdc7,0xd3df, 0xd363,0x3daf, 0xa834,0x20e7
        DCW 0xff72,0xd306, 0xd2e8,0x3dc9, 0xa87b,0x203a
        DCW 0x011c,0xd231, 0xd26d,0x3de3, 0xa8c5,0x1f8c
        DCW 0x02c7,0xd161, 0xd1f2,0x3dfc, 0xa912,0x1edc
        DCW 0x0471,0xd094, 0xd178,0x3e15, 0xa963,0x1e2b
        DCW 0x061b,0xcfcc, 0xd0fe,0x3e2d, 0xa9b7,0x1d79
        DCW 0x07c4,0xcf07, 0xd085,0x3e45, 0xaa0f,0x1cc6
        DCW 0x096d,0xce47, 0xd00c,0x3e5c, 0xaa6a,0x1c12
        DCW 0x0b14,0xcd8c, 0xcf94,0x3e72, 0xaac8,0x1b5d
        DCW 0x0cbb,0xccd4, 0xcf1c,0x3e88, 0xab29,0x1aa7
        DCW 0x0e61,0xcc21, 0xcea5,0x3e9d, 0xab8e,0x19ef
        DCW 0x1005,0xcb73, 0xce2e,0x3eb1, 0xabf6,0x1937
        DCW 0x11a8,0xcac9, 0xcdb7,0x3ec5, 0xac61,0x187e
        DCW 0x134a,0xca24, 0xcd41,0x3ed8, 0xacd0,0x17c4
        DCW 0x14ea,0xc983, 0xcccc,0x3eeb, 0xad41,0x1709
        DCW 0x1688,0xc8e8, 0xcc57,0x3efd, 0xadb6,0x164c
        DCW 0x1824,0xc850, 0xcbe2,0x3f0f, 0xae2e,0x1590
        DCW 0x19be,0xc7be, 0xcb6e,0x3f20, 0xaea9,0x14d2
        DCW 0x1b56,0xc731, 0xcafb,0x3f30, 0xaf28,0x1413
        DCW 0x1ceb,0xc6a8, 0xca88,0x3f40, 0xafa9,0x1354
        DCW 0x1e7e,0xc625, 0xca15,0x3f4f, 0xb02d,0x1294
        DCW 0x200e,0xc5a7, 0xc9a3,0x3f5d, 0xb0b5,0x11d3
        DCW 0x219c,0xc52d, 0xc932,0x3f6b, 0xb140,0x1112
        DCW 0x2326,0xc4b9, 0xc8c1,0x3f78, 0xb1cd,0x1050
        DCW 0x24ae,0xc44a, 0xc851,0x3f85, 0xb25e,0x0f8d
        DCW 0x2632,0xc3e0, 0xc7e1,0x3f91, 0xb2f2,0x0eca
        DCW 0x27b3,0xc37b, 0xc772,0x3f9c, 0xb388,0x0e06
        DCW 0x2931,0xc31c, 0xc703,0x3fa7, 0xb422,0x0d41
        DCW 0x2aaa,0xc2c1, 0xc695,0x3fb1, 0xb4be,0x0c7c
        DCW 0x2c21,0xc26d, 0xc627,0x3fbb, 0xb55e,0x0bb7
        DCW 0x2d93,0xc21d, 0xc5ba,0x3fc4, 0xb600,0x0af1
        DCW 0x2f02,0xc1d3, 0xc54e,0x3fcc, 0xb6a5,0x0a2b
        DCW 0x306c,0xc18e, 0xc4e2,0x3fd4, 0xb74d,0x0964
        DCW 0x31d2,0xc14f, 0xc476,0x3fdb, 0xb7f8,0x089d
        DCW 0x3334,0xc115, 0xc40c,0x3fe1, 0xb8a6,0x07d6
        DCW 0x3492,0xc0e0, 0xc3a1,0x3fe7, 0xb956,0x070e
        DCW 0x35eb,0xc0b1, 0xc338,0x3fec, 0xba09,0x0646
        DCW 0x373f,0xc088, 0xc2cf,0x3ff1, 0xbabf,0x057e
        DCW 0x388e,0xc064, 0xc266,0x3ff5, 0xbb77,0x04b5
        DCW 0x39d9,0xc045, 0xc1fe,0x3ff8, 0xbc32,0x03ed
        DCW 0x3b1e,0xc02c, 0xc197,0x3ffb, 0xbcf0,0x0324
        DCW 0x3c5f,0xc019, 0xc130,0x3ffd, 0xbdb0,0x025b
        DCW 0x3d9a,0xc00b, 0xc0ca,0x3fff, 0xbe73,0x0192
        DCW 0x3ed0,0xc003, 0xc065,0x4000, 0xbf38,0x00c9
        ; N=4096 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x4000,0x0000, 0x4000,0x0000, 0x4000,0x0000
        DCW 0x3fb4,0x004b, 0x3fe7,0x0019, 0x3fce,0x0032
        DCW 0x3f69,0x0097, 0x3fce,0x0032, 0x3f9b,0x0065
        DCW 0x3f1c,0x00e2, 0x3fb4,0x004b, 0x3f69,0x0097
        DCW 0x3ed0,0x012e, 0x3f9b,0x0065, 0x3f36,0x00c9
        DCW 0x3e83,0x0179, 0x3f82,0x007e, 0x3f03,0x00fb
        DCW 0x3e35,0x01c4, 0x3f69,0x0097, 0x3ed0,0x012e
        DCW 0x3de8,0x0210, 0x3f4f,0x00b0, 0x3e9c,0x0160
        DCW 0x3d9a,0x025b, 0x3f36,0x00c9, 0x3e69,0x0192
        DCW 0x3d4c,0x02a6, 0x3f1c,0x00e2, 0x3e35,0x01c4
        DCW 0x3cfd,0x02f2, 0x3f03,0x00fb, 0x3e02,0x01f7
        DCW 0x3cae,0x033d, 0x3ee9,0x0114, 0x3dce,0x0229
        DCW 0x3c5f,0x0388, 0x3ed0,0x012e, 0x3d9a,0x025b
        DCW 0x3c0f,0x03d4, 0x3eb6,0x0147, 0x3d66,0x028d
        DCW 0x3bbf,0x041f, 0x3e9c,0x0160, 0x3d31,0x02c0
        DCW 0x3b6f,0x046a, 0x3e83,0x0179, 0x3cfd,0x02f2
        DCW 0x3b1e,0x04b5, 0x3e69,0x0192, 0x3cc8,0x0324
        DCW 0x3acd,0x0500, 0x3e4f,0x01ab, 0x3c94,0x0356
        DCW 0x3a7c,0x054c, 0x3e35,0x01c4, 0x3c5f,0x0388
        DCW 0x3a2b,0x0597, 0x3e1c,0x01dd, 0x3c2a,0x03bb
        DCW 0x39d9,0x05e2, 0x3e02,0x01f7, 0x3bf4,0x03ed
        DCW 0x3987,0x062d, 0x3de8,0x0210, 0x3bbf,0x041f
        DCW 0x3934,0x0678, 0x3dce,0x0229, 0x3b8a,0x0451
        DCW 0x38e1,0x06c3, 0x3db4,0x0242, 0x3b54,0x0483
        DCW 0x388e,0x070e, 0x3d9a,0x025b, 0x3b1e,0x04b5
        DCW 0x383b,0x0759, 0x3d80,0x0274, 0x3ae8,0x04e7
        DCW 0x37e7,0x07a4, 0x3d66,0x028d, 0x3ab2,0x051a
        DCW 0x3793,0x07ef, 0x3d4c,0x02a6, 0x3a7c,0x054c
        DCW 0x373f,0x0839, 0x3d31,0x02c0, 0x3a46,0x057e
        DCW 0x36ea,0x0884, 0x3d17,0x02d9, 0x3a0f,0x05b0
        DCW 0x3695,0x08cf, 0x3cfd,0x02f2, 0x39d9,0x05e2
        DCW 0x3640,0x0919, 0x3ce3,0x030b, 0x39a2,0x0614
        DCW 0x35eb,0x0964, 0x3cc8,0x0324, 0x396b,0x0646
        DCW 0x3595,0x09af, 0x3cae,0x033d, 0x3934,0x0678
        DCW 0x353f,0x09f9, 0x3c94,0x0356, 0x38fd,0x06aa
        DCW 0x34e8,0x0a44, 0x3c79,0x036f, 0x38c6,0x06dc
        DCW 0x3492,0x0a8e, 0x3c5f,0x0388, 0x388e,0x070e
        DCW 0x343b,0x0ad8, 0x3c44,0x03a1, 0x3857,0x0740
        DCW 0x33e4,0x0b23, 0x3c2a,0x03bb, 0x381f,0x0772
        DCW 0x338c,0x0b6d, 0x3c0f,0x03d4, 0x37e7,0x07a4
        DCW 0x3334,0x0bb7, 0x3bf4,0x03ed, 0x37af,0x07d6
        DCW 0x32dc,0x0c01, 0x3bda,0x0406, 0x3777,0x0807
        DCW 0x3284,0x0c4b, 0x3bbf,0x041f, 0x373f,0x0839
        DCW 0x322b,0x0c95, 0x3ba4,0x0438, 0x3706,0x086b
        DCW 0x31d2,0x0cdf, 0x3b8a,0x0451, 0x36ce,0x089d
        DCW 0x3179,0x0d29, 0x3b6f,0x046a, 0x3695,0x08cf
        DCW 0x3120,0x0d72, 0x3b54,0x0483, 0x365d,0x0901
        DCW 0x30c6,0x0dbc, 0x3b39,0x049c, 0x3624,0x0932
        DCW 0x306c,0x0e06, 0x3b1e,0x04b5, 0x35eb,0x0964
        DCW 0x3012,0x0e4f, 0x3b03,0x04ce, 0x35b1,0x0996
        DCW 0x2fb7,0x0e99, 0x3ae8,0x04e7, 0x3578,0x09c7
        DCW 0x2f5d,0x0ee2, 0x3acd,0x0500, 0x353f,0x09f9
        DCW 0x2f02,0x0f2b, 0x3ab2,0x051a, 0x3505,0x0a2b
        DCW 0x2ea6,0x0f75, 0x3a97,0x0533, 0x34cc,0x0a5c
        DCW 0x2e4b,0x0fbe, 0x3a7c,0x054c, 0x3492,0x0a8e
        DCW 0x2def,0x1007, 0x3a61,0x0565, 0x3458,0x0ac0
        DCW 0x2d93,0x1050, 0x3a46,0x057e, 0x341e,0x0af1
        DCW 0x2d37,0x1099, 0x3a2b,0x0597, 0x33e4,0x0b23
        DCW 0x2cda,0x10e1, 0x3a0f,0x05b0, 0x33a9,0x0b54
        DCW 0x2c7e,0x112a, 0x39f4,0x05c9, 0x336f,0x0b85
        DCW 0x2c21,0x1173, 0x39d9,0x05e2, 0x3334,0x0bb7
        DCW 0x2bc4,0x11bb, 0x39bd,0x05fb, 0x32fa,0x0be8
        DCW 0x2b66,0x1204, 0x39a2,0x0614, 0x32bf,0x0c1a
        DCW 0x2b08,0x124c, 0x3987,0x062d, 0x3284,0x0c4b
        DCW 0x2aaa,0x1294, 0x396b,0x0646, 0x3249,0x0c7c
        DCW 0x2a4c,0x12dc, 0x3950,0x065f, 0x320e,0x0cae
        DCW 0x29ee,0x1324, 0x3934,0x0678, 0x31d2,0x0cdf
        DCW 0x298f,0x136c, 0x3919,0x0691, 0x3197,0x0d10
        DCW 0x2931,0x13b4, 0x38fd,0x06aa, 0x315b,0x0d41
        DCW 0x28d1,0x13fb, 0x38e1,0x06c3, 0x3120,0x0d72
        DCW 0x2872,0x1443, 0x38c6,0x06dc, 0x30e4,0x0da4
        DCW 0x2813,0x148b, 0x38aa,0x06f5, 0x30a8,0x0dd5
        DCW 0x27b3,0x14d2, 0x388e,0x070e, 0x306c,0x0e06
        DCW 0x2753,0x1519, 0x3873,0x0727, 0x3030,0x0e37
        DCW 0x26f3,0x1560, 0x3857,0x0740, 0x2ff4,0x0e68
        DCW 0x2693,0x15a7, 0x383b,0x0759, 0x2fb7,0x0e99
        DCW 0x2632,0x15ee, 0x381f,0x0772, 0x2f7b,0x0eca
        DCW 0x25d1,0x1635, 0x3803,0x078b, 0x2f3e,0x0efb
        DCW 0x2570,0x167c, 0x37e7,0x07a4, 0x2f02,0x0f2b
        DCW 0x250f,0x16c2, 0x37cb,0x07bd, 0x2ec5,0x0f5c
        DCW 0x24ae,0x1709, 0x37af,0x07d6, 0x2e88,0x0f8d
        DCW 0x244c,0x174f, 0x3793,0x07ef, 0x2e4b,0x0fbe
        DCW 0x23ea,0x1795, 0x3777,0x0807, 0x2e0e,0x0fee
        DCW 0x2388,0x17db, 0x375b,0x0820, 0x2dd1,0x101f
        DCW 0x2326,0x1821, 0x373f,0x0839, 0x2d93,0x1050
        DCW 0x22c4,0x1867, 0x3723,0x0852, 0x2d56,0x1080
        DCW 0x2261,0x18ac, 0x3706,0x086b, 0x2d18,0x10b1
        DCW 0x21ff,0x18f2, 0x36ea,0x0884, 0x2cda,0x10e1
        DCW 0x219c,0x1937, 0x36ce,0x089d, 0x2c9d,0x1112
        DCW 0x2138,0x197c, 0x36b2,0x08b6, 0x2c5f,0x1142
        DCW 0x20d5,0x19c1, 0x3695,0x08cf, 0x2c21,0x1173
        DCW 0x2072,0x1a06, 0x3679,0x08e8, 0x2be3,0x11a3
        DCW 0x200e,0x1a4b, 0x365d,0x0901, 0x2ba4,0x11d3
        DCW 0x1faa,0x1a90, 0x3640,0x0919, 0x2b66,0x1204
        DCW 0x1f46,0x1ad4, 0x3624,0x0932, 0x2b28,0x1234
        DCW 0x1ee2,0x1b19, 0x3607,0x094b, 0x2ae9,0x1264
        DCW 0x1e7e,0x1b5d, 0x35eb,0x0964, 0x2aaa,0x1294
        DCW 0x1e19,0x1ba1, 0x35ce,0x097d, 0x2a6c,0x12c4
        DCW 0x1db5,0x1be5, 0x35b1,0x0996, 0x2a2d,0x12f4
        DCW 0x1d50,0x1c29, 0x3595,0x09af, 0x29ee,0x1324
        DCW 0x1ceb,0x1c6c, 0x3578,0x09c7, 0x29af,0x1354
        DCW 0x1c86,0x1cb0, 0x355b,0x09e0, 0x2970,0x1384
        DCW 0x1c21,0x1cf3, 0x353f,0x09f9, 0x2931,0x13b4
        DCW 0x1bbb,0x1d36, 0x3522,0x0a12, 0x28f1,0x13e4
        DCW 0x1b56,0x1d79, 0x3505,0x0a2b, 0x28b2,0x1413
        DCW 0x1af0,0x1dbc, 0x34e8,0x0a44, 0x2872,0x1443
        DCW 0x1a8a,0x1dff, 0x34cc,0x0a5c, 0x2833,0x1473
        DCW 0x1a24,0x1e42, 0x34af,0x0a75, 0x27f3,0x14a2
        DCW 0x19be,0x1e84, 0x3492,0x0a8e, 0x27b3,0x14d2
        DCW 0x1958,0x1ec6, 0x3475,0x0aa7, 0x2773,0x1501
        DCW 0x18f1,0x1f08, 0x3458,0x0ac0, 0x2733,0x1531
        DCW 0x188b,0x1f4a, 0x343b,0x0ad8, 0x26f3,0x1560
        DCW 0x1824,0x1f8c, 0x341e,0x0af1, 0x26b3,0x1590
        DCW 0x17bd,0x1fcd, 0x3401,0x0b0a, 0x2672,0x15bf
        DCW 0x1756,0x200f, 0x33e4,0x0b23, 0x2632,0x15ee
        DCW 0x16ef,0x2050, 0x33c6,0x0b3b, 0x25f1,0x161d
        DCW 0x1688,0x2091, 0x33a9,0x0b54, 0x25b1,0x164c
        DCW 0x1620,0x20d1, 0x338c,0x0b6d, 0x2570,0x167c
        DCW 0x15b9,0x2112, 0x336f,0x0b85, 0x252f,0x16ab
        DCW 0x1551,0x2153, 0x3352,0x0b9e, 0x24ef,0x16da
        DCW 0x14ea,0x2193, 0x3334,0x0bb7, 0x24ae,0x1709
        DCW 0x1482,0x21d3, 0x3317,0x0bd0, 0x246d,0x1737
        DCW 0x141a,0x2213, 0x32fa,0x0be8, 0x242b,0x1766
        DCW 0x13b2,0x2253, 0x32dc,0x0c01, 0x23ea,0x1795
        DCW 0x134a,0x2292, 0x32bf,0x0c1a, 0x23a9,0x17c4
        DCW 0x12e2,0x22d2, 0x32a1,0x0c32, 0x2368,0x17f2
        DCW 0x1279,0x2311, 0x3284,0x0c4b, 0x2326,0x1821
        DCW 0x1211,0x2350, 0x3266,0x0c64, 0x22e5,0x184f
        DCW 0x11a8,0x238e, 0x3249,0x0c7c, 0x22a3,0x187e
        DCW 0x1140,0x23cd, 0x322b,0x0c95, 0x2261,0x18ac
        DCW 0x10d7,0x240b, 0x320e,0x0cae, 0x221f,0x18db
        DCW 0x106e,0x244a, 0x31f0,0x0cc6, 0x21de,0x1909
        DCW 0x1005,0x2488, 0x31d2,0x0cdf, 0x219c,0x1937
        DCW 0x0f9c,0x24c5, 0x31b5,0x0cf8, 0x215a,0x1965
        DCW 0x0f33,0x2503, 0x3197,0x0d10, 0x2117,0x1993
        DCW 0x0eca,0x2541, 0x3179,0x0d29, 0x20d5,0x19c1
        DCW 0x0e61,0x257e, 0x315b,0x0d41, 0x2093,0x19ef
        DCW 0x0df8,0x25bb, 0x313e,0x0d5a, 0x2051,0x1a1d
        DCW 0x0d8e,0x25f8, 0x3120,0x0d72, 0x200e,0x1a4b
        DCW 0x0d25,0x2634, 0x3102,0x0d8b, 0x1fcc,0x1a79
        DCW 0x0cbb,0x2671, 0x30e4,0x0da4, 0x1f89,0x1aa7
        DCW 0x0c52,0x26ad, 0x30c6,0x0dbc, 0x1f46,0x1ad4
        DCW 0x0be8,0x26e9, 0x30a8,0x0dd5, 0x1f04,0x1b02
        DCW 0x0b7e,0x2724, 0x308a,0x0ded, 0x1ec1,0x1b30
        DCW 0x0b14,0x2760, 0x306c,0x0e06, 0x1e7e,0x1b5d
        DCW 0x0aaa,0x279b, 0x304e,0x0e1e, 0x1e3b,0x1b8a
        DCW 0x0a41,0x27d6, 0x3030,0x0e37, 0x1df8,0x1bb8
        DCW 0x09d7,0x2811, 0x3012,0x0e4f, 0x1db5,0x1be5
        DCW 0x096d,0x284c, 0x2ff4,0x0e68, 0x1d72,0x1c12
        DCW 0x0902,0x2886, 0x2fd6,0x0e80, 0x1d2e,0x1c3f
        DCW 0x0898,0x28c1, 0x2fb7,0x0e99, 0x1ceb,0x1c6c
        DCW 0x082e,0x28fb, 0x2f99,0x0eb1, 0x1ca8,0x1c99
        DCW 0x07c4,0x2935, 0x2f7b,0x0eca, 0x1c64,0x1cc6
        DCW 0x075a,0x296e, 0x2f5d,0x0ee2, 0x1c21,0x1cf3
        DCW 0x06ef,0x29a7, 0x2f3e,0x0efb, 0x1bdd,0x1d20
        DCW 0x0685,0x29e1, 0x2f20,0x0f13, 0x1b99,0x1d4d
        DCW 0x061b,0x2a1a, 0x2f02,0x0f2b, 0x1b56,0x1d79
        DCW 0x05b0,0x2a52, 0x2ee3,0x0f44, 0x1b12,0x1da6
        DCW 0x0546,0x2a8b, 0x2ec5,0x0f5c, 0x1ace,0x1dd3
        DCW 0x04db,0x2ac3, 0x2ea6,0x0f75, 0x1a8a,0x1dff
        DCW 0x0471,0x2afb, 0x2e88,0x0f8d, 0x1a46,0x1e2b
        DCW 0x0406,0x2b33, 0x2e6a,0x0fa5, 0x1a02,0x1e58
        DCW 0x039c,0x2b6a, 0x2e4b,0x0fbe, 0x19be,0x1e84
        DCW 0x0331,0x2ba1, 0x2e2c,0x0fd6, 0x197a,0x1eb0
        DCW 0x02c7,0x2bd8, 0x2e0e,0x0fee, 0x1935,0x1edc
        DCW 0x025c,0x2c0f, 0x2def,0x1007, 0x18f1,0x1f08
        DCW 0x01f2,0x2c46, 0x2dd1,0x101f, 0x18ad,0x1f34
        DCW 0x0187,0x2c7c, 0x2db2,0x1037, 0x1868,0x1f60
        DCW 0x011c,0x2cb2, 0x2d93,0x1050, 0x1824,0x1f8c
        DCW 0x00b2,0x2ce8, 0x2d75,0x1068, 0x17df,0x1fb7
        DCW 0x0047,0x2d1e, 0x2d56,0x1080, 0x179b,0x1fe3
        DCW 0xffdc,0x2d53, 0x2d37,0x1099, 0x1756,0x200f
        DCW 0xff72,0x2d88, 0x2d18,0x10b1, 0x1711,0x203a
        DCW 0xff07,0x2dbd, 0x2cf9,0x10c9, 0x16cd,0x2065
        DCW 0xfe9d,0x2df2, 0x2cda,0x10e1, 0x1688,0x2091
        DCW 0xfe32,0x2e26, 0x2cbc,0x10fa, 0x1643,0x20bc
        DCW 0xfdc7,0x2e5a, 0x2c9d,0x1112, 0x15fe,0x20e7
        DCW 0xfd5d,0x2e8e, 0x2c7e,0x112a, 0x15b9,0x2112
        DCW 0xfcf2,0x2ec2, 0x2c5f,0x1142, 0x1574,0x213d
        DCW 0xfc88,0x2ef5, 0x2c40,0x115a, 0x152f,0x2168
        DCW 0xfc1d,0x2f28, 0x2c21,0x1173, 0x14ea,0x2193
        DCW 0xfbb3,0x2f5b, 0x2c02,0x118b, 0x14a5,0x21be
        DCW 0xfb48,0x2f8d, 0x2be3,0x11a3, 0x145f,0x21e8
        DCW 0xfade,0x2fc0, 0x2bc4,0x11bb, 0x141a,0x2213
        DCW 0xfa73,0x2ff2, 0x2ba4,0x11d3, 0x13d5,0x223d
        DCW 0xfa09,0x3024, 0x2b85,0x11eb, 0x138f,0x2268
        DCW 0xf99e,0x3055, 0x2b66,0x1204, 0x134a,0x2292
        DCW 0xf934,0x3087, 0x2b47,0x121c, 0x1304,0x22bc
        DCW 0xf8ca,0x30b8, 0x2b28,0x1234, 0x12bf,0x22e7
        DCW 0xf85f,0x30e8, 0x2b08,0x124c, 0x1279,0x2311
        DCW 0xf7f5,0x3119, 0x2ae9,0x1264, 0x1234,0x233b
        DCW 0xf78b,0x3149, 0x2aca,0x127c, 0x11ee,0x2365
        DCW 0xf721,0x3179, 0x2aaa,0x1294, 0x11a8,0x238e
        DCW 0xf6b7,0x31a9, 0x2a8b,0x12ac, 0x1163,0x23b8
        DCW 0xf64d,0x31d8, 0x2a6c,0x12c4, 0x111d,0x23e2
        DCW 0xf5e3,0x3207, 0x2a4c,0x12dc, 0x10d7,0x240b
        DCW 0xf579,0x3236, 0x2a2d,0x12f4, 0x1091,0x2435
        DCW 0xf50f,0x3265, 0x2a0d,0x130c, 0x104b,0x245e
        DCW 0xf4a5,0x3293, 0x29ee,0x1324, 0x1005,0x2488
        DCW 0xf43b,0x32c1, 0x29ce,0x133c, 0x0fbf,0x24b1
        DCW 0xf3d2,0x32ef, 0x29af,0x1354, 0x0f79,0x24da
        DCW 0xf368,0x331d, 0x298f,0x136c, 0x0f33,0x2503
        DCW 0xf2ff,0x334a, 0x2970,0x1384, 0x0eed,0x252c
        DCW 0xf295,0x3377, 0x2950,0x139c, 0x0ea7,0x2555
        DCW 0xf22c,0x33a3, 0x2931,0x13b4, 0x0e61,0x257e
        DCW 0xf1c2,0x33d0, 0x2911,0x13cc, 0x0e1b,0x25a6
        DCW 0xf159,0x33fc, 0x28f1,0x13e4, 0x0dd4,0x25cf
        DCW 0xf0f0,0x3428, 0x28d1,0x13fb, 0x0d8e,0x25f8
        DCW 0xf087,0x3453, 0x28b2,0x1413, 0x0d48,0x2620
        DCW 0xf01e,0x347f, 0x2892,0x142b, 0x0d01,0x2648
        DCW 0xefb5,0x34aa, 0x2872,0x1443, 0x0cbb,0x2671
        DCW 0xef4c,0x34d4, 0x2852,0x145b, 0x0c75,0x2699
        DCW 0xeee3,0x34ff, 0x2833,0x1473, 0x0c2e,0x26c1
        DCW 0xee7b,0x3529, 0x2813,0x148b, 0x0be8,0x26e9
        DCW 0xee12,0x3553, 0x27f3,0x14a2, 0x0ba1,0x2711
        DCW 0xeda9,0x357c, 0x27d3,0x14ba, 0x0b5b,0x2738
        DCW 0xed41,0x35a5, 0x27b3,0x14d2, 0x0b14,0x2760
        DCW 0xecd9,0x35ce, 0x2793,0x14ea, 0x0ace,0x2788
        DCW 0xec71,0x35f7, 0x2773,0x1501, 0x0a87,0x27af
        DCW 0xec09,0x3620, 0x2753,0x1519, 0x0a41,0x27d6
        DCW 0xeba1,0x3648, 0x2733,0x1531, 0x09fa,0x27fe
        DCW 0xeb39,0x366f, 0x2713,0x1549, 0x09b3,0x2825
        DCW 0xead1,0x3697, 0x26f3,0x1560, 0x096d,0x284c
        DCW 0xea6a,0x36be, 0x26d3,0x1578, 0x0926,0x2873
        DCW 0xea02,0x36e5, 0x26b3,0x1590, 0x08df,0x289a
        DCW 0xe99b,0x370c, 0x2693,0x15a7, 0x0898,0x28c1
        DCW 0xe933,0x3732, 0x2672,0x15bf, 0x0852,0x28e7
        DCW 0xe8cc,0x3758, 0x2652,0x15d7, 0x080b,0x290e
        DCW 0xe865,0x377e, 0x2632,0x15ee, 0x07c4,0x2935
        DCW 0xe7fe,0x37a3, 0x2612,0x1606, 0x077d,0x295b
        DCW 0xe798,0x37c8, 0x25f1,0x161d, 0x0736,0x2981
        DCW 0xe731,0x37ed, 0x25d1,0x1635, 0x06ef,0x29a7
        DCW 0xe6cb,0x3812, 0x25b1,0x164c, 0x06a9,0x29ce
        DCW 0xe664,0x3836, 0x2591,0x1664, 0x0662,0x29f4
        DCW 0xe5fe,0x385a, 0x2570,0x167c, 0x061b,0x2a1a
        DCW 0xe598,0x387d, 0x2550,0x1693, 0x05d4,0x2a3f
        DCW 0xe532,0x38a1, 0x252f,0x16ab, 0x058d,0x2a65
        DCW 0xe4cc,0x38c3, 0x250f,0x16c2, 0x0546,0x2a8b
        DCW 0xe467,0x38e6, 0x24ef,0x16da, 0x04ff,0x2ab0
        DCW 0xe401,0x3909, 0x24ce,0x16f1, 0x04b8,0x2ad6
        DCW 0xe39c,0x392b, 0x24ae,0x1709, 0x0471,0x2afb
        DCW 0xe337,0x394c, 0x248d,0x1720, 0x042a,0x2b20
        DCW 0xe2d2,0x396e, 0x246d,0x1737, 0x03e3,0x2b45
        DCW 0xe26d,0x398f, 0x244c,0x174f, 0x039c,0x2b6a
        DCW 0xe208,0x39b0, 0x242b,0x1766, 0x0355,0x2b8f
        DCW 0xe1a4,0x39d0, 0x240b,0x177e, 0x030e,0x2bb4
        DCW 0xe13f,0x39f0, 0x23ea,0x1795, 0x02c7,0x2bd8
        DCW 0xe0db,0x3a10, 0x23ca,0x17ac, 0x0280,0x2bfd
        DCW 0xe077,0x3a30, 0x23a9,0x17c4, 0x0239,0x2c21
        DCW 0xe013,0x3a4f, 0x2388,0x17db, 0x01f2,0x2c46
        DCW 0xdfaf,0x3a6e, 0x2368,0x17f2, 0x01aa,0x2c6a
        DCW 0xdf4c,0x3a8d, 0x2347,0x180a, 0x0163,0x2c8e
        DCW 0xdee9,0x3aab, 0x2326,0x1821, 0x011c,0x2cb2
        DCW 0xde85,0x3ac9, 0x2305,0x1838, 0x00d5,0x2cd6
        DCW 0xde22,0x3ae6, 0x22e5,0x184f, 0x008e,0x2cfa
        DCW 0xddc0,0x3b04, 0x22c4,0x1867, 0x0047,0x2d1e
        DCW 0xdd5d,0x3b21, 0x22a3,0x187e, 0x0000,0x2d41
        DCW 0xdcfb,0x3b3e, 0x2282,0x1895, 0xffb9,0x2d65
        DCW 0xdc98,0x3b5a, 0x2261,0x18ac, 0xff72,0x2d88
        DCW 0xdc36,0x3b76, 0x2240,0x18c3, 0xff2b,0x2dab
        DCW 0xdbd5,0x3b92, 0x221f,0x18db, 0xfee4,0x2dcf
        DCW 0xdb73,0x3bad, 0x21ff,0x18f2, 0xfe9d,0x2df2
        DCW 0xdb11,0x3bc8, 0x21de,0x1909, 0xfe56,0x2e15
        DCW 0xdab0,0x3be3, 0x21bd,0x1920, 0xfe0e,0x2e37
        DCW 0xda4f,0x3bfd, 0x219c,0x1937, 0xfdc7,0x2e5a
        DCW 0xd9ee,0x3c17, 0x217b,0x194e, 0xfd80,0x2e7d
        DCW 0xd98e,0x3c31, 0x215a,0x1965, 0xfd39,0x2e9f
        DCW 0xd92d,0x3c4b, 0x2138,0x197c, 0xfcf2,0x2ec2
        DCW 0xd8cd,0x3c64, 0x2117,0x1993, 0xfcab,0x2ee4
        DCW 0xd86d,0x3c7d, 0x20f6,0x19aa, 0xfc64,0x2f06
        DCW 0xd80d,0x3c95, 0x20d5,0x19c1, 0xfc1d,0x2f28
        DCW 0xd7ae,0x3cad, 0x20b4,0x19d8, 0xfbd6,0x2f4a
        DCW 0xd74e,0x3cc5, 0x2093,0x19ef, 0xfb8f,0x2f6c
        DCW 0xd6ef,0x3cdd, 0x2072,0x1a06, 0xfb48,0x2f8d
        DCW 0xd690,0x3cf4, 0x2051,0x1a1d, 0xfb01,0x2faf
        DCW 0xd632,0x3d0b, 0x202f,0x1a34, 0xfaba,0x2fd0
        DCW 0xd5d3,0x3d21, 0x200e,0x1a4b, 0xfa73,0x2ff2
        DCW 0xd575,0x3d37, 0x1fed,0x1a62, 0xfa2c,0x3013
        DCW 0xd517,0x3d4d, 0x1fcc,0x1a79, 0xf9e5,0x3034
        DCW 0xd4b9,0x3d63, 0x1faa,0x1a90, 0xf99e,0x3055
        DCW 0xd45c,0x3d78, 0x1f89,0x1aa7, 0xf957,0x3076
        DCW 0xd3fe,0x3d8d, 0x1f68,0x1abe, 0xf911,0x3097
        DCW 0xd3a1,0x3da1, 0x1f46,0x1ad4, 0xf8ca,0x30b8
        DCW 0xd344,0x3db5, 0x1f25,0x1aeb, 0xf883,0x30d8
        DCW 0xd2e8,0x3dc9, 0x1f04,0x1b02, 0xf83c,0x30f9
        DCW 0xd28b,0x3ddd, 0x1ee2,0x1b19, 0xf7f5,0x3119
        DCW 0xd22f,0x3df0, 0x1ec1,0x1b30, 0xf7ae,0x3139
        DCW 0xd1d4,0x3e03, 0x1e9f,0x1b46, 0xf768,0x3159
        DCW 0xd178,0x3e15, 0x1e7e,0x1b5d, 0xf721,0x3179
        DCW 0xd11d,0x3e27, 0x1e5c,0x1b74, 0xf6da,0x3199
        DCW 0xd0c2,0x3e39, 0x1e3b,0x1b8a, 0xf693,0x31b9
        DCW 0xd067,0x3e4a, 0x1e19,0x1ba1, 0xf64d,0x31d8
        DCW 0xd00c,0x3e5c, 0x1df8,0x1bb8, 0xf606,0x31f8
        DCW 0xcfb2,0x3e6c, 0x1dd6,0x1bce, 0xf5bf,0x3217
        DCW 0xcf58,0x3e7d, 0x1db5,0x1be5, 0xf579,0x3236
        DCW 0xcefe,0x3e8d, 0x1d93,0x1bfc, 0xf532,0x3255
        DCW 0xcea5,0x3e9d, 0x1d72,0x1c12, 0xf4ec,0x3274
        DCW 0xce4b,0x3eac, 0x1d50,0x1c29, 0xf4a5,0x3293
        DCW 0xcdf2,0x3ebb, 0x1d2e,0x1c3f, 0xf45f,0x32b2
        DCW 0xcd9a,0x3eca, 0x1d0d,0x1c56, 0xf418,0x32d0
        DCW 0xcd41,0x3ed8, 0x1ceb,0x1c6c, 0xf3d2,0x32ef
        DCW 0xcce9,0x3ee7, 0x1cc9,0x1c83, 0xf38b,0x330d
        DCW 0xcc91,0x3ef4, 0x1ca8,0x1c99, 0xf345,0x332c
        DCW 0xcc3a,0x3f02, 0x1c86,0x1cb0, 0xf2ff,0x334a
        DCW 0xcbe2,0x3f0f, 0x1c64,0x1cc6, 0xf2b8,0x3368
        DCW 0xcb8b,0x3f1c, 0x1c42,0x1cdd, 0xf272,0x3386
        DCW 0xcb34,0x3f28, 0x1c21,0x1cf3, 0xf22c,0x33a3
        DCW 0xcade,0x3f34, 0x1bff,0x1d0a, 0xf1e5,0x33c1
        DCW 0xca88,0x3f40, 0x1bdd,0x1d20, 0xf19f,0x33df
        DCW 0xca32,0x3f4b, 0x1bbb,0x1d36, 0xf159,0x33fc
        DCW 0xc9dc,0x3f56, 0x1b99,0x1d4d, 0xf113,0x3419
        DCW 0xc987,0x3f61, 0x1b77,0x1d63, 0xf0cd,0x3436
        DCW 0xc932,0x3f6b, 0x1b56,0x1d79, 0xf087,0x3453
        DCW 0xc8dd,0x3f75, 0x1b34,0x1d90, 0xf041,0x3470
        DCW 0xc889,0x3f7f, 0x1b12,0x1da6, 0xeffb,0x348d
        DCW 0xc835,0x3f88, 0x1af0,0x1dbc, 0xefb5,0x34aa
        DCW 0xc7e1,0x3f91, 0x1ace,0x1dd3, 0xef6f,0x34c6
        DCW 0xc78d,0x3f99, 0x1aac,0x1de9, 0xef29,0x34e2
        DCW 0xc73a,0x3fa2, 0x1a8a,0x1dff, 0xeee3,0x34ff
        DCW 0xc6e7,0x3faa, 0x1a68,0x1e15, 0xee9d,0x351b
        DCW 0xc695,0x3fb1, 0x1a46,0x1e2b, 0xee58,0x3537
        DCW 0xc643,0x3fb8, 0x1a24,0x1e42, 0xee12,0x3553
        DCW 0xc5f1,0x3fbf, 0x1a02,0x1e58, 0xedcc,0x356e
        DCW 0xc59f,0x3fc6, 0x19e0,0x1e6e, 0xed87,0x358a
        DCW 0xc54e,0x3fcc, 0x19be,0x1e84, 0xed41,0x35a5
        DCW 0xc4fd,0x3fd2, 0x199c,0x1e9a, 0xecfc,0x35c1
        DCW 0xc4ac,0x3fd7, 0x197a,0x1eb0, 0xecb6,0x35dc
        DCW 0xc45c,0x3fdc, 0x1958,0x1ec6, 0xec71,0x35f7
        DCW 0xc40c,0x3fe1, 0x1935,0x1edc, 0xec2b,0x3612
        DCW 0xc3bc,0x3fe6, 0x1913,0x1ef2, 0xebe6,0x362d
        DCW 0xc36c,0x3fea, 0x18f1,0x1f08, 0xeba1,0x3648
        DCW 0xc31d,0x3fed, 0x18cf,0x1f1e, 0xeb5b,0x3662
        DCW 0xc2cf,0x3ff1, 0x18ad,0x1f34, 0xeb16,0x367d
        DCW 0xc280,0x3ff4, 0x188b,0x1f4a, 0xead1,0x3697
        DCW 0xc232,0x3ff7, 0x1868,0x1f60, 0xea8c,0x36b1
        DCW 0xc1e4,0x3ff9, 0x1846,0x1f76, 0xea47,0x36cb
        DCW 0xc197,0x3ffb, 0x1824,0x1f8c, 0xea02,0x36e5
        DCW 0xc14a,0x3ffd, 0x1802,0x1fa2, 0xe9bd,0x36ff
        DCW 0xc0fd,0x3ffe, 0x17df,0x1fb7, 0xe978,0x3718
        DCW 0xc0b1,0x3fff, 0x17bd,0x1fcd, 0xe933,0x3732
        DCW 0xc065,0x4000, 0x179b,0x1fe3, 0xe8ef,0x374b
        DCW 0xc019,0x4000, 0x1778,0x1ff9, 0xe8aa,0x3765
        DCW 0xbfce,0x4000, 0x1756,0x200f, 0xe865,0x377e
        DCW 0xbf83,0x4000, 0x1734,0x2024, 0xe821,0x3797
        DCW 0xbf38,0x3fff, 0x1711,0x203a, 0xe7dc,0x37b0
        DCW 0xbeee,0x3ffe, 0x16ef,0x2050, 0xe798,0x37c8
        DCW 0xbea4,0x3ffc, 0x16cd,0x2065, 0xe753,0x37e1
        DCW 0xbe5a,0x3ffa, 0x16aa,0x207b, 0xe70f,0x37f9
        DCW 0xbe11,0x3ff8, 0x1688,0x2091, 0xe6cb,0x3812
        DCW 0xbdc8,0x3ff6, 0x1665,0x20a6, 0xe686,0x382a
        DCW 0xbd80,0x3ff3, 0x1643,0x20bc, 0xe642,0x3842
        DCW 0xbd38,0x3ff0, 0x1620,0x20d1, 0xe5fe,0x385a
        DCW 0xbcf0,0x3fec, 0x15fe,0x20e7, 0xe5ba,0x3871
        DCW 0xbca8,0x3fe8, 0x15db,0x20fd, 0xe576,0x3889
        DCW 0xbc61,0x3fe4, 0x15b9,0x2112, 0xe532,0x38a1
        DCW 0xbc1b,0x3fe0, 0x1596,0x2128, 0xe4ee,0x38b8
        DCW 0xbbd4,0x3fdb, 0x1574,0x213d, 0xe4aa,0x38cf
        DCW 0xbb8e,0x3fd5, 0x1551,0x2153, 0xe467,0x38e6
        DCW 0xbb49,0x3fd0, 0x152f,0x2168, 0xe423,0x38fd
        DCW 0xbb04,0x3fca, 0x150c,0x217d, 0xe3df,0x3914
        DCW 0xbabf,0x3fc4, 0x14ea,0x2193, 0xe39c,0x392b
        DCW 0xba7a,0x3fbd, 0x14c7,0x21a8, 0xe358,0x3941
        DCW 0xba36,0x3fb6, 0x14a5,0x21be, 0xe315,0x3958
        DCW 0xb9f2,0x3faf, 0x1482,0x21d3, 0xe2d2,0x396e
        DCW 0xb9af,0x3fa7, 0x145f,0x21e8, 0xe28e,0x3984
        DCW 0xb96c,0x3f9f, 0x143d,0x21fe, 0xe24b,0x399a
        DCW 0xb92a,0x3f97, 0x141a,0x2213, 0xe208,0x39b0
        DCW 0xb8e7,0x3f8e, 0x13f7,0x2228, 0xe1c5,0x39c5
        DCW 0xb8a6,0x3f85, 0x13d5,0x223d, 0xe182,0x39db
        DCW 0xb864,0x3f7b, 0x13b2,0x2253, 0xe13f,0x39f0
        DCW 0xb823,0x3f72, 0x138f,0x2268, 0xe0fc,0x3a06
        DCW 0xb7e3,0x3f68, 0x136d,0x227d, 0xe0ba,0x3a1b
        DCW 0xb7a2,0x3f5d, 0x134a,0x2292, 0xe077,0x3a30
        DCW 0xb762,0x3f52, 0x1327,0x22a7, 0xe034,0x3a45
        DCW 0xb723,0x3f47, 0x1304,0x22bc, 0xdff2,0x3a59
        DCW 0xb6e4,0x3f3c, 0x12e2,0x22d2, 0xdfaf,0x3a6e
        DCW 0xb6a5,0x3f30, 0x12bf,0x22e7, 0xdf6d,0x3a82
        DCW 0xb667,0x3f24, 0x129c,0x22fc, 0xdf2b,0x3a97
        DCW 0xb629,0x3f17, 0x1279,0x2311, 0xdee9,0x3aab
        DCW 0xb5ec,0x3f0a, 0x1257,0x2326, 0xdea6,0x3abf
        DCW 0xb5af,0x3efd, 0x1234,0x233b, 0xde64,0x3ad3
        DCW 0xb572,0x3ef0, 0x1211,0x2350, 0xde22,0x3ae6
        DCW 0xb536,0x3ee2, 0x11ee,0x2365, 0xdde1,0x3afa
        DCW 0xb4fa,0x3ed4, 0x11cb,0x237a, 0xdd9f,0x3b0e
        DCW 0xb4be,0x3ec5, 0x11a8,0x238e, 0xdd5d,0x3b21
        DCW 0xb483,0x3eb6, 0x1185,0x23a3, 0xdd1b,0x3b34
        DCW 0xb449,0x3ea7, 0x1163,0x23b8, 0xdcda,0x3b47
        DCW 0xb40f,0x3e98, 0x1140,0x23cd, 0xdc98,0x3b5a
        DCW 0xb3d5,0x3e88, 0x111d,0x23e2, 0xdc57,0x3b6d
        DCW 0xb39b,0x3e77, 0x10fa,0x23f7, 0xdc16,0x3b7f
        DCW 0xb362,0x3e67, 0x10d7,0x240b, 0xdbd5,0x3b92
        DCW 0xb32a,0x3e56, 0x10b4,0x2420, 0xdb93,0x3ba4
        DCW 0xb2f2,0x3e45, 0x1091,0x2435, 0xdb52,0x3bb6
        DCW 0xb2ba,0x3e33, 0x106e,0x244a, 0xdb11,0x3bc8
        DCW 0xb283,0x3e21, 0x104b,0x245e, 0xdad1,0x3bda
        DCW 0xb24c,0x3e0f, 0x1028,0x2473, 0xda90,0x3bec
        DCW 0xb215,0x3dfc, 0x1005,0x2488, 0xda4f,0x3bfd
        DCW 0xb1df,0x3de9, 0x0fe2,0x249c, 0xda0f,0x3c0f
        DCW 0xb1aa,0x3dd6, 0x0fbf,0x24b1, 0xd9ce,0x3c20
        DCW 0xb174,0x3dc2, 0x0f9c,0x24c5, 0xd98e,0x3c31
        DCW 0xb140,0x3daf, 0x0f79,0x24da, 0xd94d,0x3c42
        DCW 0xb10b,0x3d9a, 0x0f56,0x24ef, 0xd90d,0x3c53
        DCW 0xb0d7,0x3d86, 0x0f33,0x2503, 0xd8cd,0x3c64
        DCW 0xb0a4,0x3d71, 0x0f10,0x2518, 0xd88d,0x3c74
        DCW 0xb071,0x3d5b, 0x0eed,0x252c, 0xd84d,0x3c85
        DCW 0xb03e,0x3d46, 0x0eca,0x2541, 0xd80d,0x3c95
        DCW 0xb00c,0x3d30, 0x0ea7,0x2555, 0xd7cd,0x3ca5
        DCW 0xafda,0x3d1a, 0x0e84,0x2569, 0xd78e,0x3cb5
        DCW 0xafa9,0x3d03, 0x0e61,0x257e, 0xd74e,0x3cc5
        DCW 0xaf78,0x3cec, 0x0e3e,0x2592, 0xd70f,0x3cd5
        DCW 0xaf48,0x3cd5, 0x0e1b,0x25a6, 0xd6cf,0x3ce4
        DCW 0xaf18,0x3cbd, 0x0df8,0x25bb, 0xd690,0x3cf4
        DCW 0xaee8,0x3ca5, 0x0dd4,0x25cf, 0xd651,0x3d03
        DCW 0xaeb9,0x3c8d, 0x0db1,0x25e3, 0xd612,0x3d12
        DCW 0xae8a,0x3c74, 0x0d8e,0x25f8, 0xd5d3,0x3d21
        DCW 0xae5c,0x3c5b, 0x0d6b,0x260c, 0xd594,0x3d30
        DCW 0xae2e,0x3c42, 0x0d48,0x2620, 0xd556,0x3d3f
        DCW 0xae01,0x3c29, 0x0d25,0x2634, 0xd517,0x3d4d
        DCW 0xadd4,0x3c0f, 0x0d01,0x2648, 0xd4d8,0x3d5b
        DCW 0xada7,0x3bf5, 0x0cde,0x265c, 0xd49a,0x3d6a
        DCW 0xad7b,0x3bda, 0x0cbb,0x2671, 0xd45c,0x3d78
        DCW 0xad50,0x3bbf, 0x0c98,0x2685, 0xd41d,0x3d86
        DCW 0xad25,0x3ba4, 0x0c75,0x2699, 0xd3df,0x3d93
        DCW 0xacfa,0x3b88, 0x0c52,0x26ad, 0xd3a1,0x3da1
        DCW 0xacd0,0x3b6d, 0x0c2e,0x26c1, 0xd363,0x3daf
        DCW 0xaca6,0x3b50, 0x0c0b,0x26d5, 0xd326,0x3dbc
        DCW 0xac7d,0x3b34, 0x0be8,0x26e9, 0xd2e8,0x3dc9
        DCW 0xac54,0x3b17, 0x0bc5,0x26fd, 0xd2aa,0x3dd6
        DCW 0xac2b,0x3afa, 0x0ba1,0x2711, 0xd26d,0x3de3
        DCW 0xac03,0x3add, 0x0b7e,0x2724, 0xd22f,0x3df0
        DCW 0xabdc,0x3abf, 0x0b5b,0x2738, 0xd1f2,0x3dfc
        DCW 0xabb5,0x3aa1, 0x0b38,0x274c, 0xd1b5,0x3e09
        DCW 0xab8e,0x3a82, 0x0b14,0x2760, 0xd178,0x3e15
        DCW 0xab68,0x3a64, 0x0af1,0x2774, 0xd13b,0x3e21
        DCW 0xab42,0x3a45, 0x0ace,0x2788, 0xd0fe,0x3e2d
        DCW 0xab1d,0x3a25, 0x0aaa,0x279b, 0xd0c2,0x3e39
        DCW 0xaaf8,0x3a06, 0x0a87,0x27af, 0xd085,0x3e45
        DCW 0xaad4,0x39e6, 0x0a64,0x27c3, 0xd049,0x3e50
        DCW 0xaab0,0x39c5, 0x0a41,0x27d6, 0xd00c,0x3e5c
        DCW 0xaa8d,0x39a5, 0x0a1d,0x27ea, 0xcfd0,0x3e67
        DCW 0xaa6a,0x3984, 0x09fa,0x27fe, 0xcf94,0x3e72
        DCW 0xaa47,0x3963, 0x09d7,0x2811, 0xcf58,0x3e7d
        DCW 0xaa25,0x3941, 0x09b3,0x2825, 0xcf1c,0x3e88
        DCW 0xaa04,0x391f, 0x0990,0x2838, 0xcee0,0x3e92
        DCW 0xa9e3,0x38fd, 0x096d,0x284c, 0xcea5,0x3e9d
        DCW 0xa9c2,0x38db, 0x0949,0x2860, 0xce69,0x3ea7
        DCW 0xa9a2,0x38b8, 0x0926,0x2873, 0xce2e,0x3eb1
        DCW 0xa982,0x3895, 0x0902,0x2886, 0xcdf2,0x3ebb
        DCW 0xa963,0x3871, 0x08df,0x289a, 0xcdb7,0x3ec5
        DCW 0xa945,0x384e, 0x08bc,0x28ad, 0xcd7c,0x3ecf
        DCW 0xa926,0x382a, 0x0898,0x28c1, 0xcd41,0x3ed8
        DCW 0xa908,0x3805, 0x0875,0x28d4, 0xcd06,0x3ee2
        DCW 0xa8eb,0x37e1, 0x0852,0x28e7, 0xcccc,0x3eeb
        DCW 0xa8ce,0x37bc, 0x082e,0x28fb, 0xcc91,0x3ef4
        DCW 0xa8b2,0x3797, 0x080b,0x290e, 0xcc57,0x3efd
        DCW 0xa896,0x3771, 0x07e7,0x2921, 0xcc1c,0x3f06
        DCW 0xa87b,0x374b, 0x07c4,0x2935, 0xcbe2,0x3f0f
        DCW 0xa860,0x3725, 0x07a1,0x2948, 0xcba8,0x3f17
        DCW 0xa845,0x36ff, 0x077d,0x295b, 0xcb6e,0x3f20
        DCW 0xa82b,0x36d8, 0x075a,0x296e, 0xcb34,0x3f28
        DCW 0xa812,0x36b1, 0x0736,0x2981, 0xcafb,0x3f30
        DCW 0xa7f9,0x368a, 0x0713,0x2994, 0xcac1,0x3f38
        DCW 0xa7e0,0x3662, 0x06ef,0x29a7, 0xca88,0x3f40
        DCW 0xa7c8,0x363a, 0x06cc,0x29bb, 0xca4f,0x3f47
        DCW 0xa7b1,0x3612, 0x06a9,0x29ce, 0xca15,0x3f4f
        DCW 0xa799,0x35ea, 0x0685,0x29e1, 0xc9dc,0x3f56
        DCW 0xa783,0x35c1, 0x0662,0x29f4, 0xc9a3,0x3f5d
        DCW 0xa76d,0x3598, 0x063e,0x2a07, 0xc96b,0x3f64
        DCW 0xa757,0x356e, 0x061b,0x2a1a, 0xc932,0x3f6b
        DCW 0xa742,0x3545, 0x05f7,0x2a2c, 0xc8fa,0x3f72
        DCW 0xa72d,0x351b, 0x05d4,0x2a3f, 0xc8c1,0x3f78
        DCW 0xa719,0x34f1, 0x05b0,0x2a52, 0xc889,0x3f7f
        DCW 0xa705,0x34c6, 0x058d,0x2a65, 0xc851,0x3f85
        DCW 0xa6f2,0x349b, 0x0569,0x2a78, 0xc819,0x3f8b
        DCW 0xa6df,0x3470, 0x0546,0x2a8b, 0xc7e1,0x3f91
        DCW 0xa6cd,0x3445, 0x0522,0x2a9d, 0xc7a9,0x3f97
        DCW 0xa6bb,0x3419, 0x04ff,0x2ab0, 0xc772,0x3f9c
        DCW 0xa6a9,0x33ed, 0x04db,0x2ac3, 0xc73a,0x3fa2
        DCW 0xa699,0x33c1, 0x04b8,0x2ad6, 0xc703,0x3fa7
        DCW 0xa688,0x3395, 0x0494,0x2ae8, 0xc6cc,0x3fac
        DCW 0xa678,0x3368, 0x0471,0x2afb, 0xc695,0x3fb1
        DCW 0xa669,0x333b, 0x044d,0x2b0d, 0xc65e,0x3fb6
        DCW 0xa65a,0x330d, 0x042a,0x2b20, 0xc627,0x3fbb
        DCW 0xa64c,0x32e0, 0x0406,0x2b33, 0xc5f1,0x3fbf
        DCW 0xa63e,0x32b2, 0x03e3,0x2b45, 0xc5ba,0x3fc4
        DCW 0xa630,0x3284, 0x03bf,0x2b58, 0xc584,0x3fc8
        DCW 0xa623,0x3255, 0x039c,0x2b6a, 0xc54e,0x3fcc
        DCW 0xa617,0x3227, 0x0378,0x2b7d, 0xc518,0x3fd0
        DCW 0xa60b,0x31f8, 0x0355,0x2b8f, 0xc4e2,0x3fd4
        DCW 0xa5ff,0x31c8, 0x0331,0x2ba1, 0xc4ac,0x3fd7
        DCW 0xa5f4,0x3199, 0x030e,0x2bb4, 0xc476,0x3fdb
        DCW 0xa5ea,0x3169, 0x02ea,0x2bc6, 0xc441,0x3fde
        DCW 0xa5e0,0x3139, 0x02c7,0x2bd8, 0xc40c,0x3fe1
        DCW 0xa5d6,0x3109, 0x02a3,0x2beb, 0xc3d6,0x3fe4
        DCW 0xa5cd,0x30d8, 0x0280,0x2bfd, 0xc3a1,0x3fe7
        DCW 0xa5c4,0x30a7, 0x025c,0x2c0f, 0xc36c,0x3fea
        DCW 0xa5bc,0x3076, 0x0239,0x2c21, 0xc338,0x3fec
        DCW 0xa5b5,0x3045, 0x0215,0x2c34, 0xc303,0x3fef
        DCW 0xa5ae,0x3013, 0x01f2,0x2c46, 0xc2cf,0x3ff1
        DCW 0xa5a7,0x2fe1, 0x01ce,0x2c58, 0xc29a,0x3ff3
        DCW 0xa5a1,0x2faf, 0x01aa,0x2c6a, 0xc266,0x3ff5
        DCW 0xa59b,0x2f7d, 0x0187,0x2c7c, 0xc232,0x3ff7
        DCW 0xa596,0x2f4a, 0x0163,0x2c8e, 0xc1fe,0x3ff8
        DCW 0xa591,0x2f17, 0x0140,0x2ca0, 0xc1cb,0x3ffa
        DCW 0xa58d,0x2ee4, 0x011c,0x2cb2, 0xc197,0x3ffb
        DCW 0xa58a,0x2eb0, 0x00f9,0x2cc4, 0xc164,0x3ffc
        DCW 0xa586,0x2e7d, 0x00d5,0x2cd6, 0xc130,0x3ffd
        DCW 0xa584,0x2e49, 0x00b2,0x2ce8, 0xc0fd,0x3ffe
        DCW 0xa581,0x2e15, 0x008e,0x2cfa, 0xc0ca,0x3fff
        DCW 0xa580,0x2de0, 0x006b,0x2d0c, 0xc097,0x3fff
        DCW 0xa57f,0x2dab, 0x0047,0x2d1e, 0xc065,0x4000
        DCW 0xa57e,0x2d76, 0x0024,0x2d2f, 0xc032,0x4000
        DCW 0xa57e,0x2d41, 0x0000,0x2d41, 0xc000,0x4000
        DCW 0xa57e,0x2d0c, 0xffdc,0x2d53, 0xbfce,0x4000
        DCW 0xa57f,0x2cd6, 0xffb9,0x2d65, 0xbf9c,0x4000
        DCW 0xa580,0x2ca0, 0xff95,0x2d76, 0xbf6a,0x3fff
        DCW 0xa581,0x2c6a, 0xff72,0x2d88, 0xbf38,0x3fff
        DCW 0xa584,0x2c34, 0xff4e,0x2d9a, 0xbf07,0x3ffe
        DCW 0xa586,0x2bfd, 0xff2b,0x2dab, 0xbed5,0x3ffd
        DCW 0xa58a,0x2bc6, 0xff07,0x2dbd, 0xbea4,0x3ffc
        DCW 0xa58d,0x2b8f, 0xfee4,0x2dcf, 0xbe73,0x3ffb
        DCW 0xa591,0x2b58, 0xfec0,0x2de0, 0xbe42,0x3ffa
        DCW 0xa596,0x2b20, 0xfe9d,0x2df2, 0xbe11,0x3ff8
        DCW 0xa59b,0x2ae8, 0xfe79,0x2e03, 0xbde1,0x3ff7
        DCW 0xa5a1,0x2ab0, 0xfe56,0x2e15, 0xbdb0,0x3ff5
        DCW 0xa5a7,0x2a78, 0xfe32,0x2e26, 0xbd80,0x3ff3
        DCW 0xa5ae,0x2a3f, 0xfe0e,0x2e37, 0xbd50,0x3ff1
        DCW 0xa5b5,0x2a07, 0xfdeb,0x2e49, 0xbd20,0x3fef
        DCW 0xa5bc,0x29ce, 0xfdc7,0x2e5a, 0xbcf0,0x3fec
        DCW 0xa5c4,0x2994, 0xfda4,0x2e6b, 0xbcc0,0x3fea
        DCW 0xa5cd,0x295b, 0xfd80,0x2e7d, 0xbc91,0x3fe7
        DCW 0xa5d6,0x2921, 0xfd5d,0x2e8e, 0xbc61,0x3fe4
        DCW 0xa5e0,0x28e7, 0xfd39,0x2e9f, 0xbc32,0x3fe1
        DCW 0xa5ea,0x28ad, 0xfd16,0x2eb0, 0xbc03,0x3fde
        DCW 0xa5f4,0x2873, 0xfcf2,0x2ec2, 0xbbd4,0x3fdb
        DCW 0xa5ff,0x2838, 0xfccf,0x2ed3, 0xbba6,0x3fd7
        DCW 0xa60b,0x27fe, 0xfcab,0x2ee4, 0xbb77,0x3fd4
        DCW 0xa617,0x27c3, 0xfc88,0x2ef5, 0xbb49,0x3fd0
        DCW 0xa623,0x2788, 0xfc64,0x2f06, 0xbb1b,0x3fcc
        DCW 0xa630,0x274c, 0xfc41,0x2f17, 0xbaed,0x3fc8
        DCW 0xa63e,0x2711, 0xfc1d,0x2f28, 0xbabf,0x3fc4
        DCW 0xa64c,0x26d5, 0xfbfa,0x2f39, 0xba91,0x3fbf
        DCW 0xa65a,0x2699, 0xfbd6,0x2f4a, 0xba64,0x3fbb
        DCW 0xa669,0x265c, 0xfbb3,0x2f5b, 0xba36,0x3fb6
        DCW 0xa678,0x2620, 0xfb8f,0x2f6c, 0xba09,0x3fb1
        DCW 0xa688,0x25e3, 0xfb6c,0x2f7d, 0xb9dc,0x3fac
        DCW 0xa699,0x25a6, 0xfb48,0x2f8d, 0xb9af,0x3fa7
        DCW 0xa6a9,0x2569, 0xfb25,0x2f9e, 0xb982,0x3fa2
        DCW 0xa6bb,0x252c, 0xfb01,0x2faf, 0xb956,0x3f9c
        DCW 0xa6cd,0x24ef, 0xfade,0x2fc0, 0xb92a,0x3f97
        DCW 0xa6df,0x24b1, 0xfaba,0x2fd0, 0xb8fd,0x3f91
        DCW 0xa6f2,0x2473, 0xfa97,0x2fe1, 0xb8d1,0x3f8b
        DCW 0xa705,0x2435, 0xfa73,0x2ff2, 0xb8a6,0x3f85
        DCW 0xa719,0x23f7, 0xfa50,0x3002, 0xb87a,0x3f7f
        DCW 0xa72d,0x23b8, 0xfa2c,0x3013, 0xb84f,0x3f78
        DCW 0xa742,0x237a, 0xfa09,0x3024, 0xb823,0x3f72
        DCW 0xa757,0x233b, 0xf9e5,0x3034, 0xb7f8,0x3f6b
        DCW 0xa76d,0x22fc, 0xf9c2,0x3045, 0xb7cd,0x3f64
        DCW 0xa783,0x22bc, 0xf99e,0x3055, 0xb7a2,0x3f5d
        DCW 0xa799,0x227d, 0xf97b,0x3066, 0xb778,0x3f56
        DCW 0xa7b1,0x223d, 0xf957,0x3076, 0xb74d,0x3f4f
        DCW 0xa7c8,0x21fe, 0xf934,0x3087, 0xb723,0x3f47
        DCW 0xa7e0,0x21be, 0xf911,0x3097, 0xb6f9,0x3f40
        DCW 0xa7f9,0x217d, 0xf8ed,0x30a7, 0xb6cf,0x3f38
        DCW 0xa812,0x213d, 0xf8ca,0x30b8, 0xb6a5,0x3f30
        DCW 0xa82b,0x20fd, 0xf8a6,0x30c8, 0xb67c,0x3f28
        DCW 0xa845,0x20bc, 0xf883,0x30d8, 0xb652,0x3f20
        DCW 0xa860,0x207b, 0xf85f,0x30e8, 0xb629,0x3f17
        DCW 0xa87b,0x203a, 0xf83c,0x30f9, 0xb600,0x3f0f
        DCW 0xa896,0x1ff9, 0xf819,0x3109, 0xb5d7,0x3f06
        DCW 0xa8b2,0x1fb7, 0xf7f5,0x3119, 0xb5af,0x3efd
        DCW 0xa8ce,0x1f76, 0xf7d2,0x3129, 0xb586,0x3ef4
        DCW 0xa8eb,0x1f34, 0xf7ae,0x3139, 0xb55e,0x3eeb
        DCW 0xa908,0x1ef2, 0xf78b,0x3149, 0xb536,0x3ee2
        DCW 0xa926,0x1eb0, 0xf768,0x3159, 0xb50e,0x3ed8
        DCW 0xa945,0x1e6e, 0xf744,0x3169, 0xb4e6,0x3ecf
        DCW 0xa963,0x1e2b, 0xf721,0x3179, 0xb4be,0x3ec5
        DCW 0xa982,0x1de9, 0xf6fe,0x3189, 0xb497,0x3ebb
        DCW 0xa9a2,0x1da6, 0xf6da,0x3199, 0xb470,0x3eb1
        DCW 0xa9c2,0x1d63, 0xf6b7,0x31a9, 0xb449,0x3ea7
        DCW 0xa9e3,0x1d20, 0xf693,0x31b9, 0xb422,0x3e9d
        DCW 0xaa04,0x1cdd, 0xf670,0x31c8, 0xb3fb,0x3e92
        DCW 0xaa25,0x1c99, 0xf64d,0x31d8, 0xb3d5,0x3e88
        DCW 0xaa47,0x1c56, 0xf629,0x31e8, 0xb3ae,0x3e7d
        DCW 0xaa6a,0x1c12, 0xf606,0x31f8, 0xb388,0x3e72
        DCW 0xaa8d,0x1bce, 0xf5e3,0x3207, 0xb362,0x3e67
        DCW 0xaab0,0x1b8a, 0xf5bf,0x3217, 0xb33d,0x3e5c
        DCW 0xaad4,0x1b46, 0xf59c,0x3227, 0xb317,0x3e50
        DCW 0xaaf8,0x1b02, 0xf579,0x3236, 0xb2f2,0x3e45
        DCW 0xab1d,0x1abe, 0xf556,0x3246, 0xb2cc,0x3e39
        DCW 0xab42,0x1a79, 0xf532,0x3255, 0xb2a7,0x3e2d
        DCW 0xab68,0x1a34, 0xf50f,0x3265, 0xb283,0x3e21
        DCW 0xab8e,0x19ef, 0xf4ec,0x3274, 0xb25e,0x3e15
        DCW 0xabb5,0x19aa, 0xf4c8,0x3284, 0xb23a,0x3e09
        DCW 0xabdc,0x1965, 0xf4a5,0x3293, 0xb215,0x3dfc
        DCW 0xac03,0x1920, 0xf482,0x32a3, 0xb1f1,0x3df0
        DCW 0xac2b,0x18db, 0xf45f,0x32b2, 0xb1cd,0x3de3
        DCW 0xac54,0x1895, 0xf43b,0x32c1, 0xb1aa,0x3dd6
        DCW 0xac7d,0x184f, 0xf418,0x32d0, 0xb186,0x3dc9
        DCW 0xaca6,0x180a, 0xf3f5,0x32e0, 0xb163,0x3dbc
        DCW 0xacd0,0x17c4, 0xf3d2,0x32ef, 0xb140,0x3daf
        DCW 0xacfa,0x177e, 0xf3ae,0x32fe, 0xb11d,0x3da1
        DCW 0xad25,0x1737, 0xf38b,0x330d, 0xb0fa,0x3d93
        DCW 0xad50,0x16f1, 0xf368,0x331d, 0xb0d7,0x3d86
        DCW 0xad7b,0x16ab, 0xf345,0x332c, 0xb0b5,0x3d78
        DCW 0xada7,0x1664, 0xf322,0x333b, 0xb093,0x3d6a
        DCW 0xadd4,0x161d, 0xf2ff,0x334a, 0xb071,0x3d5b
        DCW 0xae01,0x15d7, 0xf2db,0x3359, 0xb04f,0x3d4d
        DCW 0xae2e,0x1590, 0xf2b8,0x3368, 0xb02d,0x3d3f
        DCW 0xae5c,0x1549, 0xf295,0x3377, 0xb00c,0x3d30
        DCW 0xae8a,0x1501, 0xf272,0x3386, 0xafeb,0x3d21
        DCW 0xaeb9,0x14ba, 0xf24f,0x3395, 0xafca,0x3d12
        DCW 0xaee8,0x1473, 0xf22c,0x33a3, 0xafa9,0x3d03
        DCW 0xaf18,0x142b, 0xf208,0x33b2, 0xaf88,0x3cf4
        DCW 0xaf48,0x13e4, 0xf1e5,0x33c1, 0xaf68,0x3ce4
        DCW 0xaf78,0x139c, 0xf1c2,0x33d0, 0xaf48,0x3cd5
        DCW 0xafa9,0x1354, 0xf19f,0x33df, 0xaf28,0x3cc5
        DCW 0xafda,0x130c, 0xf17c,0x33ed, 0xaf08,0x3cb5
        DCW 0xb00c,0x12c4, 0xf159,0x33fc, 0xaee8,0x3ca5
        DCW 0xb03e,0x127c, 0xf136,0x340b, 0xaec9,0x3c95
        DCW 0xb071,0x1234, 0xf113,0x3419, 0xaea9,0x3c85
        DCW 0xb0a4,0x11eb, 0xf0f0,0x3428, 0xae8a,0x3c74
        DCW 0xb0d7,0x11a3, 0xf0cd,0x3436, 0xae6b,0x3c64
        DCW 0xb10b,0x115a, 0xf0aa,0x3445, 0xae4d,0x3c53
        DCW 0xb140,0x1112, 0xf087,0x3453, 0xae2e,0x3c42
        DCW 0xb174,0x10c9, 0xf064,0x3462, 0xae10,0x3c31
        DCW 0xb1aa,0x1080, 0xf041,0x3470, 0xadf2,0x3c20
        DCW 0xb1df,0x1037, 0xf01e,0x347f, 0xadd4,0x3c0f
        DCW 0xb215,0x0fee, 0xeffb,0x348d, 0xadb6,0x3bfd
        DCW 0xb24c,0x0fa5, 0xefd8,0x349b, 0xad99,0x3bec
        DCW 0xb283,0x0f5c, 0xefb5,0x34aa, 0xad7b,0x3bda
        DCW 0xb2ba,0x0f13, 0xef92,0x34b8, 0xad5e,0x3bc8
        DCW 0xb2f2,0x0eca, 0xef6f,0x34c6, 0xad41,0x3bb6
        DCW 0xb32a,0x0e80, 0xef4c,0x34d4, 0xad25,0x3ba4
        DCW 0xb362,0x0e37, 0xef29,0x34e2, 0xad08,0x3b92
        DCW 0xb39b,0x0ded, 0xef06,0x34f1, 0xacec,0x3b7f
        DCW 0xb3d5,0x0da4, 0xeee3,0x34ff, 0xacd0,0x3b6d
        DCW 0xb40f,0x0d5a, 0xeec0,0x350d, 0xacb4,0x3b5a
        DCW 0xb449,0x0d10, 0xee9d,0x351b, 0xac98,0x3b47
        DCW 0xb483,0x0cc6, 0xee7b,0x3529, 0xac7d,0x3b34
        DCW 0xb4be,0x0c7c, 0xee58,0x3537, 0xac61,0x3b21
        DCW 0xb4fa,0x0c32, 0xee35,0x3545, 0xac46,0x3b0e
        DCW 0xb536,0x0be8, 0xee12,0x3553, 0xac2b,0x3afa
        DCW 0xb572,0x0b9e, 0xedef,0x3561, 0xac11,0x3ae6
        DCW 0xb5af,0x0b54, 0xedcc,0x356e, 0xabf6,0x3ad3
        DCW 0xb5ec,0x0b0a, 0xeda9,0x357c, 0xabdc,0x3abf
        DCW 0xb629,0x0ac0, 0xed87,0x358a, 0xabc2,0x3aab
        DCW 0xb667,0x0a75, 0xed64,0x3598, 0xaba8,0x3a97
        DCW 0xb6a5,0x0a2b, 0xed41,0x35a5, 0xab8e,0x3a82
        DCW 0xb6e4,0x09e0, 0xed1e,0x35b3, 0xab75,0x3a6e
        DCW 0xb723,0x0996, 0xecfc,0x35c1, 0xab5b,0x3a59
        DCW 0xb762,0x094b, 0xecd9,0x35ce, 0xab42,0x3a45
        DCW 0xb7a2,0x0901, 0xecb6,0x35dc, 0xab29,0x3a30
        DCW 0xb7e3,0x08b6, 0xec93,0x35ea, 0xab11,0x3a1b
        DCW 0xb823,0x086b, 0xec71,0x35f7, 0xaaf8,0x3a06
        DCW 0xb864,0x0820, 0xec4e,0x3605, 0xaae0,0x39f0
        DCW 0xb8a6,0x07d6, 0xec2b,0x3612, 0xaac8,0x39db
        DCW 0xb8e7,0x078b, 0xec09,0x3620, 0xaab0,0x39c5
        DCW 0xb92a,0x0740, 0xebe6,0x362d, 0xaa98,0x39b0
        DCW 0xb96c,0x06f5, 0xebc3,0x363a, 0xaa81,0x399a
        DCW 0xb9af,0x06aa, 0xeba1,0x3648, 0xaa6a,0x3984
        DCW 0xb9f2,0x065f, 0xeb7e,0x3655, 0xaa53,0x396e
        DCW 0xba36,0x0614, 0xeb5b,0x3662, 0xaa3c,0x3958
        DCW 0xba7a,0x05c9, 0xeb39,0x366f, 0xaa25,0x3941
        DCW 0xbabf,0x057e, 0xeb16,0x367d, 0xaa0f,0x392b
        DCW 0xbb04,0x0533, 0xeaf4,0x368a, 0xa9f9,0x3914
        DCW 0xbb49,0x04e7, 0xead1,0x3697, 0xa9e3,0x38fd
        DCW 0xbb8e,0x049c, 0xeaaf,0x36a4, 0xa9cd,0x38e6
        DCW 0xbbd4,0x0451, 0xea8c,0x36b1, 0xa9b7,0x38cf
        DCW 0xbc1b,0x0406, 0xea6a,0x36be, 0xa9a2,0x38b8
        DCW 0xbc61,0x03bb, 0xea47,0x36cb, 0xa98d,0x38a1
        DCW 0xbca8,0x036f, 0xea25,0x36d8, 0xa978,0x3889
        DCW 0xbcf0,0x0324, 0xea02,0x36e5, 0xa963,0x3871
        DCW 0xbd38,0x02d9, 0xe9e0,0x36f2, 0xa94f,0x385a
        DCW 0xbd80,0x028d, 0xe9bd,0x36ff, 0xa93a,0x3842
        DCW 0xbdc8,0x0242, 0xe99b,0x370c, 0xa926,0x382a
        DCW 0xbe11,0x01f7, 0xe978,0x3718, 0xa912,0x3812
        DCW 0xbe5a,0x01ab, 0xe956,0x3725, 0xa8ff,0x37f9
        DCW 0xbea4,0x0160, 0xe933,0x3732, 0xa8eb,0x37e1
        DCW 0xbeee,0x0114, 0xe911,0x373f, 0xa8d8,0x37c8
        DCW 0xbf38,0x00c9, 0xe8ef,0x374b, 0xa8c5,0x37b0
        DCW 0xbf83,0x007e, 0xe8cc,0x3758, 0xa8b2,0x3797
        DCW 0xbfce,0x0032, 0xe8aa,0x3765, 0xa89f,0x377e
        DCW 0xc019,0xffe7, 0xe888,0x3771, 0xa88d,0x3765
        DCW 0xc065,0xff9b, 0xe865,0x377e, 0xa87b,0x374b
        DCW 0xc0b1,0xff50, 0xe843,0x378a, 0xa869,0x3732
        DCW 0xc0fd,0xff05, 0xe821,0x3797, 0xa857,0x3718
        DCW 0xc14a,0xfeb9, 0xe7fe,0x37a3, 0xa845,0x36ff
        DCW 0xc197,0xfe6e, 0xe7dc,0x37b0, 0xa834,0x36e5
        DCW 0xc1e4,0xfe23, 0xe7ba,0x37bc, 0xa823,0x36cb
        DCW 0xc232,0xfdd7, 0xe798,0x37c8, 0xa812,0x36b1
        DCW 0xc280,0xfd8c, 0xe775,0x37d5, 0xa801,0x3697
        DCW 0xc2cf,0xfd40, 0xe753,0x37e1, 0xa7f1,0x367d
        DCW 0xc31d,0xfcf5, 0xe731,0x37ed, 0xa7e0,0x3662
        DCW 0xc36c,0xfcaa, 0xe70f,0x37f9, 0xa7d0,0x3648
        DCW 0xc3bc,0xfc5f, 0xe6ed,0x3805, 0xa7c0,0x362d
        DCW 0xc40c,0xfc13, 0xe6cb,0x3812, 0xa7b1,0x3612
        DCW 0xc45c,0xfbc8, 0xe6a8,0x381e, 0xa7a1,0x35f7
        DCW 0xc4ac,0xfb7d, 0xe686,0x382a, 0xa792,0x35dc
        DCW 0xc4fd,0xfb32, 0xe664,0x3836, 0xa783,0x35c1
        DCW 0xc54e,0xfae6, 0xe642,0x3842, 0xa774,0x35a5
        DCW 0xc59f,0xfa9b, 0xe620,0x384e, 0xa765,0x358a
        DCW 0xc5f1,0xfa50, 0xe5fe,0x385a, 0xa757,0x356e
        DCW 0xc643,0xfa05, 0xe5dc,0x3866, 0xa749,0x3553
        DCW 0xc695,0xf9ba, 0xe5ba,0x3871, 0xa73b,0x3537
        DCW 0xc6e7,0xf96f, 0xe598,0x387d, 0xa72d,0x351b
        DCW 0xc73a,0xf924, 0xe576,0x3889, 0xa71f,0x34ff
        DCW 0xc78d,0xf8d9, 0xe554,0x3895, 0xa712,0x34e2
        DCW 0xc7e1,0xf88e, 0xe532,0x38a1, 0xa705,0x34c6
        DCW 0xc835,0xf843, 0xe510,0x38ac, 0xa6f8,0x34aa
        DCW 0xc889,0xf7f9, 0xe4ee,0x38b8, 0xa6eb,0x348d
        DCW 0xc8dd,0xf7ae, 0xe4cc,0x38c3, 0xa6df,0x3470
        DCW 0xc932,0xf763, 0xe4aa,0x38cf, 0xa6d3,0x3453
        DCW 0xc987,0xf718, 0xe489,0x38db, 0xa6c7,0x3436
        DCW 0xc9dc,0xf6ce, 0xe467,0x38e6, 0xa6bb,0x3419
        DCW 0xca32,0xf683, 0xe445,0x38f2, 0xa6af,0x33fc
        DCW 0xca88,0xf639, 0xe423,0x38fd, 0xa6a4,0x33df
        DCW 0xcade,0xf5ee, 0xe401,0x3909, 0xa699,0x33c1
        DCW 0xcb34,0xf5a4, 0xe3df,0x3914, 0xa68e,0x33a3
        DCW 0xcb8b,0xf559, 0xe3be,0x391f, 0xa683,0x3386
        DCW 0xcbe2,0xf50f, 0xe39c,0x392b, 0xa678,0x3368
        DCW 0xcc3a,0xf4c5, 0xe37a,0x3936, 0xa66e,0x334a
        DCW 0xcc91,0xf47b, 0xe358,0x3941, 0xa664,0x332c
        DCW 0xcce9,0xf430, 0xe337,0x394c, 0xa65a,0x330d
        DCW 0xcd41,0xf3e6, 0xe315,0x3958, 0xa650,0x32ef
        DCW 0xcd9a,0xf39c, 0xe2f3,0x3963, 0xa647,0x32d0
        DCW 0xcdf2,0xf352, 0xe2d2,0x396e, 0xa63e,0x32b2
        DCW 0xce4b,0xf308, 0xe2b0,0x3979, 0xa635,0x3293
        DCW 0xcea5,0xf2bf, 0xe28e,0x3984, 0xa62c,0x3274
        DCW 0xcefe,0xf275, 0xe26d,0x398f, 0xa623,0x3255
        DCW 0xcf58,0xf22b, 0xe24b,0x399a, 0xa61b,0x3236
        DCW 0xcfb2,0xf1e2, 0xe22a,0x39a5, 0xa613,0x3217
        DCW 0xd00c,0xf198, 0xe208,0x39b0, 0xa60b,0x31f8
        DCW 0xd067,0xf14f, 0xe1e7,0x39bb, 0xa603,0x31d8
        DCW 0xd0c2,0xf105, 0xe1c5,0x39c5, 0xa5fb,0x31b9
        DCW 0xd11d,0xf0bc, 0xe1a4,0x39d0, 0xa5f4,0x3199
        DCW 0xd178,0xf073, 0xe182,0x39db, 0xa5ed,0x3179
        DCW 0xd1d4,0xf02a, 0xe161,0x39e6, 0xa5e6,0x3159
        DCW 0xd22f,0xefe1, 0xe13f,0x39f0, 0xa5e0,0x3139
        DCW 0xd28b,0xef98, 0xe11e,0x39fb, 0xa5d9,0x3119
        DCW 0xd2e8,0xef4f, 0xe0fc,0x3a06, 0xa5d3,0x30f9
        DCW 0xd344,0xef06, 0xe0db,0x3a10, 0xa5cd,0x30d8
        DCW 0xd3a1,0xeebe, 0xe0ba,0x3a1b, 0xa5c7,0x30b8
        DCW 0xd3fe,0xee75, 0xe098,0x3a25, 0xa5c2,0x3097
        DCW 0xd45c,0xee2d, 0xe077,0x3a30, 0xa5bc,0x3076
        DCW 0xd4b9,0xede4, 0xe056,0x3a3a, 0xa5b7,0x3055
        DCW 0xd517,0xed9c, 0xe034,0x3a45, 0xa5b2,0x3034
        DCW 0xd575,0xed54, 0xe013,0x3a4f, 0xa5ae,0x3013
        DCW 0xd5d3,0xed0c, 0xdff2,0x3a59, 0xa5a9,0x2ff2
        DCW 0xd632,0xecc4, 0xdfd1,0x3a64, 0xa5a5,0x2fd0
        DCW 0xd690,0xec7c, 0xdfaf,0x3a6e, 0xa5a1,0x2faf
        DCW 0xd6ef,0xec34, 0xdf8e,0x3a78, 0xa59d,0x2f8d
        DCW 0xd74e,0xebed, 0xdf6d,0x3a82, 0xa599,0x2f6c
        DCW 0xd7ae,0xeba5, 0xdf4c,0x3a8d, 0xa596,0x2f4a
        DCW 0xd80d,0xeb5e, 0xdf2b,0x3a97, 0xa593,0x2f28
        DCW 0xd86d,0xeb16, 0xdf0a,0x3aa1, 0xa590,0x2f06
        DCW 0xd8cd,0xeacf, 0xdee9,0x3aab, 0xa58d,0x2ee4
        DCW 0xd92d,0xea88, 0xdec8,0x3ab5, 0xa58b,0x2ec2
        DCW 0xd98e,0xea41, 0xdea6,0x3abf, 0xa588,0x2e9f
        DCW 0xd9ee,0xe9fa, 0xde85,0x3ac9, 0xa586,0x2e7d
        DCW 0xda4f,0xe9b4, 0xde64,0x3ad3, 0xa585,0x2e5a
        DCW 0xdab0,0xe96d, 0xde43,0x3add, 0xa583,0x2e37
        DCW 0xdb11,0xe926, 0xde22,0x3ae6, 0xa581,0x2e15
        DCW 0xdb73,0xe8e0, 0xde01,0x3af0, 0xa580,0x2df2
        DCW 0xdbd5,0xe89a, 0xdde1,0x3afa, 0xa57f,0x2dcf
        DCW 0xdc36,0xe854, 0xddc0,0x3b04, 0xa57f,0x2dab
        DCW 0xdc98,0xe80e, 0xdd9f,0x3b0e, 0xa57e,0x2d88
        DCW 0xdcfb,0xe7c8, 0xdd7e,0x3b17, 0xa57e,0x2d65
        DCW 0xdd5d,0xe782, 0xdd5d,0x3b21, 0xa57e,0x2d41
        DCW 0xddc0,0xe73d, 0xdd3c,0x3b2a, 0xa57e,0x2d1e
        DCW 0xde22,0xe6f7, 0xdd1b,0x3b34, 0xa57e,0x2cfa
        DCW 0xde85,0xe6b2, 0xdcfb,0x3b3e, 0xa57f,0x2cd6
        DCW 0xdee9,0xe66d, 0xdcda,0x3b47, 0xa57f,0x2cb2
        DCW 0xdf4c,0xe628, 0xdcb9,0x3b50, 0xa580,0x2c8e
        DCW 0xdfaf,0xe5e3, 0xdc98,0x3b5a, 0xa581,0x2c6a
        DCW 0xe013,0xe59e, 0xdc78,0x3b63, 0xa583,0x2c46
        DCW 0xe077,0xe559, 0xdc57,0x3b6d, 0xa585,0x2c21
        DCW 0xe0db,0xe515, 0xdc36,0x3b76, 0xa586,0x2bfd
        DCW 0xe13f,0xe4d0, 0xdc16,0x3b7f, 0xa588,0x2bd8
        DCW 0xe1a4,0xe48c, 0xdbf5,0x3b88, 0xa58b,0x2bb4
        DCW 0xe208,0xe448, 0xdbd5,0x3b92, 0xa58d,0x2b8f
        DCW 0xe26d,0xe404, 0xdbb4,0x3b9b, 0xa590,0x2b6a
        DCW 0xe2d2,0xe3c1, 0xdb93,0x3ba4, 0xa593,0x2b45
        DCW 0xe337,0xe37d, 0xdb73,0x3bad, 0xa596,0x2b20
        DCW 0xe39c,0xe33a, 0xdb52,0x3bb6, 0xa599,0x2afb
        DCW 0xe401,0xe2f6, 0xdb32,0x3bbf, 0xa59d,0x2ad6
        DCW 0xe467,0xe2b3, 0xdb11,0x3bc8, 0xa5a1,0x2ab0
        DCW 0xe4cc,0xe270, 0xdaf1,0x3bd1, 0xa5a5,0x2a8b
        DCW 0xe532,0xe22d, 0xdad1,0x3bda, 0xa5a9,0x2a65
        DCW 0xe598,0xe1eb, 0xdab0,0x3be3, 0xa5ae,0x2a3f
        DCW 0xe5fe,0xe1a8, 0xda90,0x3bec, 0xa5b2,0x2a1a
        DCW 0xe664,0xe166, 0xda6f,0x3bf5, 0xa5b7,0x29f4
        DCW 0xe6cb,0xe124, 0xda4f,0x3bfd, 0xa5bc,0x29ce
        DCW 0xe731,0xe0e2, 0xda2f,0x3c06, 0xa5c2,0x29a7
        DCW 0xe798,0xe0a0, 0xda0f,0x3c0f, 0xa5c7,0x2981
        DCW 0xe7fe,0xe05e, 0xd9ee,0x3c17, 0xa5cd,0x295b
        DCW 0xe865,0xe01d, 0xd9ce,0x3c20, 0xa5d3,0x2935
        DCW 0xe8cc,0xdfdc, 0xd9ae,0x3c29, 0xa5d9,0x290e
        DCW 0xe933,0xdf9b, 0xd98e,0x3c31, 0xa5e0,0x28e7
        DCW 0xe99b,0xdf5a, 0xd96d,0x3c3a, 0xa5e6,0x28c1
        DCW 0xea02,0xdf19, 0xd94d,0x3c42, 0xa5ed,0x289a
        DCW 0xea6a,0xded8, 0xd92d,0x3c4b, 0xa5f4,0x2873
        DCW 0xead1,0xde98, 0xd90d,0x3c53, 0xa5fb,0x284c
        DCW 0xeb39,0xde58, 0xd8ed,0x3c5b, 0xa603,0x2825
        DCW 0xeba1,0xde18, 0xd8cd,0x3c64, 0xa60b,0x27fe
        DCW 0xec09,0xddd8, 0xd8ad,0x3c6c, 0xa613,0x27d6
        DCW 0xec71,0xdd98, 0xd88d,0x3c74, 0xa61b,0x27af
        DCW 0xecd9,0xdd59, 0xd86d,0x3c7d, 0xa623,0x2788
        DCW 0xed41,0xdd19, 0xd84d,0x3c85, 0xa62c,0x2760
        DCW 0xeda9,0xdcda, 0xd82d,0x3c8d, 0xa635,0x2738
        DCW 0xee12,0xdc9b, 0xd80d,0x3c95, 0xa63e,0x2711
        DCW 0xee7b,0xdc5d, 0xd7ed,0x3c9d, 0xa647,0x26e9
        DCW 0xeee3,0xdc1e, 0xd7cd,0x3ca5, 0xa650,0x26c1
        DCW 0xef4c,0xdbe0, 0xd7ae,0x3cad, 0xa65a,0x2699
        DCW 0xefb5,0xdba2, 0xd78e,0x3cb5, 0xa664,0x2671
        DCW 0xf01e,0xdb64, 0xd76e,0x3cbd, 0xa66e,0x2648
        DCW 0xf087,0xdb26, 0xd74e,0x3cc5, 0xa678,0x2620
        DCW 0xf0f0,0xdae8, 0xd72f,0x3ccd, 0xa683,0x25f8
        DCW 0xf159,0xdaab, 0xd70f,0x3cd5, 0xa68e,0x25cf
        DCW 0xf1c2,0xda6e, 0xd6ef,0x3cdd, 0xa699,0x25a6
        DCW 0xf22c,0xda31, 0xd6cf,0x3ce4, 0xa6a4,0x257e
        DCW 0xf295,0xd9f4, 0xd6b0,0x3cec, 0xa6af,0x2555
        DCW 0xf2ff,0xd9b8, 0xd690,0x3cf4, 0xa6bb,0x252c
        DCW 0xf368,0xd97b, 0xd671,0x3cfb, 0xa6c7,0x2503
        DCW 0xf3d2,0xd93f, 0xd651,0x3d03, 0xa6d3,0x24da
        DCW 0xf43b,0xd903, 0xd632,0x3d0b, 0xa6df,0x24b1
        DCW 0xf4a5,0xd8c8, 0xd612,0x3d12, 0xa6eb,0x2488
        DCW 0xf50f,0xd88c, 0xd5f3,0x3d1a, 0xa6f8,0x245e
        DCW 0xf579,0xd851, 0xd5d3,0x3d21, 0xa705,0x2435
        DCW 0xf5e3,0xd816, 0xd5b4,0x3d28, 0xa712,0x240b
        DCW 0xf64d,0xd7db, 0xd594,0x3d30, 0xa71f,0x23e2
        DCW 0xf6b7,0xd7a0, 0xd575,0x3d37, 0xa72d,0x23b8
        DCW 0xf721,0xd766, 0xd556,0x3d3f, 0xa73b,0x238e
        DCW 0xf78b,0xd72c, 0xd536,0x3d46, 0xa749,0x2365
        DCW 0xf7f5,0xd6f2, 0xd517,0x3d4d, 0xa757,0x233b
        DCW 0xf85f,0xd6b8, 0xd4f8,0x3d54, 0xa765,0x2311
        DCW 0xf8ca,0xd67f, 0xd4d8,0x3d5b, 0xa774,0x22e7
        DCW 0xf934,0xd645, 0xd4b9,0x3d63, 0xa783,0x22bc
        DCW 0xf99e,0xd60c, 0xd49a,0x3d6a, 0xa792,0x2292
        DCW 0xfa09,0xd5d4, 0xd47b,0x3d71, 0xa7a1,0x2268
        DCW 0xfa73,0xd59b, 0xd45c,0x3d78, 0xa7b1,0x223d
        DCW 0xfade,0xd563, 0xd43c,0x3d7f, 0xa7c0,0x2213
        DCW 0xfb48,0xd52a, 0xd41d,0x3d86, 0xa7d0,0x21e8
        DCW 0xfbb3,0xd4f3, 0xd3fe,0x3d8d, 0xa7e0,0x21be
        DCW 0xfc1d,0xd4bb, 0xd3df,0x3d93, 0xa7f1,0x2193
        DCW 0xfc88,0xd483, 0xd3c0,0x3d9a, 0xa801,0x2168
        DCW 0xfcf2,0xd44c, 0xd3a1,0x3da1, 0xa812,0x213d
        DCW 0xfd5d,0xd415, 0xd382,0x3da8, 0xa823,0x2112
        DCW 0xfdc7,0xd3df, 0xd363,0x3daf, 0xa834,0x20e7
        DCW 0xfe32,0xd3a8, 0xd344,0x3db5, 0xa845,0x20bc
        DCW 0xfe9d,0xd372, 0xd326,0x3dbc, 0xa857,0x2091
        DCW 0xff07,0xd33c, 0xd307,0x3dc2, 0xa869,0x2065
        DCW 0xff72,0xd306, 0xd2e8,0x3dc9, 0xa87b,0x203a
        DCW 0xffdc,0xd2d1, 0xd2c9,0x3dd0, 0xa88d,0x200f
        DCW 0x0047,0xd29b, 0xd2aa,0x3dd6, 0xa89f,0x1fe3
        DCW 0x00b2,0xd266, 0xd28b,0x3ddd, 0xa8b2,0x1fb7
        DCW 0x011c,0xd231, 0xd26d,0x3de3, 0xa8c5,0x1f8c
        DCW 0x0187,0xd1fd, 0xd24e,0x3de9, 0xa8d8,0x1f60
        DCW 0x01f2,0xd1c9, 0xd22f,0x3df0, 0xa8eb,0x1f34
        DCW 0x025c,0xd195, 0xd211,0x3df6, 0xa8ff,0x1f08
        DCW 0x02c7,0xd161, 0xd1f2,0x3dfc, 0xa912,0x1edc
        DCW 0x0331,0xd12d, 0xd1d4,0x3e03, 0xa926,0x1eb0
        DCW 0x039c,0xd0fa, 0xd1b5,0x3e09, 0xa93a,0x1e84
        DCW 0x0406,0xd0c7, 0xd196,0x3e0f, 0xa94f,0x1e58
        DCW 0x0471,0xd094, 0xd178,0x3e15, 0xa963,0x1e2b
        DCW 0x04db,0xd062, 0xd15a,0x3e1b, 0xa978,0x1dff
        DCW 0x0546,0xd030, 0xd13b,0x3e21, 0xa98d,0x1dd3
        DCW 0x05b0,0xcffe, 0xd11d,0x3e27, 0xa9a2,0x1da6
        DCW 0x061b,0xcfcc, 0xd0fe,0x3e2d, 0xa9b7,0x1d79
        DCW 0x0685,0xcf9a, 0xd0e0,0x3e33, 0xa9cd,0x1d4d
        DCW 0x06ef,0xcf69, 0xd0c2,0x3e39, 0xa9e3,0x1d20
        DCW 0x075a,0xcf38, 0xd0a3,0x3e3f, 0xa9f9,0x1cf3
        DCW 0x07c4,0xcf07, 0xd085,0x3e45, 0xaa0f,0x1cc6
        DCW 0x082e,0xced7, 0xd067,0x3e4a, 0xaa25,0x1c99
        DCW 0x0898,0xcea7, 0xd049,0x3e50, 0xaa3c,0x1c6c
        DCW 0x0902,0xce77, 0xd02a,0x3e56, 0xaa53,0x1c3f
        DCW 0x096d,0xce47, 0xd00c,0x3e5c, 0xaa6a,0x1c12
        DCW 0x09d7,0xce18, 0xcfee,0x3e61, 0xaa81,0x1be5
        DCW 0x0a41,0xcde9, 0xcfd0,0x3e67, 0xaa98,0x1bb8
        DCW 0x0aaa,0xcdba, 0xcfb2,0x3e6c, 0xaab0,0x1b8a
        DCW 0x0b14,0xcd8c, 0xcf94,0x3e72, 0xaac8,0x1b5d
        DCW 0x0b7e,0xcd5d, 0xcf76,0x3e77, 0xaae0,0x1b30
        DCW 0x0be8,0xcd30, 0xcf58,0x3e7d, 0xaaf8,0x1b02
        DCW 0x0c52,0xcd02, 0xcf3a,0x3e82, 0xab11,0x1ad4
        DCW 0x0cbb,0xccd4, 0xcf1c,0x3e88, 0xab29,0x1aa7
        DCW 0x0d25,0xcca7, 0xcefe,0x3e8d, 0xab42,0x1a79
        DCW 0x0d8e,0xcc7a, 0xcee0,0x3e92, 0xab5b,0x1a4b
        DCW 0x0df8,0xcc4e, 0xcec2,0x3e98, 0xab75,0x1a1d
        DCW 0x0e61,0xcc21, 0xcea5,0x3e9d, 0xab8e,0x19ef
        DCW 0x0eca,0xcbf5, 0xce87,0x3ea2, 0xaba8,0x19c1
        DCW 0x0f33,0xcbca, 0xce69,0x3ea7, 0xabc2,0x1993
        DCW 0x0f9c,0xcb9e, 0xce4b,0x3eac, 0xabdc,0x1965
        DCW 0x1005,0xcb73, 0xce2e,0x3eb1, 0xabf6,0x1937
        DCW 0x106e,0xcb48, 0xce10,0x3eb6, 0xac11,0x1909
        DCW 0x10d7,0xcb1e, 0xcdf2,0x3ebb, 0xac2b,0x18db
        DCW 0x1140,0xcaf3, 0xcdd5,0x3ec0, 0xac46,0x18ac
        DCW 0x11a8,0xcac9, 0xcdb7,0x3ec5, 0xac61,0x187e
        DCW 0x1211,0xca9f, 0xcd9a,0x3eca, 0xac7d,0x184f
        DCW 0x1279,0xca76, 0xcd7c,0x3ecf, 0xac98,0x1821
        DCW 0x12e2,0xca4d, 0xcd5f,0x3ed4, 0xacb4,0x17f2
        DCW 0x134a,0xca24, 0xcd41,0x3ed8, 0xacd0,0x17c4
        DCW 0x13b2,0xc9fb, 0xcd24,0x3edd, 0xacec,0x1795
        DCW 0x141a,0xc9d3, 0xcd06,0x3ee2, 0xad08,0x1766
        DCW 0x1482,0xc9ab, 0xcce9,0x3ee7, 0xad25,0x1737
        DCW 0x14ea,0xc983, 0xcccc,0x3eeb, 0xad41,0x1709
        DCW 0x1551,0xc95c, 0xccae,0x3ef0, 0xad5e,0x16da
        DCW 0x15b9,0xc935, 0xcc91,0x3ef4, 0xad7b,0x16ab
        DCW 0x1620,0xc90e, 0xcc74,0x3ef9, 0xad99,0x167c
        DCW 0x1688,0xc8e8, 0xcc57,0x3efd, 0xadb6,0x164c
        DCW 0x16ef,0xc8c1, 0xcc3a,0x3f02, 0xadd4,0x161d
        DCW 0x1756,0xc89b, 0xcc1c,0x3f06, 0xadf2,0x15ee
        DCW 0x17bd,0xc876, 0xcbff,0x3f0a, 0xae10,0x15bf
        DCW 0x1824,0xc850, 0xcbe2,0x3f0f, 0xae2e,0x1590
        DCW 0x188b,0xc82b, 0xcbc5,0x3f13, 0xae4d,0x1560
        DCW 0x18f1,0xc807, 0xcba8,0x3f17, 0xae6b,0x1531
        DCW 0x1958,0xc7e2, 0xcb8b,0x3f1c, 0xae8a,0x1501
        DCW 0x19be,0xc7be, 0xcb6e,0x3f20, 0xaea9,0x14d2
        DCW 0x1a24,0xc79a, 0xcb51,0x3f24, 0xaec9,0x14a2
        DCW 0x1a8a,0xc777, 0xcb34,0x3f28, 0xaee8,0x1473
        DCW 0x1af0,0xc754, 0xcb18,0x3f2c, 0xaf08,0x1443
        DCW 0x1b56,0xc731, 0xcafb,0x3f30, 0xaf28,0x1413
        DCW 0x1bbb,0xc70e, 0xcade,0x3f34, 0xaf48,0x13e4
        DCW 0x1c21,0xc6ec, 0xcac1,0x3f38, 0xaf68,0x13b4
        DCW 0x1c86,0xc6ca, 0xcaa5,0x3f3c, 0xaf88,0x1384
        DCW 0x1ceb,0xc6a8, 0xca88,0x3f40, 0xafa9,0x1354
        DCW 0x1d50,0xc687, 0xca6b,0x3f43, 0xafca,0x1324
        DCW 0x1db5,0xc666, 0xca4f,0x3f47, 0xafeb,0x12f4
        DCW 0x1e19,0xc645, 0xca32,0x3f4b, 0xb00c,0x12c4
        DCW 0x1e7e,0xc625, 0xca15,0x3f4f, 0xb02d,0x1294
        DCW 0x1ee2,0xc605, 0xc9f9,0x3f52, 0xb04f,0x1264
        DCW 0x1f46,0xc5e5, 0xc9dc,0x3f56, 0xb071,0x1234
        DCW 0x1faa,0xc5c6, 0xc9c0,0x3f5a, 0xb093,0x1204
        DCW 0x200e,0xc5a7, 0xc9a3,0x3f5d, 0xb0b5,0x11d3
        DCW 0x2072,0xc588, 0xc987,0x3f61, 0xb0d7,0x11a3
        DCW 0x20d5,0xc569, 0xc96b,0x3f64, 0xb0fa,0x1173
        DCW 0x2138,0xc54b, 0xc94e,0x3f68, 0xb11d,0x1142
        DCW 0x219c,0xc52d, 0xc932,0x3f6b, 0xb140,0x1112
        DCW 0x21ff,0xc510, 0xc916,0x3f6e, 0xb163,0x10e1
        DCW 0x2261,0xc4f2, 0xc8fa,0x3f72, 0xb186,0x10b1
        DCW 0x22c4,0xc4d6, 0xc8dd,0x3f75, 0xb1aa,0x1080
        DCW 0x2326,0xc4b9, 0xc8c1,0x3f78, 0xb1cd,0x1050
        DCW 0x2388,0xc49d, 0xc8a5,0x3f7b, 0xb1f1,0x101f
        DCW 0x23ea,0xc481, 0xc889,0x3f7f, 0xb215,0x0fee
        DCW 0x244c,0xc465, 0xc86d,0x3f82, 0xb23a,0x0fbe
        DCW 0x24ae,0xc44a, 0xc851,0x3f85, 0xb25e,0x0f8d
        DCW 0x250f,0xc42f, 0xc835,0x3f88, 0xb283,0x0f5c
        DCW 0x2570,0xc414, 0xc819,0x3f8b, 0xb2a7,0x0f2b
        DCW 0x25d1,0xc3fa, 0xc7fd,0x3f8e, 0xb2cc,0x0efb
        DCW 0x2632,0xc3e0, 0xc7e1,0x3f91, 0xb2f2,0x0eca
        DCW 0x2693,0xc3c6, 0xc7c5,0x3f94, 0xb317,0x0e99
        DCW 0x26f3,0xc3ad, 0xc7a9,0x3f97, 0xb33d,0x0e68
        DCW 0x2753,0xc394, 0xc78d,0x3f99, 0xb362,0x0e37
        DCW 0x27b3,0xc37b, 0xc772,0x3f9c, 0xb388,0x0e06
        DCW 0x2813,0xc363, 0xc756,0x3f9f, 0xb3ae,0x0dd5
        DCW 0x2872,0xc34b, 0xc73a,0x3fa2, 0xb3d5,0x0da4
        DCW 0x28d1,0xc333, 0xc71f,0x3fa4, 0xb3fb,0x0d72
        DCW 0x2931,0xc31c, 0xc703,0x3fa7, 0xb422,0x0d41
        DCW 0x298f,0xc305, 0xc6e7,0x3faa, 0xb449,0x0d10
        DCW 0x29ee,0xc2ee, 0xc6cc,0x3fac, 0xb470,0x0cdf
        DCW 0x2a4c,0xc2d8, 0xc6b0,0x3faf, 0xb497,0x0cae
        DCW 0x2aaa,0xc2c1, 0xc695,0x3fb1, 0xb4be,0x0c7c
        DCW 0x2b08,0xc2ac, 0xc679,0x3fb4, 0xb4e6,0x0c4b
        DCW 0x2b66,0xc296, 0xc65e,0x3fb6, 0xb50e,0x0c1a
        DCW 0x2bc4,0xc281, 0xc643,0x3fb8, 0xb536,0x0be8
        DCW 0x2c21,0xc26d, 0xc627,0x3fbb, 0xb55e,0x0bb7
        DCW 0x2c7e,0xc258, 0xc60c,0x3fbd, 0xb586,0x0b85
        DCW 0x2cda,0xc244, 0xc5f1,0x3fbf, 0xb5af,0x0b54
        DCW 0x2d37,0xc230, 0xc5d5,0x3fc1, 0xb5d7,0x0b23
        DCW 0x2d93,0xc21d, 0xc5ba,0x3fc4, 0xb600,0x0af1
        DCW 0x2def,0xc20a, 0xc59f,0x3fc6, 0xb629,0x0ac0
        DCW 0x2e4b,0xc1f7, 0xc584,0x3fc8, 0xb652,0x0a8e
        DCW 0x2ea6,0xc1e5, 0xc569,0x3fca, 0xb67c,0x0a5c
        DCW 0x2f02,0xc1d3, 0xc54e,0x3fcc, 0xb6a5,0x0a2b
        DCW 0x2f5d,0xc1c1, 0xc533,0x3fce, 0xb6cf,0x09f9
        DCW 0x2fb7,0xc1b0, 0xc518,0x3fd0, 0xb6f9,0x09c7
        DCW 0x3012,0xc19f, 0xc4fd,0x3fd2, 0xb723,0x0996
        DCW 0x306c,0xc18e, 0xc4e2,0x3fd4, 0xb74d,0x0964
        DCW 0x30c6,0xc17e, 0xc4c7,0x3fd5, 0xb778,0x0932
        DCW 0x3120,0xc16e, 0xc4ac,0x3fd7, 0xb7a2,0x0901
        DCW 0x3179,0xc15e, 0xc491,0x3fd9, 0xb7cd,0x08cf
        DCW 0x31d2,0xc14f, 0xc476,0x3fdb, 0xb7f8,0x089d
        DCW 0x322b,0xc140, 0xc45c,0x3fdc, 0xb823,0x086b
        DCW 0x3284,0xc131, 0xc441,0x3fde, 0xb84f,0x0839
        DCW 0x32dc,0xc123, 0xc426,0x3fe0, 0xb87a,0x0807
        DCW 0x3334,0xc115, 0xc40c,0x3fe1, 0xb8a6,0x07d6
        DCW 0x338c,0xc107, 0xc3f1,0x3fe3, 0xb8d1,0x07a4
        DCW 0x33e4,0xc0fa, 0xc3d6,0x3fe4, 0xb8fd,0x0772
        DCW 0x343b,0xc0ed, 0xc3bc,0x3fe6, 0xb92a,0x0740
        DCW 0x3492,0xc0e0, 0xc3a1,0x3fe7, 0xb956,0x070e
        DCW 0x34e8,0xc0d4, 0xc387,0x3fe8, 0xb982,0x06dc
        DCW 0x353f,0xc0c8, 0xc36c,0x3fea, 0xb9af,0x06aa
        DCW 0x3595,0xc0bd, 0xc352,0x3feb, 0xb9dc,0x0678
        DCW 0x35eb,0xc0b1, 0xc338,0x3fec, 0xba09,0x0646
        DCW 0x3640,0xc0a6, 0xc31d,0x3fed, 0xba36,0x0614
        DCW 0x3695,0xc09c, 0xc303,0x3fef, 0xba64,0x05e2
        DCW 0x36ea,0xc092, 0xc2e9,0x3ff0, 0xba91,0x05b0
        DCW 0x373f,0xc088, 0xc2cf,0x3ff1, 0xbabf,0x057e
        DCW 0x3793,0xc07e, 0xc2b4,0x3ff2, 0xbaed,0x054c
        DCW 0x37e7,0xc075, 0xc29a,0x3ff3, 0xbb1b,0x051a
        DCW 0x383b,0xc06c, 0xc280,0x3ff4, 0xbb49,0x04e7
        DCW 0x388e,0xc064, 0xc266,0x3ff5, 0xbb77,0x04b5
        DCW 0x38e1,0xc05c, 0xc24c,0x3ff6, 0xbba6,0x0483
        DCW 0x3934,0xc054, 0xc232,0x3ff7, 0xbbd4,0x0451
        DCW 0x3987,0xc04c, 0xc218,0x3ff7, 0xbc03,0x041f
        DCW 0x39d9,0xc045, 0xc1fe,0x3ff8, 0xbc32,0x03ed
        DCW 0x3a2b,0xc03f, 0xc1e4,0x3ff9, 0xbc61,0x03bb
        DCW 0x3a7c,0xc038, 0xc1cb,0x3ffa, 0xbc91,0x0388
        DCW 0x3acd,0xc032, 0xc1b1,0x3ffa, 0xbcc0,0x0356
        DCW 0x3b1e,0xc02c, 0xc197,0x3ffb, 0xbcf0,0x0324
        DCW 0x3b6f,0xc027, 0xc17d,0x3ffc, 0xbd20,0x02f2
        DCW 0x3bbf,0xc022, 0xc164,0x3ffc, 0xbd50,0x02c0
        DCW 0x3c0f,0xc01d, 0xc14a,0x3ffd, 0xbd80,0x028d
        DCW 0x3c5f,0xc019, 0xc130,0x3ffd, 0xbdb0,0x025b
        DCW 0x3cae,0xc015, 0xc117,0x3ffe, 0xbde1,0x0229
        DCW 0x3cfd,0xc011, 0xc0fd,0x3ffe, 0xbe11,0x01f7
        DCW 0x3d4c,0xc00e, 0xc0e4,0x3ffe, 0xbe42,0x01c4
        DCW 0x3d9a,0xc00b, 0xc0ca,0x3fff, 0xbe73,0x0192
        DCW 0x3de8,0xc009, 0xc0b1,0x3fff, 0xbea4,0x0160
        DCW 0x3e35,0xc006, 0xc097,0x3fff, 0xbed5,0x012e
        DCW 0x3e83,0xc004, 0xc07e,0x4000, 0xbf07,0x00fb
        DCW 0x3ed0,0xc003, 0xc065,0x4000, 0xbf38,0x00c9
        DCW 0x3f1c,0xc002, 0xc04c,0x4000, 0xbf6a,0x0097
        DCW 0x3f69,0xc001, 0xc032,0x4000, 0xbf9c,0x0065
        DCW 0x3fb4,0xc000, 0xc019,0x4000, 0xbfce,0x0032

;// Example 8.18

        ; Complex conjugate multiply a=(xr+i*xi)*(cr-i*ci)
        ;  x = xr + i*xi (two 16-bits packed in 32-bit)
        ;  w = cr + i*ci (two 16-bits packed in 32-bit)
        MACRO
        C_MUL9e $a, $x, $w
        SMULBT  t0, $x, $w              ; xr*ci
        SMULBB  $a._r, $x, $w           ; xr*cr
        SMULTB  $a._i, $x, $w           ; xi*cr
        SMLATT  $a._r, $x, $w, $a._r    ; xr*cr+xi*ci
        SUB     $a._i, $a._i, t0        ; xi*cr-xr*ci
        MEND

        ; void fft_16_arm9e(short *y, short *x, unsigned int N)
fft_16_arm9e
        STMFD   sp!, {r4-r11, lr}
        MOV     t0, #0            ; bit reversed counter
        MVN     R, #0x80000000    ; R=0x7FFFFFFF
first_stage_arm9e
        ; first stage load and bit reverse
        ADDS    t1,  x, t0, LSL#2 ; t1=&x[t0] and clear carry
        C_LDR   x0, t1, N
        C_LDR   x2, t1, N
        C_LDR   x1, t1, N
        C_LDR   x3, t1, N
        C_FFT4  0
        C_STR   x0, y, #4
        C_STR   x1, y, #4
        C_STR   x2, y, #4
        C_STR   y3, y, #4
        ; bit reversed increment modulo (N/4)
        RSC     t0, t0, N, LSR #2 ; t0 = (N/4)-t0-1
        CLZ     t1, t0            ; find leading 1
        EORS    t0, t0, R, ASR t1 ; toggle bits below leading 1
        BNE     first_stage_arm9e ; loop if count non zero
        ; finished the first stage
        SUB     x, y, N, LSL #2   ; x = working buffer
        MOV     R, #16
        MOVS    S, N, LSR#4
        LDMEQFD sp!, {r4-r11, pc}
        ADR     c, fft_table_arm9e
next_stage_arm9e
        ; S = the number of blocks
        ; R = the number of samples in each block
        STMFD   sp!, {x, S}
        ADD     t0, R, R, LSL#1
        ADD     x, x, t0
        SUB     S, S, #1<<16
next_block_arm9e
        ADD     S, S, R, LSL#(16-2)
next_butterfly_arm9e
        ; S=((number butterflies left-1)<<16)
        ;   + (number of blocks left)
        LDR     x2_r, [x], -R   ; packed data
        LDR     x2_i, [c], #4   ; packed coefficients
        LDR     x1_r, [x], -R
        LDR     x1_i, [c], #4
        LDR     x0_r, [x], -R
        LDR     x0_i, [c], #4
        C_MUL9e x3, x2_r, x2_i
        C_MUL9e x2, x1_r, x1_i
        C_MUL9e x1, x0_r, x0_i
        C_LDR   x0, x, #0
        C_FFT4  15              ; coefficients are Q15
        C_STR   x0, x, R
        C_STR   x1, x, R
        C_STR   x2, x, R
        C_STR   y3, x, #4
        SUBS    S, S, #1<<16
        BGE     next_butterfly_arm9e
        ADD     t0, R, R, LSL#1
        ADD     x, x, t0
        SUB     S, S, #1
        MOVS    t1, S, LSL#16
        SUBNE   c, c, t0
        BNE     next_block_arm9e
        LDMFD   sp!, {x, S}
        MOV     R, R, LSL#2     ; quadrouple block size
        MOVS    S, S, LSR#2     ; quarter number of blocks
        BNE     next_stage_arm9e
        LDMFD   sp!, {r4-r11, pc}
        
fft_table_arm9e
        ; FFT twiddle table of triplets E(3t), E(t), E(2t)
        ; Where E(t)=cos(t)+i*sin(t) at Q15
        ; N=16 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x7fff,0x0000, 0x7fff,0x0000, 0x7fff,0x0000
        DCW 0x30fc,0x7642, 0x7642,0x30fc, 0x5a82,0x5a82
        DCW 0xa57e,0x5a82, 0x5a82,0x5a82, 0x0000,0x7fff
        DCW 0x89be,0xcf04, 0x30fc,0x7642, 0xa57e,0x5a82
        ; N=64 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x7fff,0x0000, 0x7fff,0x0000, 0x7fff,0x0000
        DCW 0x7a7d,0x2528, 0x7f62,0x0c8c, 0x7d8a,0x18f9
        DCW 0x6a6e,0x471d, 0x7d8a,0x18f9, 0x7642,0x30fc
        DCW 0x5134,0x62f2, 0x7a7d,0x2528, 0x6a6e,0x471d
        DCW 0x30fc,0x7642, 0x7642,0x30fc, 0x5a82,0x5a82
        DCW 0x0c8c,0x7f62, 0x70e3,0x3c57, 0x471d,0x6a6e
        DCW 0xe707,0x7d8a, 0x6a6e,0x471d, 0x30fc,0x7642
        DCW 0xc3a9,0x70e3, 0x62f2,0x5134, 0x18f9,0x7d8a
        DCW 0xa57e,0x5a82, 0x5a82,0x5a82, 0x0000,0x7fff
        DCW 0x8f1d,0x3c57, 0x5134,0x62f2, 0xe707,0x7d8a
        DCW 0x8276,0x18f9, 0x471d,0x6a6e, 0xcf04,0x7642
        DCW 0x809e,0xf374, 0x3c57,0x70e3, 0xb8e3,0x6a6e
        DCW 0x89be,0xcf04, 0x30fc,0x7642, 0xa57e,0x5a82
        DCW 0x9d0e,0xaecc, 0x2528,0x7a7d, 0x9592,0x471d
        DCW 0xb8e3,0x9592, 0x18f9,0x7d8a, 0x89be,0x30fc
        DCW 0xdad8,0x8583, 0x0c8c,0x7f62, 0x8276,0x18f9
        ; N=256 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x7fff,0x0000, 0x7fff,0x0000, 0x7fff,0x0000
        DCW 0x7fa7,0x096b, 0x7ff6,0x0324, 0x7fd9,0x0648
        DCW 0x7e9d,0x12c8, 0x7fd9,0x0648, 0x7f62,0x0c8c
        DCW 0x7ce4,0x1c0c, 0x7fa7,0x096b, 0x7e9d,0x12c8
        DCW 0x7a7d,0x2528, 0x7f62,0x0c8c, 0x7d8a,0x18f9
        DCW 0x776c,0x2e11, 0x7f0a,0x0fab, 0x7c2a,0x1f1a
        DCW 0x73b6,0x36ba, 0x7e9d,0x12c8, 0x7a7d,0x2528
        DCW 0x6f5f,0x3f17, 0x7e1e,0x15e2, 0x7885,0x2b1f
        DCW 0x6a6e,0x471d, 0x7d8a,0x18f9, 0x7642,0x30fc
        DCW 0x64e9,0x4ec0, 0x7ce4,0x1c0c, 0x73b6,0x36ba
        DCW 0x5ed7,0x55f6, 0x7c2a,0x1f1a, 0x70e3,0x3c57
        DCW 0x5843,0x5cb4, 0x7b5d,0x2224, 0x6dca,0x41ce
        DCW 0x5134,0x62f2, 0x7a7d,0x2528, 0x6a6e,0x471d
        DCW 0x49b4,0x68a7, 0x798a,0x2827, 0x66d0,0x4c40
        DCW 0x41ce,0x6dca, 0x7885,0x2b1f, 0x62f2,0x5134
        DCW 0x398d,0x7255, 0x776c,0x2e11, 0x5ed7,0x55f6
        DCW 0x30fc,0x7642, 0x7642,0x30fc, 0x5a82,0x5a82
        DCW 0x2827,0x798a, 0x7505,0x33df, 0x55f6,0x5ed7
        DCW 0x1f1a,0x7c2a, 0x73b6,0x36ba, 0x5134,0x62f2
        DCW 0x15e2,0x7e1e, 0x7255,0x398d, 0x4c40,0x66d0
        DCW 0x0c8c,0x7f62, 0x70e3,0x3c57, 0x471d,0x6a6e
        DCW 0x0324,0x7ff6, 0x6f5f,0x3f17, 0x41ce,0x6dca
        DCW 0xf9b8,0x7fd9, 0x6dca,0x41ce, 0x3c57,0x70e3
        DCW 0xf055,0x7f0a, 0x6c24,0x447b, 0x36ba,0x73b6
        DCW 0xe707,0x7d8a, 0x6a6e,0x471d, 0x30fc,0x7642
        DCW 0xdddc,0x7b5d, 0x68a7,0x49b4, 0x2b1f,0x7885
        DCW 0xd4e1,0x7885, 0x66d0,0x4c40, 0x2528,0x7a7d
        DCW 0xcc21,0x7505, 0x64e9,0x4ec0, 0x1f1a,0x7c2a
        DCW 0xc3a9,0x70e3, 0x62f2,0x5134, 0x18f9,0x7d8a
        DCW 0xbb85,0x6c24, 0x60ec,0x539b, 0x12c8,0x7e9d
        DCW 0xb3c0,0x66d0, 0x5ed7,0x55f6, 0x0c8c,0x7f62
        DCW 0xac65,0x60ec, 0x5cb4,0x5843, 0x0648,0x7fd9
        DCW 0xa57e,0x5a82, 0x5a82,0x5a82, 0x0000,0x7fff
        DCW 0x9f14,0x539b, 0x5843,0x5cb4, 0xf9b8,0x7fd9
        DCW 0x9930,0x4c40, 0x55f6,0x5ed7, 0xf374,0x7f62
        DCW 0x93dc,0x447b, 0x539b,0x60ec, 0xed38,0x7e9d
        DCW 0x8f1d,0x3c57, 0x5134,0x62f2, 0xe707,0x7d8a
        DCW 0x8afb,0x33df, 0x4ec0,0x64e9, 0xe0e6,0x7c2a
        DCW 0x877b,0x2b1f, 0x4c40,0x66d0, 0xdad8,0x7a7d
        DCW 0x84a3,0x2224, 0x49b4,0x68a7, 0xd4e1,0x7885
        DCW 0x8276,0x18f9, 0x471d,0x6a6e, 0xcf04,0x7642
        DCW 0x80f6,0x0fab, 0x447b,0x6c24, 0xc946,0x73b6
        DCW 0x8027,0x0648, 0x41ce,0x6dca, 0xc3a9,0x70e3
        DCW 0x800a,0xfcdc, 0x3f17,0x6f5f, 0xbe32,0x6dca
        DCW 0x809e,0xf374, 0x3c57,0x70e3, 0xb8e3,0x6a6e
        DCW 0x81e2,0xea1e, 0x398d,0x7255, 0xb3c0,0x66d0
        DCW 0x83d6,0xe0e6, 0x36ba,0x73b6, 0xaecc,0x62f2
        DCW 0x8676,0xd7d9, 0x33df,0x7505, 0xaa0a,0x5ed7
        DCW 0x89be,0xcf04, 0x30fc,0x7642, 0xa57e,0x5a82
        DCW 0x8dab,0xc673, 0x2e11,0x776c, 0xa129,0x55f6
        DCW 0x9236,0xbe32, 0x2b1f,0x7885, 0x9d0e,0x5134
        DCW 0x9759,0xb64c, 0x2827,0x798a, 0x9930,0x4c40
        DCW 0x9d0e,0xaecc, 0x2528,0x7a7d, 0x9592,0x471d
        DCW 0xa34c,0xa7bd, 0x2224,0x7b5d, 0x9236,0x41ce
        DCW 0xaa0a,0xa129, 0x1f1a,0x7c2a, 0x8f1d,0x3c57
        DCW 0xb140,0x9b17, 0x1c0c,0x7ce4, 0x8c4a,0x36ba
        DCW 0xb8e3,0x9592, 0x18f9,0x7d8a, 0x89be,0x30fc
        DCW 0xc0e9,0x90a1, 0x15e2,0x7e1e, 0x877b,0x2b1f
        DCW 0xc946,0x8c4a, 0x12c8,0x7e9d, 0x8583,0x2528
        DCW 0xd1ef,0x8894, 0x0fab,0x7f0a, 0x83d6,0x1f1a
        DCW 0xdad8,0x8583, 0x0c8c,0x7f62, 0x8276,0x18f9
        DCW 0xe3f4,0x831c, 0x096b,0x7fa7, 0x8163,0x12c8
        DCW 0xed38,0x8163, 0x0648,0x7fd9, 0x809e,0x0c8c
        DCW 0xf695,0x8059, 0x0324,0x7ff6, 0x8027,0x0648
        ; N=1024 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x7fff,0x0000, 0x7fff,0x0000, 0x7fff,0x0000
        DCW 0x7ffa,0x025b, 0x7fff,0x00c9, 0x7ffe,0x0192
        DCW 0x7fea,0x04b6, 0x7ffe,0x0192, 0x7ff6,0x0324
        DCW 0x7fce,0x0711, 0x7ffa,0x025b, 0x7fea,0x04b6
        DCW 0x7fa7,0x096b, 0x7ff6,0x0324, 0x7fd9,0x0648
        DCW 0x7f75,0x0bc4, 0x7ff1,0x03ed, 0x7fc2,0x07d9
        DCW 0x7f38,0x0e1c, 0x7fea,0x04b6, 0x7fa7,0x096b
        DCW 0x7ef0,0x1073, 0x7fe2,0x057f, 0x7f87,0x0afb
        DCW 0x7e9d,0x12c8, 0x7fd9,0x0648, 0x7f62,0x0c8c
        DCW 0x7e3f,0x151c, 0x7fce,0x0711, 0x7f38,0x0e1c
        DCW 0x7dd6,0x176e, 0x7fc2,0x07d9, 0x7f0a,0x0fab
        DCW 0x7d63,0x19be, 0x7fb5,0x08a2, 0x7ed6,0x113a
        DCW 0x7ce4,0x1c0c, 0x7fa7,0x096b, 0x7e9d,0x12c8
        DCW 0x7c5a,0x1e57, 0x7f98,0x0a33, 0x7e60,0x1455
        DCW 0x7bc6,0x209f, 0x7f87,0x0afb, 0x7e1e,0x15e2
        DCW 0x7b27,0x22e5, 0x7f75,0x0bc4, 0x7dd6,0x176e
        DCW 0x7a7d,0x2528, 0x7f62,0x0c8c, 0x7d8a,0x18f9
        DCW 0x79c9,0x2768, 0x7f4e,0x0d54, 0x7d3a,0x1a83
        DCW 0x790a,0x29a4, 0x7f38,0x0e1c, 0x7ce4,0x1c0c
        DCW 0x7840,0x2bdc, 0x7f22,0x0ee4, 0x7c89,0x1d93
        DCW 0x776c,0x2e11, 0x7f0a,0x0fab, 0x7c2a,0x1f1a
        DCW 0x768e,0x3042, 0x7ef0,0x1073, 0x7bc6,0x209f
        DCW 0x75a6,0x326e, 0x7ed6,0x113a, 0x7b5d,0x2224
        DCW 0x74b3,0x3497, 0x7eba,0x1201, 0x7aef,0x23a7
        DCW 0x73b6,0x36ba, 0x7e9d,0x12c8, 0x7a7d,0x2528
        DCW 0x72af,0x38d9, 0x7e7f,0x138f, 0x7a06,0x26a8
        DCW 0x719e,0x3af3, 0x7e60,0x1455, 0x798a,0x2827
        DCW 0x7083,0x3d08, 0x7e3f,0x151c, 0x790a,0x29a4
        DCW 0x6f5f,0x3f17, 0x7e1e,0x15e2, 0x7885,0x2b1f
        DCW 0x6e31,0x4121, 0x7dfb,0x16a8, 0x77fb,0x2c99
        DCW 0x6cf9,0x4326, 0x7dd6,0x176e, 0x776c,0x2e11
        DCW 0x6bb8,0x4524, 0x7db1,0x1833, 0x76d9,0x2f87
        DCW 0x6a6e,0x471d, 0x7d8a,0x18f9, 0x7642,0x30fc
        DCW 0x691a,0x490f, 0x7d63,0x19be, 0x75a6,0x326e
        DCW 0x67bd,0x4afb, 0x7d3a,0x1a83, 0x7505,0x33df
        DCW 0x6657,0x4ce1, 0x7d0f,0x1b47, 0x7460,0x354e
        DCW 0x64e9,0x4ec0, 0x7ce4,0x1c0c, 0x73b6,0x36ba
        DCW 0x6371,0x5098, 0x7cb7,0x1cd0, 0x7308,0x3825
        DCW 0x61f1,0x5269, 0x7c89,0x1d93, 0x7255,0x398d
        DCW 0x6068,0x5433, 0x7c5a,0x1e57, 0x719e,0x3af3
        DCW 0x5ed7,0x55f6, 0x7c2a,0x1f1a, 0x70e3,0x3c57
        DCW 0x5d3e,0x57b1, 0x7bf9,0x1fdd, 0x7023,0x3db8
        DCW 0x5b9d,0x5964, 0x7bc6,0x209f, 0x6f5f,0x3f17
        DCW 0x59f4,0x5b10, 0x7b92,0x2162, 0x6e97,0x4074
        DCW 0x5843,0x5cb4, 0x7b5d,0x2224, 0x6dca,0x41ce
        DCW 0x568a,0x5e50, 0x7b27,0x22e5, 0x6cf9,0x4326
        DCW 0x54ca,0x5fe4, 0x7aef,0x23a7, 0x6c24,0x447b
        DCW 0x5303,0x616f, 0x7ab7,0x2467, 0x6b4b,0x45cd
        DCW 0x5134,0x62f2, 0x7a7d,0x2528, 0x6a6e,0x471d
        DCW 0x4f5e,0x646c, 0x7a42,0x25e8, 0x698c,0x486a
        DCW 0x4d81,0x65de, 0x7a06,0x26a8, 0x68a7,0x49b4
        DCW 0x4b9e,0x6747, 0x79c9,0x2768, 0x67bd,0x4afb
        DCW 0x49b4,0x68a7, 0x798a,0x2827, 0x66d0,0x4c40
        DCW 0x47c4,0x69fd, 0x794a,0x28e5, 0x65de,0x4d81
        DCW 0x45cd,0x6b4b, 0x790a,0x29a4, 0x64e9,0x4ec0
        DCW 0x43d1,0x6c8f, 0x78c8,0x2a62, 0x63ef,0x4ffb
        DCW 0x41ce,0x6dca, 0x7885,0x2b1f, 0x62f2,0x5134
        DCW 0x3fc6,0x6efb, 0x7840,0x2bdc, 0x61f1,0x5269
        DCW 0x3db8,0x7023, 0x77fb,0x2c99, 0x60ec,0x539b
        DCW 0x3ba5,0x7141, 0x77b4,0x2d55, 0x5fe4,0x54ca
        DCW 0x398d,0x7255, 0x776c,0x2e11, 0x5ed7,0x55f6
        DCW 0x3770,0x735f, 0x7723,0x2ecc, 0x5dc8,0x571e
        DCW 0x354e,0x7460, 0x76d9,0x2f87, 0x5cb4,0x5843
        DCW 0x3327,0x7556, 0x768e,0x3042, 0x5b9d,0x5964
        DCW 0x30fc,0x7642, 0x7642,0x30fc, 0x5a82,0x5a82
        DCW 0x2ecc,0x7723, 0x75f4,0x31b5, 0x5964,0x5b9d
        DCW 0x2c99,0x77fb, 0x75a6,0x326e, 0x5843,0x5cb4
        DCW 0x2a62,0x78c8, 0x7556,0x3327, 0x571e,0x5dc8
        DCW 0x2827,0x798a, 0x7505,0x33df, 0x55f6,0x5ed7
        DCW 0x25e8,0x7a42, 0x74b3,0x3497, 0x54ca,0x5fe4
        DCW 0x23a7,0x7aef, 0x7460,0x354e, 0x539b,0x60ec
        DCW 0x2162,0x7b92, 0x740b,0x3604, 0x5269,0x61f1
        DCW 0x1f1a,0x7c2a, 0x73b6,0x36ba, 0x5134,0x62f2
        DCW 0x1cd0,0x7cb7, 0x735f,0x3770, 0x4ffb,0x63ef
        DCW 0x1a83,0x7d3a, 0x7308,0x3825, 0x4ec0,0x64e9
        DCW 0x1833,0x7db1, 0x72af,0x38d9, 0x4d81,0x65de
        DCW 0x15e2,0x7e1e, 0x7255,0x398d, 0x4c40,0x66d0
        DCW 0x138f,0x7e7f, 0x71fa,0x3a40, 0x4afb,0x67bd
        DCW 0x113a,0x7ed6, 0x719e,0x3af3, 0x49b4,0x68a7
        DCW 0x0ee4,0x7f22, 0x7141,0x3ba5, 0x486a,0x698c
        DCW 0x0c8c,0x7f62, 0x70e3,0x3c57, 0x471d,0x6a6e
        DCW 0x0a33,0x7f98, 0x7083,0x3d08, 0x45cd,0x6b4b
        DCW 0x07d9,0x7fc2, 0x7023,0x3db8, 0x447b,0x6c24
        DCW 0x057f,0x7fe2, 0x6fc2,0x3e68, 0x4326,0x6cf9
        DCW 0x0324,0x7ff6, 0x6f5f,0x3f17, 0x41ce,0x6dca
        DCW 0x00c9,0x7fff, 0x6efb,0x3fc6, 0x4074,0x6e97
        DCW 0xfe6e,0x7ffe, 0x6e97,0x4074, 0x3f17,0x6f5f
        DCW 0xfc13,0x7ff1, 0x6e31,0x4121, 0x3db8,0x7023
        DCW 0xf9b8,0x7fd9, 0x6dca,0x41ce, 0x3c57,0x70e3
        DCW 0xf75e,0x7fb5, 0x6d62,0x427a, 0x3af3,0x719e
        DCW 0xf505,0x7f87, 0x6cf9,0x4326, 0x398d,0x7255
        DCW 0xf2ac,0x7f4e, 0x6c8f,0x43d1, 0x3825,0x7308
        DCW 0xf055,0x7f0a, 0x6c24,0x447b, 0x36ba,0x73b6
        DCW 0xedff,0x7eba, 0x6bb8,0x4524, 0x354e,0x7460
        DCW 0xebab,0x7e60, 0x6b4b,0x45cd, 0x33df,0x7505
        DCW 0xe958,0x7dfb, 0x6add,0x4675, 0x326e,0x75a6
        DCW 0xe707,0x7d8a, 0x6a6e,0x471d, 0x30fc,0x7642
        DCW 0xe4b9,0x7d0f, 0x69fd,0x47c4, 0x2f87,0x76d9
        DCW 0xe26d,0x7c89, 0x698c,0x486a, 0x2e11,0x776c
        DCW 0xe023,0x7bf9, 0x691a,0x490f, 0x2c99,0x77fb
        DCW 0xdddc,0x7b5d, 0x68a7,0x49b4, 0x2b1f,0x7885
        DCW 0xdb99,0x7ab7, 0x6832,0x4a58, 0x29a4,0x790a
        DCW 0xd958,0x7a06, 0x67bd,0x4afb, 0x2827,0x798a
        DCW 0xd71b,0x794a, 0x6747,0x4b9e, 0x26a8,0x7a06
        DCW 0xd4e1,0x7885, 0x66d0,0x4c40, 0x2528,0x7a7d
        DCW 0xd2ab,0x77b4, 0x6657,0x4ce1, 0x23a7,0x7aef
        DCW 0xd079,0x76d9, 0x65de,0x4d81, 0x2224,0x7b5d
        DCW 0xce4b,0x75f4, 0x6564,0x4e21, 0x209f,0x7bc6
        DCW 0xcc21,0x7505, 0x64e9,0x4ec0, 0x1f1a,0x7c2a
        DCW 0xc9fc,0x740b, 0x646c,0x4f5e, 0x1d93,0x7c89
        DCW 0xc7db,0x7308, 0x63ef,0x4ffb, 0x1c0c,0x7ce4
        DCW 0xc5c0,0x71fa, 0x6371,0x5098, 0x1a83,0x7d3a
        DCW 0xc3a9,0x70e3, 0x62f2,0x5134, 0x18f9,0x7d8a
        DCW 0xc198,0x6fc2, 0x6272,0x51cf, 0x176e,0x7dd6
        DCW 0xbf8c,0x6e97, 0x61f1,0x5269, 0x15e2,0x7e1e
        DCW 0xbd86,0x6d62, 0x616f,0x5303, 0x1455,0x7e60
        DCW 0xbb85,0x6c24, 0x60ec,0x539b, 0x12c8,0x7e9d
        DCW 0xb98b,0x6add, 0x6068,0x5433, 0x113a,0x7ed6
        DCW 0xb796,0x698c, 0x5fe4,0x54ca, 0x0fab,0x7f0a
        DCW 0xb5a8,0x6832, 0x5f5e,0x5560, 0x0e1c,0x7f38
        DCW 0xb3c0,0x66d0, 0x5ed7,0x55f6, 0x0c8c,0x7f62
        DCW 0xb1df,0x6564, 0x5e50,0x568a, 0x0afb,0x7f87
        DCW 0xb005,0x63ef, 0x5dc8,0x571e, 0x096b,0x7fa7
        DCW 0xae31,0x6272, 0x5d3e,0x57b1, 0x07d9,0x7fc2
        DCW 0xac65,0x60ec, 0x5cb4,0x5843, 0x0648,0x7fd9
        DCW 0xaaa0,0x5f5e, 0x5c29,0x58d4, 0x04b6,0x7fea
        DCW 0xa8e2,0x5dc8, 0x5b9d,0x5964, 0x0324,0x7ff6
        DCW 0xa72c,0x5c29, 0x5b10,0x59f4, 0x0192,0x7ffe
        DCW 0xa57e,0x5a82, 0x5a82,0x5a82, 0x0000,0x7fff
        DCW 0xa3d7,0x58d4, 0x59f4,0x5b10, 0xfe6e,0x7ffe
        DCW 0xa238,0x571e, 0x5964,0x5b9d, 0xfcdc,0x7ff6
        DCW 0xa0a2,0x5560, 0x58d4,0x5c29, 0xfb4a,0x7fea
        DCW 0x9f14,0x539b, 0x5843,0x5cb4, 0xf9b8,0x7fd9
        DCW 0x9d8e,0x51cf, 0x57b1,0x5d3e, 0xf827,0x7fc2
        DCW 0x9c11,0x4ffb, 0x571e,0x5dc8, 0xf695,0x7fa7
        DCW 0x9a9c,0x4e21, 0x568a,0x5e50, 0xf505,0x7f87
        DCW 0x9930,0x4c40, 0x55f6,0x5ed7, 0xf374,0x7f62
        DCW 0x97ce,0x4a58, 0x5560,0x5f5e, 0xf1e4,0x7f38
        DCW 0x9674,0x486a, 0x54ca,0x5fe4, 0xf055,0x7f0a
        DCW 0x9523,0x4675, 0x5433,0x6068, 0xeec6,0x7ed6
        DCW 0x93dc,0x447b, 0x539b,0x60ec, 0xed38,0x7e9d
        DCW 0x929e,0x427a, 0x5303,0x616f, 0xebab,0x7e60
        DCW 0x9169,0x4074, 0x5269,0x61f1, 0xea1e,0x7e1e
        DCW 0x903e,0x3e68, 0x51cf,0x6272, 0xe892,0x7dd6
        DCW 0x8f1d,0x3c57, 0x5134,0x62f2, 0xe707,0x7d8a
        DCW 0x8e06,0x3a40, 0x5098,0x6371, 0xe57d,0x7d3a
        DCW 0x8cf8,0x3825, 0x4ffb,0x63ef, 0xe3f4,0x7ce4
        DCW 0x8bf5,0x3604, 0x4f5e,0x646c, 0xe26d,0x7c89
        DCW 0x8afb,0x33df, 0x4ec0,0x64e9, 0xe0e6,0x7c2a
        DCW 0x8a0c,0x31b5, 0x4e21,0x6564, 0xdf61,0x7bc6
        DCW 0x8927,0x2f87, 0x4d81,0x65de, 0xdddc,0x7b5d
        DCW 0x884c,0x2d55, 0x4ce1,0x6657, 0xdc59,0x7aef
        DCW 0x877b,0x2b1f, 0x4c40,0x66d0, 0xdad8,0x7a7d
        DCW 0x86b6,0x28e5, 0x4b9e,0x6747, 0xd958,0x7a06
        DCW 0x85fa,0x26a8, 0x4afb,0x67bd, 0xd7d9,0x798a
        DCW 0x8549,0x2467, 0x4a58,0x6832, 0xd65c,0x790a
        DCW 0x84a3,0x2224, 0x49b4,0x68a7, 0xd4e1,0x7885
        DCW 0x8407,0x1fdd, 0x490f,0x691a, 0xd367,0x77fb
        DCW 0x8377,0x1d93, 0x486a,0x698c, 0xd1ef,0x776c
        DCW 0x82f1,0x1b47, 0x47c4,0x69fd, 0xd079,0x76d9
        DCW 0x8276,0x18f9, 0x471d,0x6a6e, 0xcf04,0x7642
        DCW 0x8205,0x16a8, 0x4675,0x6add, 0xcd92,0x75a6
        DCW 0x81a0,0x1455, 0x45cd,0x6b4b, 0xcc21,0x7505
        DCW 0x8146,0x1201, 0x4524,0x6bb8, 0xcab2,0x7460
        DCW 0x80f6,0x0fab, 0x447b,0x6c24, 0xc946,0x73b6
        DCW 0x80b2,0x0d54, 0x43d1,0x6c8f, 0xc7db,0x7308
        DCW 0x8079,0x0afb, 0x4326,0x6cf9, 0xc673,0x7255
        DCW 0x804b,0x08a2, 0x427a,0x6d62, 0xc50d,0x719e
        DCW 0x8027,0x0648, 0x41ce,0x6dca, 0xc3a9,0x70e3
        DCW 0x800f,0x03ed, 0x4121,0x6e31, 0xc248,0x7023
        DCW 0x8002,0x0192, 0x4074,0x6e97, 0xc0e9,0x6f5f
        DCW 0x8001,0xff37, 0x3fc6,0x6efb, 0xbf8c,0x6e97
        DCW 0x800a,0xfcdc, 0x3f17,0x6f5f, 0xbe32,0x6dca
        DCW 0x801e,0xfa81, 0x3e68,0x6fc2, 0xbcda,0x6cf9
        DCW 0x803e,0xf827, 0x3db8,0x7023, 0xbb85,0x6c24
        DCW 0x8068,0xf5cd, 0x3d08,0x7083, 0xba33,0x6b4b
        DCW 0x809e,0xf374, 0x3c57,0x70e3, 0xb8e3,0x6a6e
        DCW 0x80de,0xf11c, 0x3ba5,0x7141, 0xb796,0x698c
        DCW 0x812a,0xeec6, 0x3af3,0x719e, 0xb64c,0x68a7
        DCW 0x8181,0xec71, 0x3a40,0x71fa, 0xb505,0x67bd
        DCW 0x81e2,0xea1e, 0x398d,0x7255, 0xb3c0,0x66d0
        DCW 0x824f,0xe7cd, 0x38d9,0x72af, 0xb27f,0x65de
        DCW 0x82c6,0xe57d, 0x3825,0x7308, 0xb140,0x64e9
        DCW 0x8349,0xe330, 0x3770,0x735f, 0xb005,0x63ef
        DCW 0x83d6,0xe0e6, 0x36ba,0x73b6, 0xaecc,0x62f2
        DCW 0x846e,0xde9e, 0x3604,0x740b, 0xad97,0x61f1
        DCW 0x8511,0xdc59, 0x354e,0x7460, 0xac65,0x60ec
        DCW 0x85be,0xda18, 0x3497,0x74b3, 0xab36,0x5fe4
        DCW 0x8676,0xd7d9, 0x33df,0x7505, 0xaa0a,0x5ed7
        DCW 0x8738,0xd59e, 0x3327,0x7556, 0xa8e2,0x5dc8
        DCW 0x8805,0xd367, 0x326e,0x75a6, 0xa7bd,0x5cb4
        DCW 0x88dd,0xd134, 0x31b5,0x75f4, 0xa69c,0x5b9d
        DCW 0x89be,0xcf04, 0x30fc,0x7642, 0xa57e,0x5a82
        DCW 0x8aaa,0xccd9, 0x3042,0x768e, 0xa463,0x5964
        DCW 0x8ba0,0xcab2, 0x2f87,0x76d9, 0xa34c,0x5843
        DCW 0x8ca1,0xc890, 0x2ecc,0x7723, 0xa238,0x571e
        DCW 0x8dab,0xc673, 0x2e11,0x776c, 0xa129,0x55f6
        DCW 0x8ebf,0xc45b, 0x2d55,0x77b4, 0xa01c,0x54ca
        DCW 0x8fdd,0xc248, 0x2c99,0x77fb, 0x9f14,0x539b
        DCW 0x9105,0xc03a, 0x2bdc,0x7840, 0x9e0f,0x5269
        DCW 0x9236,0xbe32, 0x2b1f,0x7885, 0x9d0e,0x5134
        DCW 0x9371,0xbc2f, 0x2a62,0x78c8, 0x9c11,0x4ffb
        DCW 0x94b5,0xba33, 0x29a4,0x790a, 0x9b17,0x4ec0
        DCW 0x9603,0xb83c, 0x28e5,0x794a, 0x9a22,0x4d81
        DCW 0x9759,0xb64c, 0x2827,0x798a, 0x9930,0x4c40
        DCW 0x98b9,0xb462, 0x2768,0x79c9, 0x9843,0x4afb
        DCW 0x9a22,0xb27f, 0x26a8,0x7a06, 0x9759,0x49b4
        DCW 0x9b94,0xb0a2, 0x25e8,0x7a42, 0x9674,0x486a
        DCW 0x9d0e,0xaecc, 0x2528,0x7a7d, 0x9592,0x471d
        DCW 0x9e91,0xacfd, 0x2467,0x7ab7, 0x94b5,0x45cd
        DCW 0xa01c,0xab36, 0x23a7,0x7aef, 0x93dc,0x447b
        DCW 0xa1b0,0xa976, 0x22e5,0x7b27, 0x9307,0x4326
        DCW 0xa34c,0xa7bd, 0x2224,0x7b5d, 0x9236,0x41ce
        DCW 0xa4f0,0xa60c, 0x2162,0x7b92, 0x9169,0x4074
        DCW 0xa69c,0xa463, 0x209f,0x7bc6, 0x90a1,0x3f17
        DCW 0xa84f,0xa2c2, 0x1fdd,0x7bf9, 0x8fdd,0x3db8
        DCW 0xaa0a,0xa129, 0x1f1a,0x7c2a, 0x8f1d,0x3c57
        DCW 0xabcd,0x9f98, 0x1e57,0x7c5a, 0x8e62,0x3af3
        DCW 0xad97,0x9e0f, 0x1d93,0x7c89, 0x8dab,0x398d
        DCW 0xaf68,0x9c8f, 0x1cd0,0x7cb7, 0x8cf8,0x3825
        DCW 0xb140,0x9b17, 0x1c0c,0x7ce4, 0x8c4a,0x36ba
        DCW 0xb31f,0x99a9, 0x1b47,0x7d0f, 0x8ba0,0x354e
        DCW 0xb505,0x9843, 0x1a83,0x7d3a, 0x8afb,0x33df
        DCW 0xb6f1,0x96e6, 0x19be,0x7d63, 0x8a5a,0x326e
        DCW 0xb8e3,0x9592, 0x18f9,0x7d8a, 0x89be,0x30fc
        DCW 0xbadc,0x9448, 0x1833,0x7db1, 0x8927,0x2f87
        DCW 0xbcda,0x9307, 0x176e,0x7dd6, 0x8894,0x2e11
        DCW 0xbedf,0x91cf, 0x16a8,0x7dfb, 0x8805,0x2c99
        DCW 0xc0e9,0x90a1, 0x15e2,0x7e1e, 0x877b,0x2b1f
        DCW 0xc2f8,0x8f7d, 0x151c,0x7e3f, 0x86f6,0x29a4
        DCW 0xc50d,0x8e62, 0x1455,0x7e60, 0x8676,0x2827
        DCW 0xc727,0x8d51, 0x138f,0x7e7f, 0x85fa,0x26a8
        DCW 0xc946,0x8c4a, 0x12c8,0x7e9d, 0x8583,0x2528
        DCW 0xcb69,0x8b4d, 0x1201,0x7eba, 0x8511,0x23a7
        DCW 0xcd92,0x8a5a, 0x113a,0x7ed6, 0x84a3,0x2224
        DCW 0xcfbe,0x8972, 0x1073,0x7ef0, 0x843a,0x209f
        DCW 0xd1ef,0x8894, 0x0fab,0x7f0a, 0x83d6,0x1f1a
        DCW 0xd424,0x87c0, 0x0ee4,0x7f22, 0x8377,0x1d93
        DCW 0xd65c,0x86f6, 0x0e1c,0x7f38, 0x831c,0x1c0c
        DCW 0xd898,0x8637, 0x0d54,0x7f4e, 0x82c6,0x1a83
        DCW 0xdad8,0x8583, 0x0c8c,0x7f62, 0x8276,0x18f9
        DCW 0xdd1b,0x84d9, 0x0bc4,0x7f75, 0x822a,0x176e
        DCW 0xdf61,0x843a, 0x0afb,0x7f87, 0x81e2,0x15e2
        DCW 0xe1a9,0x83a6, 0x0a33,0x7f98, 0x81a0,0x1455
        DCW 0xe3f4,0x831c, 0x096b,0x7fa7, 0x8163,0x12c8
        DCW 0xe642,0x829d, 0x08a2,0x7fb5, 0x812a,0x113a
        DCW 0xe892,0x822a, 0x07d9,0x7fc2, 0x80f6,0x0fab
        DCW 0xeae4,0x81c1, 0x0711,0x7fce, 0x80c8,0x0e1c
        DCW 0xed38,0x8163, 0x0648,0x7fd9, 0x809e,0x0c8c
        DCW 0xef8d,0x8110, 0x057f,0x7fe2, 0x8079,0x0afb
        DCW 0xf1e4,0x80c8, 0x04b6,0x7fea, 0x8059,0x096b
        DCW 0xf43c,0x808b, 0x03ed,0x7ff1, 0x803e,0x07d9
        DCW 0xf695,0x8059, 0x0324,0x7ff6, 0x8027,0x0648
        DCW 0xf8ef,0x8032, 0x025b,0x7ffa, 0x8016,0x04b6
        DCW 0xfb4a,0x8016, 0x0192,0x7ffe, 0x800a,0x0324
        DCW 0xfda5,0x8006, 0x00c9,0x7fff, 0x8002,0x0192
        ; N=4096 t=2*PI*k/N for k=0,1,2,..,N/4-1
        DCW 0x7fff,0x0000, 0x7fff,0x0000, 0x7fff,0x0000
        DCW 0x7fff,0x0097, 0x7fff,0x0032, 0x7fff,0x0065
        DCW 0x7fff,0x012e, 0x7fff,0x0065, 0x7fff,0x00c9
        DCW 0x7ffd,0x01c4, 0x7fff,0x0097, 0x7fff,0x012e
        DCW 0x7ffa,0x025b, 0x7fff,0x00c9, 0x7ffe,0x0192
        DCW 0x7ff7,0x02f2, 0x7fff,0x00fb, 0x7ffc,0x01f7
        DCW 0x7ff4,0x0389, 0x7fff,0x012e, 0x7ffa,0x025b
        DCW 0x7fef,0x041f, 0x7ffe,0x0160, 0x7ff8,0x02c0
        DCW 0x7fea,0x04b6, 0x7ffe,0x0192, 0x7ff6,0x0324
        DCW 0x7fe4,0x054d, 0x7ffd,0x01c4, 0x7ff4,0x0389
        DCW 0x7fdd,0x05e3, 0x7ffc,0x01f7, 0x7ff1,0x03ed
        DCW 0x7fd6,0x067a, 0x7ffb,0x0229, 0x7fed,0x0452
        DCW 0x7fce,0x0711, 0x7ffa,0x025b, 0x7fea,0x04b6
        DCW 0x7fc5,0x07a7, 0x7ff9,0x028d, 0x7fe6,0x051b
        DCW 0x7fbc,0x083e, 0x7ff8,0x02c0, 0x7fe2,0x057f
        DCW 0x7fb2,0x08d4, 0x7ff7,0x02f2, 0x7fdd,0x05e3
        DCW 0x7fa7,0x096b, 0x7ff6,0x0324, 0x7fd9,0x0648
        DCW 0x7f9c,0x0a01, 0x7ff5,0x0356, 0x7fd3,0x06ac
        DCW 0x7f90,0x0a97, 0x7ff4,0x0389, 0x7fce,0x0711
        DCW 0x7f83,0x0b2d, 0x7ff2,0x03bb, 0x7fc8,0x0775
        DCW 0x7f75,0x0bc4, 0x7ff1,0x03ed, 0x7fc2,0x07d9
        DCW 0x7f67,0x0c5a, 0x7fef,0x041f, 0x7fbc,0x083e
        DCW 0x7f58,0x0cf0, 0x7fed,0x0452, 0x7fb5,0x08a2
        DCW 0x7f49,0x0d86, 0x7fec,0x0484, 0x7fae,0x0906
        DCW 0x7f38,0x0e1c, 0x7fea,0x04b6, 0x7fa7,0x096b
        DCW 0x7f27,0x0eb2, 0x7fe8,0x04e8, 0x7fa0,0x09cf
        DCW 0x7f16,0x0f47, 0x7fe6,0x051b, 0x7f98,0x0a33
        DCW 0x7f03,0x0fdd, 0x7fe4,0x054d, 0x7f90,0x0a97
        DCW 0x7ef0,0x1073, 0x7fe2,0x057f, 0x7f87,0x0afb
        DCW 0x7edd,0x1108, 0x7fe0,0x05b1, 0x7f7e,0x0b60
        DCW 0x7ec8,0x119e, 0x7fdd,0x05e3, 0x7f75,0x0bc4
        DCW 0x7eb3,0x1233, 0x7fdb,0x0616, 0x7f6c,0x0c28
        DCW 0x7e9d,0x12c8, 0x7fd9,0x0648, 0x7f62,0x0c8c
        DCW 0x7e87,0x135d, 0x7fd6,0x067a, 0x7f58,0x0cf0
        DCW 0x7e70,0x13f2, 0x7fd3,0x06ac, 0x7f4e,0x0d54
        DCW 0x7e58,0x1487, 0x7fd1,0x06de, 0x7f43,0x0db8
        DCW 0x7e3f,0x151c, 0x7fce,0x0711, 0x7f38,0x0e1c
        DCW 0x7e26,0x15b1, 0x7fcb,0x0743, 0x7f2d,0x0e80
        DCW 0x7e0c,0x1645, 0x7fc8,0x0775, 0x7f22,0x0ee4
        DCW 0x7df2,0x16da, 0x7fc5,0x07a7, 0x7f16,0x0f47
        DCW 0x7dd6,0x176e, 0x7fc2,0x07d9, 0x7f0a,0x0fab
        DCW 0x7dba,0x1802, 0x7fbf,0x080c, 0x7efd,0x100f
        DCW 0x7d9e,0x1896, 0x7fbc,0x083e, 0x7ef0,0x1073
        DCW 0x7d81,0x192a, 0x7fb9,0x0870, 0x7ee3,0x10d6
        DCW 0x7d63,0x19be, 0x7fb5,0x08a2, 0x7ed6,0x113a
        DCW 0x7d44,0x1a51, 0x7fb2,0x08d4, 0x7ec8,0x119e
        DCW 0x7d25,0x1ae5, 0x7fae,0x0906, 0x7eba,0x1201
        DCW 0x7d05,0x1b78, 0x7fab,0x0938, 0x7eac,0x1265
        DCW 0x7ce4,0x1c0c, 0x7fa7,0x096b, 0x7e9d,0x12c8
        DCW 0x7cc2,0x1c9f, 0x7fa3,0x099d, 0x7e8e,0x132b
        DCW 0x7ca0,0x1d31, 0x7fa0,0x09cf, 0x7e7f,0x138f
        DCW 0x7c7e,0x1dc4, 0x7f9c,0x0a01, 0x7e70,0x13f2
        DCW 0x7c5a,0x1e57, 0x7f98,0x0a33, 0x7e60,0x1455
        DCW 0x7c36,0x1ee9, 0x7f94,0x0a65, 0x7e50,0x14b9
        DCW 0x7c11,0x1f7b, 0x7f90,0x0a97, 0x7e3f,0x151c
        DCW 0x7bec,0x200e, 0x7f8b,0x0ac9, 0x7e2f,0x157f
        DCW 0x7bc6,0x209f, 0x7f87,0x0afb, 0x7e1e,0x15e2
        DCW 0x7b9f,0x2131, 0x7f83,0x0b2d, 0x7e0c,0x1645
        DCW 0x7b78,0x21c3, 0x7f7e,0x0b60, 0x7dfb,0x16a8
        DCW 0x7b50,0x2254, 0x7f7a,0x0b92, 0x7de9,0x170b
        DCW 0x7b27,0x22e5, 0x7f75,0x0bc4, 0x7dd6,0x176e
        DCW 0x7afd,0x2376, 0x7f71,0x0bf6, 0x7dc4,0x17d1
        DCW 0x7ad3,0x2407, 0x7f6c,0x0c28, 0x7db1,0x1833
        DCW 0x7aa8,0x2498, 0x7f67,0x0c5a, 0x7d9e,0x1896
        DCW 0x7a7d,0x2528, 0x7f62,0x0c8c, 0x7d8a,0x18f9
        DCW 0x7a51,0x25b8, 0x7f5d,0x0cbe, 0x7d77,0x195b
        DCW 0x7a24,0x2648, 0x7f58,0x0cf0, 0x7d63,0x19be
        DCW 0x79f7,0x26d8, 0x7f53,0x0d22, 0x7d4e,0x1a20
        DCW 0x79c9,0x2768, 0x7f4e,0x0d54, 0x7d3a,0x1a83
        DCW 0x799a,0x27f7, 0x7f49,0x0d86, 0x7d25,0x1ae5
        DCW 0x796a,0x2886, 0x7f43,0x0db8, 0x7d0f,0x1b47
        DCW 0x793a,0x2915, 0x7f3e,0x0dea, 0x7cfa,0x1ba9
        DCW 0x790a,0x29a4, 0x7f38,0x0e1c, 0x7ce4,0x1c0c
        DCW 0x78d8,0x2a32, 0x7f33,0x0e4e, 0x7cce,0x1c6e
        DCW 0x78a6,0x2ac1, 0x7f2d,0x0e80, 0x7cb7,0x1cd0
        DCW 0x7874,0x2b4f, 0x7f27,0x0eb2, 0x7ca0,0x1d31
        DCW 0x7840,0x2bdc, 0x7f22,0x0ee4, 0x7c89,0x1d93
        DCW 0x780c,0x2c6a, 0x7f1c,0x0f15, 0x7c72,0x1df5
        DCW 0x77d8,0x2cf7, 0x7f16,0x0f47, 0x7c5a,0x1e57
        DCW 0x77a2,0x2d84, 0x7f10,0x0f79, 0x7c42,0x1eb8
        DCW 0x776c,0x2e11, 0x7f0a,0x0fab, 0x7c2a,0x1f1a
        DCW 0x7736,0x2e9e, 0x7f03,0x0fdd, 0x7c11,0x1f7b
        DCW 0x76fe,0x2f2a, 0x7efd,0x100f, 0x7bf9,0x1fdd
        DCW 0x76c7,0x2fb6, 0x7ef7,0x1041, 0x7bdf,0x203e
        DCW 0x768e,0x3042, 0x7ef0,0x1073, 0x7bc6,0x209f
        DCW 0x7655,0x30cd, 0x7eea,0x10a4, 0x7bac,0x2101
        DCW 0x761b,0x3159, 0x7ee3,0x10d6, 0x7b92,0x2162
        DCW 0x75e1,0x31e4, 0x7edd,0x1108, 0x7b78,0x21c3
        DCW 0x75a6,0x326e, 0x7ed6,0x113a, 0x7b5d,0x2224
        DCW 0x756a,0x32f9, 0x7ecf,0x116c, 0x7b42,0x2284
        DCW 0x752d,0x3383, 0x7ec8,0x119e, 0x7b27,0x22e5
        DCW 0x74f0,0x340d, 0x7ec1,0x11cf, 0x7b0b,0x2346
        DCW 0x74b3,0x3497, 0x7eba,0x1201, 0x7aef,0x23a7
        DCW 0x7475,0x3520, 0x7eb3,0x1233, 0x7ad3,0x2407
        DCW 0x7436,0x35a9, 0x7eac,0x1265, 0x7ab7,0x2467
        DCW 0x73f6,0x3632, 0x7ea5,0x1296, 0x7a9a,0x24c8
        DCW 0x73b6,0x36ba, 0x7e9d,0x12c8, 0x7a7d,0x2528
        DCW 0x7375,0x3742, 0x7e96,0x12fa, 0x7a60,0x2588
        DCW 0x7334,0x37ca, 0x7e8e,0x132b, 0x7a42,0x25e8
        DCW 0x72f2,0x3852, 0x7e87,0x135d, 0x7a24,0x2648
        DCW 0x72af,0x38d9, 0x7e7f,0x138f, 0x7a06,0x26a8
        DCW 0x726c,0x3960, 0x7e78,0x13c1, 0x79e7,0x2708
        DCW 0x7228,0x39e7, 0x7e70,0x13f2, 0x79c9,0x2768
        DCW 0x71e3,0x3a6d, 0x7e68,0x1424, 0x79aa,0x27c7
        DCW 0x719e,0x3af3, 0x7e60,0x1455, 0x798a,0x2827
        DCW 0x7158,0x3b79, 0x7e58,0x1487, 0x796a,0x2886
        DCW 0x7112,0x3bfe, 0x7e50,0x14b9, 0x794a,0x28e5
        DCW 0x70cb,0x3c83, 0x7e48,0x14ea, 0x792a,0x2945
        DCW 0x7083,0x3d08, 0x7e3f,0x151c, 0x790a,0x29a4
        DCW 0x703b,0x3d8c, 0x7e37,0x154d, 0x78e9,0x2a03
        DCW 0x6ff2,0x3e10, 0x7e2f,0x157f, 0x78c8,0x2a62
        DCW 0x6fa9,0x3e94, 0x7e26,0x15b1, 0x78a6,0x2ac1
        DCW 0x6f5f,0x3f17, 0x7e1e,0x15e2, 0x7885,0x2b1f
        DCW 0x6f14,0x3f9a, 0x7e15,0x1614, 0x7863,0x2b7e
        DCW 0x6ec9,0x401d, 0x7e0c,0x1645, 0x7840,0x2bdc
        DCW 0x6e7d,0x409f, 0x7e03,0x1677, 0x781e,0x2c3b
        DCW 0x6e31,0x4121, 0x7dfb,0x16a8, 0x77fb,0x2c99
        DCW 0x6de4,0x41a3, 0x7df2,0x16da, 0x77d8,0x2cf7
        DCW 0x6d96,0x4224, 0x7de9,0x170b, 0x77b4,0x2d55
        DCW 0x6d48,0x42a5, 0x7de0,0x173c, 0x7790,0x2db3
        DCW 0x6cf9,0x4326, 0x7dd6,0x176e, 0x776c,0x2e11
        DCW 0x6caa,0x43a6, 0x7dcd,0x179f, 0x7748,0x2e6f
        DCW 0x6c5a,0x4426, 0x7dc4,0x17d1, 0x7723,0x2ecc
        DCW 0x6c09,0x44a5, 0x7dba,0x1802, 0x76fe,0x2f2a
        DCW 0x6bb8,0x4524, 0x7db1,0x1833, 0x76d9,0x2f87
        DCW 0x6b66,0x45a3, 0x7da7,0x1865, 0x76b4,0x2fe5
        DCW 0x6b14,0x4621, 0x7d9e,0x1896, 0x768e,0x3042
        DCW 0x6ac1,0x469f, 0x7d94,0x18c7, 0x7668,0x309f
        DCW 0x6a6e,0x471d, 0x7d8a,0x18f9, 0x7642,0x30fc
        DCW 0x6a1a,0x479a, 0x7d81,0x192a, 0x761b,0x3159
        DCW 0x69c5,0x4817, 0x7d77,0x195b, 0x75f4,0x31b5
        DCW 0x6970,0x4893, 0x7d6d,0x198d, 0x75cd,0x3212
        DCW 0x691a,0x490f, 0x7d63,0x19be, 0x75a6,0x326e
        DCW 0x68c4,0x498b, 0x7d58,0x19ef, 0x757e,0x32cb
        DCW 0x686d,0x4a06, 0x7d4e,0x1a20, 0x7556,0x3327
        DCW 0x6815,0x4a81, 0x7d44,0x1a51, 0x752d,0x3383
        DCW 0x67bd,0x4afb, 0x7d3a,0x1a83, 0x7505,0x33df
        DCW 0x6764,0x4b75, 0x7d2f,0x1ab4, 0x74dc,0x343b
        DCW 0x670b,0x4bef, 0x7d25,0x1ae5, 0x74b3,0x3497
        DCW 0x66b2,0x4c68, 0x7d1a,0x1b16, 0x7489,0x34f2
        DCW 0x6657,0x4ce1, 0x7d0f,0x1b47, 0x7460,0x354e
        DCW 0x65fc,0x4d59, 0x7d05,0x1b78, 0x7436,0x35a9
        DCW 0x65a1,0x4dd1, 0x7cfa,0x1ba9, 0x740b,0x3604
        DCW 0x6545,0x4e49, 0x7cef,0x1bda, 0x73e1,0x365f
        DCW 0x64e9,0x4ec0, 0x7ce4,0x1c0c, 0x73b6,0x36ba
        DCW 0x648b,0x4f37, 0x7cd9,0x1c3d, 0x738b,0x3715
        DCW 0x642e,0x4fad, 0x7cce,0x1c6e, 0x735f,0x3770
        DCW 0x63d0,0x5023, 0x7cc2,0x1c9f, 0x7334,0x37ca
        DCW 0x6371,0x5098, 0x7cb7,0x1cd0, 0x7308,0x3825
        DCW 0x6312,0x510d, 0x7cac,0x1d01, 0x72dc,0x387f
        DCW 0x62b2,0x5181, 0x7ca0,0x1d31, 0x72af,0x38d9
        DCW 0x6252,0x51f5, 0x7c95,0x1d62, 0x7282,0x3933
        DCW 0x61f1,0x5269, 0x7c89,0x1d93, 0x7255,0x398d
        DCW 0x6190,0x52dc, 0x7c7e,0x1dc4, 0x7228,0x39e7
        DCW 0x612e,0x534f, 0x7c72,0x1df5, 0x71fa,0x3a40
        DCW 0x60cb,0x53c1, 0x7c66,0x1e26, 0x71cc,0x3a9a
        DCW 0x6068,0x5433, 0x7c5a,0x1e57, 0x719e,0x3af3
        DCW 0x6005,0x54a4, 0x7c4e,0x1e88, 0x7170,0x3b4c
        DCW 0x5fa1,0x5515, 0x7c42,0x1eb8, 0x7141,0x3ba5
        DCW 0x5f3c,0x5586, 0x7c36,0x1ee9, 0x7112,0x3bfe
        DCW 0x5ed7,0x55f6, 0x7c2a,0x1f1a, 0x70e3,0x3c57
        DCW 0x5e72,0x5665, 0x7c1e,0x1f4b, 0x70b3,0x3caf
        DCW 0x5e0c,0x56d4, 0x7c11,0x1f7b, 0x7083,0x3d08
        DCW 0x5da5,0x5743, 0x7c05,0x1fac, 0x7053,0x3d60
        DCW 0x5d3e,0x57b1, 0x7bf9,0x1fdd, 0x7023,0x3db8
        DCW 0x5cd7,0x581e, 0x7bec,0x200e, 0x6ff2,0x3e10
        DCW 0x5c6f,0x588c, 0x7bdf,0x203e, 0x6fc2,0x3e68
        DCW 0x5c06,0x58f8, 0x7bd3,0x206f, 0x6f90,0x3ec0
        DCW 0x5b9d,0x5964, 0x7bc6,0x209f, 0x6f5f,0x3f17
        DCW 0x5b34,0x59d0, 0x7bb9,0x20d0, 0x6f2d,0x3f6f
        DCW 0x5ac9,0x5a3b, 0x7bac,0x2101, 0x6efb,0x3fc6
        DCW 0x5a5f,0x5aa6, 0x7b9f,0x2131, 0x6ec9,0x401d
        DCW 0x59f4,0x5b10, 0x7b92,0x2162, 0x6e97,0x4074
        DCW 0x5988,0x5b7a, 0x7b85,0x2192, 0x6e64,0x40cb
        DCW 0x591c,0x5be3, 0x7b78,0x21c3, 0x6e31,0x4121
        DCW 0x58b0,0x5c4c, 0x7b6a,0x21f3, 0x6dfe,0x4178
        DCW 0x5843,0x5cb4, 0x7b5d,0x2224, 0x6dca,0x41ce
        DCW 0x57d5,0x5d1c, 0x7b50,0x2254, 0x6d96,0x4224
        DCW 0x5767,0x5d83, 0x7b42,0x2284, 0x6d62,0x427a
        DCW 0x56f9,0x5dea, 0x7b34,0x22b5, 0x6d2e,0x42d0
        DCW 0x568a,0x5e50, 0x7b27,0x22e5, 0x6cf9,0x4326
        DCW 0x561b,0x5eb6, 0x7b19,0x2316, 0x6cc4,0x437b
        DCW 0x55ab,0x5f1b, 0x7b0b,0x2346, 0x6c8f,0x43d1
        DCW 0x553b,0x5f80, 0x7afd,0x2376, 0x6c5a,0x4426
        DCW 0x54ca,0x5fe4, 0x7aef,0x23a7, 0x6c24,0x447b
        DCW 0x5459,0x6047, 0x7ae1,0x23d7, 0x6bee,0x44d0
        DCW 0x53e7,0x60aa, 0x7ad3,0x2407, 0x6bb8,0x4524
        DCW 0x5375,0x610d, 0x7ac5,0x2437, 0x6b82,0x4579
        DCW 0x5303,0x616f, 0x7ab7,0x2467, 0x6b4b,0x45cd
        DCW 0x5290,0x61d1, 0x7aa8,0x2498, 0x6b14,0x4621
        DCW 0x521c,0x6232, 0x7a9a,0x24c8, 0x6add,0x4675
        DCW 0x51a8,0x6292, 0x7a8c,0x24f8, 0x6aa5,0x46c9
        DCW 0x5134,0x62f2, 0x7a7d,0x2528, 0x6a6e,0x471d
        DCW 0x50bf,0x6351, 0x7a6e,0x2558, 0x6a36,0x4770
        DCW 0x504a,0x63b0, 0x7a60,0x2588, 0x69fd,0x47c4
        DCW 0x4fd4,0x640f, 0x7a51,0x25b8, 0x69c5,0x4817
        DCW 0x4f5e,0x646c, 0x7a42,0x25e8, 0x698c,0x486a
        DCW 0x4ee8,0x64ca, 0x7a33,0x2618, 0x6953,0x48bd
        DCW 0x4e71,0x6526, 0x7a24,0x2648, 0x691a,0x490f
        DCW 0x4df9,0x6582, 0x7a15,0x2678, 0x68e0,0x4962
        DCW 0x4d81,0x65de, 0x7a06,0x26a8, 0x68a7,0x49b4
        DCW 0x4d09,0x6639, 0x79f7,0x26d8, 0x686d,0x4a06
        DCW 0x4c91,0x6693, 0x79e7,0x2708, 0x6832,0x4a58
        DCW 0x4c17,0x66ed, 0x79d8,0x2738, 0x67f8,0x4aaa
        DCW 0x4b9e,0x6747, 0x79c9,0x2768, 0x67bd,0x4afb
        DCW 0x4b24,0x67a0, 0x79b9,0x2797, 0x6782,0x4b4d
        DCW 0x4aaa,0x67f8, 0x79aa,0x27c7, 0x6747,0x4b9e
        DCW 0x4a2f,0x6850, 0x799a,0x27f7, 0x670b,0x4bef
        DCW 0x49b4,0x68a7, 0x798a,0x2827, 0x66d0,0x4c40
        DCW 0x4939,0x68fd, 0x797a,0x2856, 0x6693,0x4c91
        DCW 0x48bd,0x6953, 0x796a,0x2886, 0x6657,0x4ce1
        DCW 0x4840,0x69a9, 0x795b,0x28b6, 0x661b,0x4d31
        DCW 0x47c4,0x69fd, 0x794a,0x28e5, 0x65de,0x4d81
        DCW 0x4747,0x6a52, 0x793a,0x2915, 0x65a1,0x4dd1
        DCW 0x46c9,0x6aa5, 0x792a,0x2945, 0x6564,0x4e21
        DCW 0x464b,0x6af8, 0x791a,0x2974, 0x6526,0x4e71
        DCW 0x45cd,0x6b4b, 0x790a,0x29a4, 0x64e9,0x4ec0
        DCW 0x454f,0x6b9d, 0x78f9,0x29d3, 0x64ab,0x4f0f
        DCW 0x44d0,0x6bee, 0x78e9,0x2a03, 0x646c,0x4f5e
        DCW 0x4450,0x6c3f, 0x78d8,0x2a32, 0x642e,0x4fad
        DCW 0x43d1,0x6c8f, 0x78c8,0x2a62, 0x63ef,0x4ffb
        DCW 0x4351,0x6cdf, 0x78b7,0x2a91, 0x63b0,0x504a
        DCW 0x42d0,0x6d2e, 0x78a6,0x2ac1, 0x6371,0x5098
        DCW 0x424f,0x6d7c, 0x7895,0x2af0, 0x6332,0x50e6
        DCW 0x41ce,0x6dca, 0x7885,0x2b1f, 0x62f2,0x5134
        DCW 0x414d,0x6e17, 0x7874,0x2b4f, 0x62b2,0x5181
        DCW 0x40cb,0x6e64, 0x7863,0x2b7e, 0x6272,0x51cf
        DCW 0x4048,0x6eb0, 0x7851,0x2bad, 0x6232,0x521c
        DCW 0x3fc6,0x6efb, 0x7840,0x2bdc, 0x61f1,0x5269
        DCW 0x3f43,0x6f46, 0x782f,0x2c0c, 0x61b0,0x52b6
        DCW 0x3ec0,0x6f90, 0x781e,0x2c3b, 0x616f,0x5303
        DCW 0x3e3c,0x6fda, 0x780c,0x2c6a, 0x612e,0x534f
        DCW 0x3db8,0x7023, 0x77fb,0x2c99, 0x60ec,0x539b
        DCW 0x3d34,0x706b, 0x77e9,0x2cc8, 0x60aa,0x53e7
        DCW 0x3caf,0x70b3, 0x77d8,0x2cf7, 0x6068,0x5433
        DCW 0x3c2a,0x70fa, 0x77c6,0x2d26, 0x6026,0x547f
        DCW 0x3ba5,0x7141, 0x77b4,0x2d55, 0x5fe4,0x54ca
        DCW 0x3b20,0x7187, 0x77a2,0x2d84, 0x5fa1,0x5515
        DCW 0x3a9a,0x71cc, 0x7790,0x2db3, 0x5f5e,0x5560
        DCW 0x3a13,0x7211, 0x777e,0x2de2, 0x5f1b,0x55ab
        DCW 0x398d,0x7255, 0x776c,0x2e11, 0x5ed7,0x55f6
        DCW 0x3906,0x7299, 0x775a,0x2e40, 0x5e94,0x5640
        DCW 0x387f,0x72dc, 0x7748,0x2e6f, 0x5e50,0x568a
        DCW 0x37f7,0x731e, 0x7736,0x2e9e, 0x5e0c,0x56d4
        DCW 0x3770,0x735f, 0x7723,0x2ecc, 0x5dc8,0x571e
        DCW 0x36e8,0x73a0, 0x7711,0x2efb, 0x5d83,0x5767
        DCW 0x365f,0x73e1, 0x76fe,0x2f2a, 0x5d3e,0x57b1
        DCW 0x35d7,0x7421, 0x76ec,0x2f59, 0x5cf9,0x57fa
        DCW 0x354e,0x7460, 0x76d9,0x2f87, 0x5cb4,0x5843
        DCW 0x34c4,0x749e, 0x76c7,0x2fb6, 0x5c6f,0x588c
        DCW 0x343b,0x74dc, 0x76b4,0x2fe5, 0x5c29,0x58d4
        DCW 0x33b1,0x7519, 0x76a1,0x3013, 0x5be3,0x591c
        DCW 0x3327,0x7556, 0x768e,0x3042, 0x5b9d,0x5964
        DCW 0x329d,0x7592, 0x767b,0x3070, 0x5b57,0x59ac
        DCW 0x3212,0x75cd, 0x7668,0x309f, 0x5b10,0x59f4
        DCW 0x3187,0x7608, 0x7655,0x30cd, 0x5ac9,0x5a3b
        DCW 0x30fc,0x7642, 0x7642,0x30fc, 0x5a82,0x5a82
        DCW 0x3070,0x767b, 0x762e,0x312a, 0x5a3b,0x5ac9
        DCW 0x2fe5,0x76b4, 0x761b,0x3159, 0x59f4,0x5b10
        DCW 0x2f59,0x76ec, 0x7608,0x3187, 0x59ac,0x5b57
        DCW 0x2ecc,0x7723, 0x75f4,0x31b5, 0x5964,0x5b9d
        DCW 0x2e40,0x775a, 0x75e1,0x31e4, 0x591c,0x5be3
        DCW 0x2db3,0x7790, 0x75cd,0x3212, 0x58d4,0x5c29
        DCW 0x2d26,0x77c6, 0x75b9,0x3240, 0x588c,0x5c6f
        DCW 0x2c99,0x77fb, 0x75a6,0x326e, 0x5843,0x5cb4
        DCW 0x2c0c,0x782f, 0x7592,0x329d, 0x57fa,0x5cf9
        DCW 0x2b7e,0x7863, 0x757e,0x32cb, 0x57b1,0x5d3e
        DCW 0x2af0,0x7895, 0x756a,0x32f9, 0x5767,0x5d83
        DCW 0x2a62,0x78c8, 0x7556,0x3327, 0x571e,0x5dc8
        DCW 0x29d3,0x78f9, 0x7542,0x3355, 0x56d4,0x5e0c
        DCW 0x2945,0x792a, 0x752d,0x3383, 0x568a,0x5e50
        DCW 0x28b6,0x795b, 0x7519,0x33b1, 0x5640,0x5e94
        DCW 0x2827,0x798a, 0x7505,0x33df, 0x55f6,0x5ed7
        DCW 0x2797,0x79b9, 0x74f0,0x340d, 0x55ab,0x5f1b
        DCW 0x2708,0x79e7, 0x74dc,0x343b, 0x5560,0x5f5e
        DCW 0x2678,0x7a15, 0x74c7,0x3469, 0x5515,0x5fa1
        DCW 0x25e8,0x7a42, 0x74b3,0x3497, 0x54ca,0x5fe4
        DCW 0x2558,0x7a6e, 0x749e,0x34c4, 0x547f,0x6026
        DCW 0x24c8,0x7a9a, 0x7489,0x34f2, 0x5433,0x6068
        DCW 0x2437,0x7ac5, 0x7475,0x3520, 0x53e7,0x60aa
        DCW 0x23a7,0x7aef, 0x7460,0x354e, 0x539b,0x60ec
        DCW 0x2316,0x7b19, 0x744b,0x357b, 0x534f,0x612e
        DCW 0x2284,0x7b42, 0x7436,0x35a9, 0x5303,0x616f
        DCW 0x21f3,0x7b6a, 0x7421,0x35d7, 0x52b6,0x61b0
        DCW 0x2162,0x7b92, 0x740b,0x3604, 0x5269,0x61f1
        DCW 0x20d0,0x7bb9, 0x73f6,0x3632, 0x521c,0x6232
        DCW 0x203e,0x7bdf, 0x73e1,0x365f, 0x51cf,0x6272
        DCW 0x1fac,0x7c05, 0x73cb,0x368d, 0x5181,0x62b2
        DCW 0x1f1a,0x7c2a, 0x73b6,0x36ba, 0x5134,0x62f2
        DCW 0x1e88,0x7c4e, 0x73a0,0x36e8, 0x50e6,0x6332
        DCW 0x1df5,0x7c72, 0x738b,0x3715, 0x5098,0x6371
        DCW 0x1d62,0x7c95, 0x7375,0x3742, 0x504a,0x63b0
        DCW 0x1cd0,0x7cb7, 0x735f,0x3770, 0x4ffb,0x63ef
        DCW 0x1c3d,0x7cd9, 0x734a,0x379d, 0x4fad,0x642e
        DCW 0x1ba9,0x7cfa, 0x7334,0x37ca, 0x4f5e,0x646c
        DCW 0x1b16,0x7d1a, 0x731e,0x37f7, 0x4f0f,0x64ab
        DCW 0x1a83,0x7d3a, 0x7308,0x3825, 0x4ec0,0x64e9
        DCW 0x19ef,0x7d58, 0x72f2,0x3852, 0x4e71,0x6526
        DCW 0x195b,0x7d77, 0x72dc,0x387f, 0x4e21,0x6564
        DCW 0x18c7,0x7d94, 0x72c5,0x38ac, 0x4dd1,0x65a1
        DCW 0x1833,0x7db1, 0x72af,0x38d9, 0x4d81,0x65de
        DCW 0x179f,0x7dcd, 0x7299,0x3906, 0x4d31,0x661b
        DCW 0x170b,0x7de9, 0x7282,0x3933, 0x4ce1,0x6657
        DCW 0x1677,0x7e03, 0x726c,0x3960, 0x4c91,0x6693
        DCW 0x15e2,0x7e1e, 0x7255,0x398d, 0x4c40,0x66d0
        DCW 0x154d,0x7e37, 0x723f,0x39ba, 0x4bef,0x670b
        DCW 0x14b9,0x7e50, 0x7228,0x39e7, 0x4b9e,0x6747
        DCW 0x1424,0x7e68, 0x7211,0x3a13, 0x4b4d,0x6782
        DCW 0x138f,0x7e7f, 0x71fa,0x3a40, 0x4afb,0x67bd
        DCW 0x12fa,0x7e96, 0x71e3,0x3a6d, 0x4aaa,0x67f8
        DCW 0x1265,0x7eac, 0x71cc,0x3a9a, 0x4a58,0x6832
        DCW 0x11cf,0x7ec1, 0x71b5,0x3ac6, 0x4a06,0x686d
        DCW 0x113a,0x7ed6, 0x719e,0x3af3, 0x49b4,0x68a7
        DCW 0x10a4,0x7eea, 0x7187,0x3b20, 0x4962,0x68e0
        DCW 0x100f,0x7efd, 0x7170,0x3b4c, 0x490f,0x691a
        DCW 0x0f79,0x7f10, 0x7158,0x3b79, 0x48bd,0x6953
        DCW 0x0ee4,0x7f22, 0x7141,0x3ba5, 0x486a,0x698c
        DCW 0x0e4e,0x7f33, 0x712a,0x3bd2, 0x4817,0x69c5
        DCW 0x0db8,0x7f43, 0x7112,0x3bfe, 0x47c4,0x69fd
        DCW 0x0d22,0x7f53, 0x70fa,0x3c2a, 0x4770,0x6a36
        DCW 0x0c8c,0x7f62, 0x70e3,0x3c57, 0x471d,0x6a6e
        DCW 0x0bf6,0x7f71, 0x70cb,0x3c83, 0x46c9,0x6aa5
        DCW 0x0b60,0x7f7e, 0x70b3,0x3caf, 0x4675,0x6add
        DCW 0x0ac9,0x7f8b, 0x709b,0x3cdc, 0x4621,0x6b14
        DCW 0x0a33,0x7f98, 0x7083,0x3d08, 0x45cd,0x6b4b
        DCW 0x099d,0x7fa3, 0x706b,0x3d34, 0x4579,0x6b82
        DCW 0x0906,0x7fae, 0x7053,0x3d60, 0x4524,0x6bb8
        DCW 0x0870,0x7fb9, 0x703b,0x3d8c, 0x44d0,0x6bee
        DCW 0x07d9,0x7fc2, 0x7023,0x3db8, 0x447b,0x6c24
        DCW 0x0743,0x7fcb, 0x700b,0x3de4, 0x4426,0x6c5a
        DCW 0x06ac,0x7fd3, 0x6ff2,0x3e10, 0x43d1,0x6c8f
        DCW 0x0616,0x7fdb, 0x6fda,0x3e3c, 0x437b,0x6cc4
        DCW 0x057f,0x7fe2, 0x6fc2,0x3e68, 0x4326,0x6cf9
        DCW 0x04e8,0x7fe8, 0x6fa9,0x3e94, 0x42d0,0x6d2e
        DCW 0x0452,0x7fed, 0x6f90,0x3ec0, 0x427a,0x6d62
        DCW 0x03bb,0x7ff2, 0x6f78,0x3eec, 0x4224,0x6d96
        DCW 0x0324,0x7ff6, 0x6f5f,0x3f17, 0x41ce,0x6dca
        DCW 0x028d,0x7ff9, 0x6f46,0x3f43, 0x4178,0x6dfe
        DCW 0x01f7,0x7ffc, 0x6f2d,0x3f6f, 0x4121,0x6e31
        DCW 0x0160,0x7ffe, 0x6f14,0x3f9a, 0x40cb,0x6e64
        DCW 0x00c9,0x7fff, 0x6efb,0x3fc6, 0x4074,0x6e97
        DCW 0x0032,0x7fff, 0x6ee2,0x3ff1, 0x401d,0x6ec9
        DCW 0xff9b,0x7fff, 0x6ec9,0x401d, 0x3fc6,0x6efb
        DCW 0xff05,0x7fff, 0x6eb0,0x4048, 0x3f6f,0x6f2d
        DCW 0xfe6e,0x7ffe, 0x6e97,0x4074, 0x3f17,0x6f5f
        DCW 0xfdd7,0x7ffb, 0x6e7d,0x409f, 0x3ec0,0x6f90
        DCW 0xfd40,0x7ff8, 0x6e64,0x40cb, 0x3e68,0x6fc2
        DCW 0xfcaa,0x7ff5, 0x6e4a,0x40f6, 0x3e10,0x6ff2
        DCW 0xfc13,0x7ff1, 0x6e31,0x4121, 0x3db8,0x7023
        DCW 0xfb7c,0x7fec, 0x6e17,0x414d, 0x3d60,0x7053
        DCW 0xfae5,0x7fe6, 0x6dfe,0x4178, 0x3d08,0x7083
        DCW 0xfa4f,0x7fe0, 0x6de4,0x41a3, 0x3caf,0x70b3
        DCW 0xf9b8,0x7fd9, 0x6dca,0x41ce, 0x3c57,0x70e3
        DCW 0xf922,0x7fd1, 0x6db0,0x41f9, 0x3bfe,0x7112
        DCW 0xf88b,0x7fc8, 0x6d96,0x4224, 0x3ba5,0x7141
        DCW 0xf7f4,0x7fbf, 0x6d7c,0x424f, 0x3b4c,0x7170
        DCW 0xf75e,0x7fb5, 0x6d62,0x427a, 0x3af3,0x719e
        DCW 0xf6c8,0x7fab, 0x6d48,0x42a5, 0x3a9a,0x71cc
        DCW 0xf631,0x7fa0, 0x6d2e,0x42d0, 0x3a40,0x71fa
        DCW 0xf59b,0x7f94, 0x6d14,0x42fb, 0x39e7,0x7228
        DCW 0xf505,0x7f87, 0x6cf9,0x4326, 0x398d,0x7255
        DCW 0xf46e,0x7f7a, 0x6cdf,0x4351, 0x3933,0x7282
        DCW 0xf3d8,0x7f6c, 0x6cc4,0x437b, 0x38d9,0x72af
        DCW 0xf342,0x7f5d, 0x6caa,0x43a6, 0x387f,0x72dc
        DCW 0xf2ac,0x7f4e, 0x6c8f,0x43d1, 0x3825,0x7308
        DCW 0xf216,0x7f3e, 0x6c75,0x43fb, 0x37ca,0x7334
        DCW 0xf180,0x7f2d, 0x6c5a,0x4426, 0x3770,0x735f
        DCW 0xf0eb,0x7f1c, 0x6c3f,0x4450, 0x3715,0x738b
        DCW 0xf055,0x7f0a, 0x6c24,0x447b, 0x36ba,0x73b6
        DCW 0xefbf,0x7ef7, 0x6c09,0x44a5, 0x365f,0x73e1
        DCW 0xef2a,0x7ee3, 0x6bee,0x44d0, 0x3604,0x740b
        DCW 0xee94,0x7ecf, 0x6bd3,0x44fa, 0x35a9,0x7436
        DCW 0xedff,0x7eba, 0x6bb8,0x4524, 0x354e,0x7460
        DCW 0xed6a,0x7ea5, 0x6b9d,0x454f, 0x34f2,0x7489
        DCW 0xecd5,0x7e8e, 0x6b82,0x4579, 0x3497,0x74b3
        DCW 0xec3f,0x7e78, 0x6b66,0x45a3, 0x343b,0x74dc
        DCW 0xebab,0x7e60, 0x6b4b,0x45cd, 0x33df,0x7505
        DCW 0xeb16,0x7e48, 0x6b30,0x45f7, 0x3383,0x752d
        DCW 0xea81,0x7e2f, 0x6b14,0x4621, 0x3327,0x7556
        DCW 0xe9ec,0x7e15, 0x6af8,0x464b, 0x32cb,0x757e
        DCW 0xe958,0x7dfb, 0x6add,0x4675, 0x326e,0x75a6
        DCW 0xe8c4,0x7de0, 0x6ac1,0x469f, 0x3212,0x75cd
        DCW 0xe82f,0x7dc4, 0x6aa5,0x46c9, 0x31b5,0x75f4
        DCW 0xe79b,0x7da7, 0x6a89,0x46f3, 0x3159,0x761b
        DCW 0xe707,0x7d8a, 0x6a6e,0x471d, 0x30fc,0x7642
        DCW 0xe673,0x7d6d, 0x6a52,0x4747, 0x309f,0x7668
        DCW 0xe5e0,0x7d4e, 0x6a36,0x4770, 0x3042,0x768e
        DCW 0xe54c,0x7d2f, 0x6a1a,0x479a, 0x2fe5,0x76b4
        DCW 0xe4b9,0x7d0f, 0x69fd,0x47c4, 0x2f87,0x76d9
        DCW 0xe426,0x7cef, 0x69e1,0x47ed, 0x2f2a,0x76fe
        DCW 0xe392,0x7cce, 0x69c5,0x4817, 0x2ecc,0x7723
        DCW 0xe2ff,0x7cac, 0x69a9,0x4840, 0x2e6f,0x7748
        DCW 0xe26d,0x7c89, 0x698c,0x486a, 0x2e11,0x776c
        DCW 0xe1da,0x7c66, 0x6970,0x4893, 0x2db3,0x7790
        DCW 0xe148,0x7c42, 0x6953,0x48bd, 0x2d55,0x77b4
        DCW 0xe0b5,0x7c1e, 0x6937,0x48e6, 0x2cf7,0x77d8
        DCW 0xe023,0x7bf9, 0x691a,0x490f, 0x2c99,0x77fb
        DCW 0xdf91,0x7bd3, 0x68fd,0x4939, 0x2c3b,0x781e
        DCW 0xdeff,0x7bac, 0x68e0,0x4962, 0x2bdc,0x7840
        DCW 0xde6e,0x7b85, 0x68c4,0x498b, 0x2b7e,0x7863
        DCW 0xdddc,0x7b5d, 0x68a7,0x49b4, 0x2b1f,0x7885
        DCW 0xdd4b,0x7b34, 0x688a,0x49dd, 0x2ac1,0x78a6
        DCW 0xdcba,0x7b0b, 0x686d,0x4a06, 0x2a62,0x78c8
        DCW 0xdc29,0x7ae1, 0x6850,0x4a2f, 0x2a03,0x78e9
        DCW 0xdb99,0x7ab7, 0x6832,0x4a58, 0x29a4,0x790a
        DCW 0xdb08,0x7a8c, 0x6815,0x4a81, 0x2945,0x792a
        DCW 0xda78,0x7a60, 0x67f8,0x4aaa, 0x28e5,0x794a
        DCW 0xd9e8,0x7a33, 0x67da,0x4ad3, 0x2886,0x796a
        DCW 0xd958,0x7a06, 0x67bd,0x4afb, 0x2827,0x798a
        DCW 0xd8c8,0x79d8, 0x67a0,0x4b24, 0x27c7,0x79aa
        DCW 0xd839,0x79aa, 0x6782,0x4b4d, 0x2768,0x79c9
        DCW 0xd7aa,0x797a, 0x6764,0x4b75, 0x2708,0x79e7
        DCW 0xd71b,0x794a, 0x6747,0x4b9e, 0x26a8,0x7a06
        DCW 0xd68c,0x791a, 0x6729,0x4bc7, 0x2648,0x7a24
        DCW 0xd5fd,0x78e9, 0x670b,0x4bef, 0x25e8,0x7a42
        DCW 0xd56f,0x78b7, 0x66ed,0x4c17, 0x2588,0x7a60
        DCW 0xd4e1,0x7885, 0x66d0,0x4c40, 0x2528,0x7a7d
        DCW 0xd453,0x7851, 0x66b2,0x4c68, 0x24c8,0x7a9a
        DCW 0xd3c5,0x781e, 0x6693,0x4c91, 0x2467,0x7ab7
        DCW 0xd338,0x77e9, 0x6675,0x4cb9, 0x2407,0x7ad3
        DCW 0xd2ab,0x77b4, 0x6657,0x4ce1, 0x23a7,0x7aef
        DCW 0xd21e,0x777e, 0x6639,0x4d09, 0x2346,0x7b0b
        DCW 0xd191,0x7748, 0x661b,0x4d31, 0x22e5,0x7b27
        DCW 0xd105,0x7711, 0x65fc,0x4d59, 0x2284,0x7b42
        DCW 0xd079,0x76d9, 0x65de,0x4d81, 0x2224,0x7b5d
        DCW 0xcfed,0x76a1, 0x65c0,0x4da9, 0x21c3,0x7b78
        DCW 0xcf61,0x7668, 0x65a1,0x4dd1, 0x2162,0x7b92
        DCW 0xced6,0x762e, 0x6582,0x4df9, 0x2101,0x7bac
        DCW 0xce4b,0x75f4, 0x6564,0x4e21, 0x209f,0x7bc6
        DCW 0xcdc0,0x75b9, 0x6545,0x4e49, 0x203e,0x7bdf
        DCW 0xcd35,0x757e, 0x6526,0x4e71, 0x1fdd,0x7bf9
        DCW 0xccab,0x7542, 0x6507,0x4e98, 0x1f7b,0x7c11
        DCW 0xcc21,0x7505, 0x64e9,0x4ec0, 0x1f1a,0x7c2a
        DCW 0xcb97,0x74c7, 0x64ca,0x4ee8, 0x1eb8,0x7c42
        DCW 0xcb0e,0x7489, 0x64ab,0x4f0f, 0x1e57,0x7c5a
        DCW 0xca85,0x744b, 0x648b,0x4f37, 0x1df5,0x7c72
        DCW 0xc9fc,0x740b, 0x646c,0x4f5e, 0x1d93,0x7c89
        DCW 0xc973,0x73cb, 0x644d,0x4f85, 0x1d31,0x7ca0
        DCW 0xc8eb,0x738b, 0x642e,0x4fad, 0x1cd0,0x7cb7
        DCW 0xc863,0x734a, 0x640f,0x4fd4, 0x1c6e,0x7cce
        DCW 0xc7db,0x7308, 0x63ef,0x4ffb, 0x1c0c,0x7ce4
        DCW 0xc754,0x72c5, 0x63d0,0x5023, 0x1ba9,0x7cfa
        DCW 0xc6cd,0x7282, 0x63b0,0x504a, 0x1b47,0x7d0f
        DCW 0xc646,0x723f, 0x6391,0x5071, 0x1ae5,0x7d25
        DCW 0xc5c0,0x71fa, 0x6371,0x5098, 0x1a83,0x7d3a
        DCW 0xc53a,0x71b5, 0x6351,0x50bf, 0x1a20,0x7d4e
        DCW 0xc4b4,0x7170, 0x6332,0x50e6, 0x19be,0x7d63
        DCW 0xc42e,0x712a, 0x6312,0x510d, 0x195b,0x7d77
        DCW 0xc3a9,0x70e3, 0x62f2,0x5134, 0x18f9,0x7d8a
        DCW 0xc324,0x709b, 0x62d2,0x515b, 0x1896,0x7d9e
        DCW 0xc2a0,0x7053, 0x62b2,0x5181, 0x1833,0x7db1
        DCW 0xc21c,0x700b, 0x6292,0x51a8, 0x17d1,0x7dc4
        DCW 0xc198,0x6fc2, 0x6272,0x51cf, 0x176e,0x7dd6
        DCW 0xc114,0x6f78, 0x6252,0x51f5, 0x170b,0x7de9
        DCW 0xc091,0x6f2d, 0x6232,0x521c, 0x16a8,0x7dfb
        DCW 0xc00f,0x6ee2, 0x6211,0x5243, 0x1645,0x7e0c
        DCW 0xbf8c,0x6e97, 0x61f1,0x5269, 0x15e2,0x7e1e
        DCW 0xbf0a,0x6e4a, 0x61d1,0x5290, 0x157f,0x7e2f
        DCW 0xbe88,0x6dfe, 0x61b0,0x52b6, 0x151c,0x7e3f
        DCW 0xbe07,0x6db0, 0x6190,0x52dc, 0x14b9,0x7e50
        DCW 0xbd86,0x6d62, 0x616f,0x5303, 0x1455,0x7e60
        DCW 0xbd05,0x6d14, 0x614e,0x5329, 0x13f2,0x7e70
        DCW 0xbc85,0x6cc4, 0x612e,0x534f, 0x138f,0x7e7f
        DCW 0xbc05,0x6c75, 0x610d,0x5375, 0x132b,0x7e8e
        DCW 0xbb85,0x6c24, 0x60ec,0x539b, 0x12c8,0x7e9d
        DCW 0xbb06,0x6bd3, 0x60cb,0x53c1, 0x1265,0x7eac
        DCW 0xba87,0x6b82, 0x60aa,0x53e7, 0x1201,0x7eba
        DCW 0xba09,0x6b30, 0x6089,0x540d, 0x119e,0x7ec8
        DCW 0xb98b,0x6add, 0x6068,0x5433, 0x113a,0x7ed6
        DCW 0xb90d,0x6a89, 0x6047,0x5459, 0x10d6,0x7ee3
        DCW 0xb890,0x6a36, 0x6026,0x547f, 0x1073,0x7ef0
        DCW 0xb813,0x69e1, 0x6005,0x54a4, 0x100f,0x7efd
        DCW 0xb796,0x698c, 0x5fe4,0x54ca, 0x0fab,0x7f0a
        DCW 0xb71a,0x6937, 0x5fc2,0x54f0, 0x0f47,0x7f16
        DCW 0xb69e,0x68e0, 0x5fa1,0x5515, 0x0ee4,0x7f22
        DCW 0xb623,0x688a, 0x5f80,0x553b, 0x0e80,0x7f2d
        DCW 0xb5a8,0x6832, 0x5f5e,0x5560, 0x0e1c,0x7f38
        DCW 0xb52d,0x67da, 0x5f3c,0x5586, 0x0db8,0x7f43
        DCW 0xb4b3,0x6782, 0x5f1b,0x55ab, 0x0d54,0x7f4e
        DCW 0xb439,0x6729, 0x5ef9,0x55d0, 0x0cf0,0x7f58
        DCW 0xb3c0,0x66d0, 0x5ed7,0x55f6, 0x0c8c,0x7f62
        DCW 0xb347,0x6675, 0x5eb6,0x561b, 0x0c28,0x7f6c
        DCW 0xb2cf,0x661b, 0x5e94,0x5640, 0x0bc4,0x7f75
        DCW 0xb257,0x65c0, 0x5e72,0x5665, 0x0b60,0x7f7e
        DCW 0xb1df,0x6564, 0x5e50,0x568a, 0x0afb,0x7f87
        DCW 0xb168,0x6507, 0x5e2e,0x56af, 0x0a97,0x7f90
        DCW 0xb0f1,0x64ab, 0x5e0c,0x56d4, 0x0a33,0x7f98
        DCW 0xb07b,0x644d, 0x5dea,0x56f9, 0x09cf,0x7fa0
        DCW 0xb005,0x63ef, 0x5dc8,0x571e, 0x096b,0x7fa7
        DCW 0xaf8f,0x6391, 0x5da5,0x5743, 0x0906,0x7fae
        DCW 0xaf1a,0x6332, 0x5d83,0x5767, 0x08a2,0x7fb5
        DCW 0xaea5,0x62d2, 0x5d61,0x578c, 0x083e,0x7fbc
        DCW 0xae31,0x6272, 0x5d3e,0x57b1, 0x07d9,0x7fc2
        DCW 0xadbd,0x6211, 0x5d1c,0x57d5, 0x0775,0x7fc8
        DCW 0xad4a,0x61b0, 0x5cf9,0x57fa, 0x0711,0x7fce
        DCW 0xacd7,0x614e, 0x5cd7,0x581e, 0x06ac,0x7fd3
        DCW 0xac65,0x60ec, 0x5cb4,0x5843, 0x0648,0x7fd9
        DCW 0xabf3,0x6089, 0x5c91,0x5867, 0x05e3,0x7fdd
        DCW 0xab81,0x6026, 0x5c6f,0x588c, 0x057f,0x7fe2
        DCW 0xab10,0x5fc2, 0x5c4c,0x58b0, 0x051b,0x7fe6
        DCW 0xaaa0,0x5f5e, 0x5c29,0x58d4, 0x04b6,0x7fea
        DCW 0xaa30,0x5ef9, 0x5c06,0x58f8, 0x0452,0x7fed
        DCW 0xa9c0,0x5e94, 0x5be3,0x591c, 0x03ed,0x7ff1
        DCW 0xa951,0x5e2e, 0x5bc0,0x5940, 0x0389,0x7ff4
        DCW 0xa8e2,0x5dc8, 0x5b9d,0x5964, 0x0324,0x7ff6
        DCW 0xa874,0x5d61, 0x5b7a,0x5988, 0x02c0,0x7ff8
        DCW 0xa806,0x5cf9, 0x5b57,0x59ac, 0x025b,0x7ffa
        DCW 0xa799,0x5c91, 0x5b34,0x59d0, 0x01f7,0x7ffc
        DCW 0xa72c,0x5c29, 0x5b10,0x59f4, 0x0192,0x7ffe
        DCW 0xa6c0,0x5bc0, 0x5aed,0x5a18, 0x012e,0x7fff
        DCW 0xa654,0x5b57, 0x5ac9,0x5a3b, 0x00c9,0x7fff
        DCW 0xa5e8,0x5aed, 0x5aa6,0x5a5f, 0x0065,0x7fff
        DCW 0xa57e,0x5a82, 0x5a82,0x5a82, 0x0000,0x7fff
        DCW 0xa513,0x5a18, 0x5a5f,0x5aa6, 0xff9b,0x7fff
        DCW 0xa4a9,0x59ac, 0x5a3b,0x5ac9, 0xff37,0x7fff
        DCW 0xa440,0x5940, 0x5a18,0x5aed, 0xfed2,0x7fff
        DCW 0xa3d7,0x58d4, 0x59f4,0x5b10, 0xfe6e,0x7ffe
        DCW 0xa36f,0x5867, 0x59d0,0x5b34, 0xfe09,0x7ffc
        DCW 0xa307,0x57fa, 0x59ac,0x5b57, 0xfda5,0x7ffa
        DCW 0xa29f,0x578c, 0x5988,0x5b7a, 0xfd40,0x7ff8
        DCW 0xa238,0x571e, 0x5964,0x5b9d, 0xfcdc,0x7ff6
        DCW 0xa1d2,0x56af, 0x5940,0x5bc0, 0xfc77,0x7ff4
        DCW 0xa16c,0x5640, 0x591c,0x5be3, 0xfc13,0x7ff1
        DCW 0xa107,0x55d0, 0x58f8,0x5c06, 0xfbae,0x7fed
        DCW 0xa0a2,0x5560, 0x58d4,0x5c29, 0xfb4a,0x7fea
        DCW 0xa03e,0x54f0, 0x58b0,0x5c4c, 0xfae5,0x7fe6
        DCW 0x9fda,0x547f, 0x588c,0x5c6f, 0xfa81,0x7fe2
        DCW 0x9f77,0x540d, 0x5867,0x5c91, 0xfa1d,0x7fdd
        DCW 0x9f14,0x539b, 0x5843,0x5cb4, 0xf9b8,0x7fd9
        DCW 0x9eb2,0x5329, 0x581e,0x5cd7, 0xf954,0x7fd3
        DCW 0x9e50,0x52b6, 0x57fa,0x5cf9, 0xf8ef,0x7fce
        DCW 0x9def,0x5243, 0x57d5,0x5d1c, 0xf88b,0x7fc8
        DCW 0x9d8e,0x51cf, 0x57b1,0x5d3e, 0xf827,0x7fc2
        DCW 0x9d2e,0x515b, 0x578c,0x5d61, 0xf7c2,0x7fbc
        DCW 0x9cce,0x50e6, 0x5767,0x5d83, 0xf75e,0x7fb5
        DCW 0x9c6f,0x5071, 0x5743,0x5da5, 0xf6fa,0x7fae
        DCW 0x9c11,0x4ffb, 0x571e,0x5dc8, 0xf695,0x7fa7
        DCW 0x9bb3,0x4f85, 0x56f9,0x5dea, 0xf631,0x7fa0
        DCW 0x9b55,0x4f0f, 0x56d4,0x5e0c, 0xf5cd,0x7f98
        DCW 0x9af9,0x4e98, 0x56af,0x5e2e, 0xf569,0x7f90
        DCW 0x9a9c,0x4e21, 0x568a,0x5e50, 0xf505,0x7f87
        DCW 0x9a40,0x4da9, 0x5665,0x5e72, 0xf4a0,0x7f7e
        DCW 0x99e5,0x4d31, 0x5640,0x5e94, 0xf43c,0x7f75
        DCW 0x998b,0x4cb9, 0x561b,0x5eb6, 0xf3d8,0x7f6c
        DCW 0x9930,0x4c40, 0x55f6,0x5ed7, 0xf374,0x7f62
        DCW 0x98d7,0x4bc7, 0x55d0,0x5ef9, 0xf310,0x7f58
        DCW 0x987e,0x4b4d, 0x55ab,0x5f1b, 0xf2ac,0x7f4e
        DCW 0x9826,0x4ad3, 0x5586,0x5f3c, 0xf248,0x7f43
        DCW 0x97ce,0x4a58, 0x5560,0x5f5e, 0xf1e4,0x7f38
        DCW 0x9776,0x49dd, 0x553b,0x5f80, 0xf180,0x7f2d
        DCW 0x9720,0x4962, 0x5515,0x5fa1, 0xf11c,0x7f22
        DCW 0x96c9,0x48e6, 0x54f0,0x5fc2, 0xf0b9,0x7f16
        DCW 0x9674,0x486a, 0x54ca,0x5fe4, 0xf055,0x7f0a
        DCW 0x961f,0x47ed, 0x54a4,0x6005, 0xeff1,0x7efd
        DCW 0x95ca,0x4770, 0x547f,0x6026, 0xef8d,0x7ef0
        DCW 0x9577,0x46f3, 0x5459,0x6047, 0xef2a,0x7ee3
        DCW 0x9523,0x4675, 0x5433,0x6068, 0xeec6,0x7ed6
        DCW 0x94d0,0x45f7, 0x540d,0x6089, 0xee62,0x7ec8
        DCW 0x947e,0x4579, 0x53e7,0x60aa, 0xedff,0x7eba
        DCW 0x942d,0x44fa, 0x53c1,0x60cb, 0xed9b,0x7eac
        DCW 0x93dc,0x447b, 0x539b,0x60ec, 0xed38,0x7e9d
        DCW 0x938b,0x43fb, 0x5375,0x610d, 0xecd5,0x7e8e
        DCW 0x933c,0x437b, 0x534f,0x612e, 0xec71,0x7e7f
        DCW 0x92ec,0x42fb, 0x5329,0x614e, 0xec0e,0x7e70
        DCW 0x929e,0x427a, 0x5303,0x616f, 0xebab,0x7e60
        DCW 0x9250,0x41f9, 0x52dc,0x6190, 0xeb47,0x7e50
        DCW 0x9202,0x4178, 0x52b6,0x61b0, 0xeae4,0x7e3f
        DCW 0x91b6,0x40f6, 0x5290,0x61d1, 0xea81,0x7e2f
        DCW 0x9169,0x4074, 0x5269,0x61f1, 0xea1e,0x7e1e
        DCW 0x911e,0x3ff1, 0x5243,0x6211, 0xe9bb,0x7e0c
        DCW 0x90d3,0x3f6f, 0x521c,0x6232, 0xe958,0x7dfb
        DCW 0x9088,0x3eec, 0x51f5,0x6252, 0xe8f5,0x7de9
        DCW 0x903e,0x3e68, 0x51cf,0x6272, 0xe892,0x7dd6
        DCW 0x8ff5,0x3de4, 0x51a8,0x6292, 0xe82f,0x7dc4
        DCW 0x8fad,0x3d60, 0x5181,0x62b2, 0xe7cd,0x7db1
        DCW 0x8f65,0x3cdc, 0x515b,0x62d2, 0xe76a,0x7d9e
        DCW 0x8f1d,0x3c57, 0x5134,0x62f2, 0xe707,0x7d8a
        DCW 0x8ed6,0x3bd2, 0x510d,0x6312, 0xe6a5,0x7d77
        DCW 0x8e90,0x3b4c, 0x50e6,0x6332, 0xe642,0x7d63
        DCW 0x8e4b,0x3ac6, 0x50bf,0x6351, 0xe5e0,0x7d4e
        DCW 0x8e06,0x3a40, 0x5098,0x6371, 0xe57d,0x7d3a
        DCW 0x8dc1,0x39ba, 0x5071,0x6391, 0xe51b,0x7d25
        DCW 0x8d7e,0x3933, 0x504a,0x63b0, 0xe4b9,0x7d0f
        DCW 0x8d3b,0x38ac, 0x5023,0x63d0, 0xe457,0x7cfa
        DCW 0x8cf8,0x3825, 0x4ffb,0x63ef, 0xe3f4,0x7ce4
        DCW 0x8cb6,0x379d, 0x4fd4,0x640f, 0xe392,0x7cce
        DCW 0x8c75,0x3715, 0x4fad,0x642e, 0xe330,0x7cb7
        DCW 0x8c35,0x368d, 0x4f85,0x644d, 0xe2cf,0x7ca0
        DCW 0x8bf5,0x3604, 0x4f5e,0x646c, 0xe26d,0x7c89
        DCW 0x8bb5,0x357b, 0x4f37,0x648b, 0xe20b,0x7c72
        DCW 0x8b77,0x34f2, 0x4f0f,0x64ab, 0xe1a9,0x7c5a
        DCW 0x8b39,0x3469, 0x4ee8,0x64ca, 0xe148,0x7c42
        DCW 0x8afb,0x33df, 0x4ec0,0x64e9, 0xe0e6,0x7c2a
        DCW 0x8abe,0x3355, 0x4e98,0x6507, 0xe085,0x7c11
        DCW 0x8a82,0x32cb, 0x4e71,0x6526, 0xe023,0x7bf9
        DCW 0x8a47,0x3240, 0x4e49,0x6545, 0xdfc2,0x7bdf
        DCW 0x8a0c,0x31b5, 0x4e21,0x6564, 0xdf61,0x7bc6
        DCW 0x89d2,0x312a, 0x4df9,0x6582, 0xdeff,0x7bac
        DCW 0x8998,0x309f, 0x4dd1,0x65a1, 0xde9e,0x7b92
        DCW 0x895f,0x3013, 0x4da9,0x65c0, 0xde3d,0x7b78
        DCW 0x8927,0x2f87, 0x4d81,0x65de, 0xdddc,0x7b5d
        DCW 0x88ef,0x2efb, 0x4d59,0x65fc, 0xdd7c,0x7b42
        DCW 0x88b8,0x2e6f, 0x4d31,0x661b, 0xdd1b,0x7b27
        DCW 0x8882,0x2de2, 0x4d09,0x6639, 0xdcba,0x7b0b
        DCW 0x884c,0x2d55, 0x4ce1,0x6657, 0xdc59,0x7aef
        DCW 0x8817,0x2cc8, 0x4cb9,0x6675, 0xdbf9,0x7ad3
        DCW 0x87e2,0x2c3b, 0x4c91,0x6693, 0xdb99,0x7ab7
        DCW 0x87af,0x2bad, 0x4c68,0x66b2, 0xdb38,0x7a9a
        DCW 0x877b,0x2b1f, 0x4c40,0x66d0, 0xdad8,0x7a7d
        DCW 0x8749,0x2a91, 0x4c17,0x66ed, 0xda78,0x7a60
        DCW 0x8717,0x2a03, 0x4bef,0x670b, 0xda18,0x7a42
        DCW 0x86e6,0x2974, 0x4bc7,0x6729, 0xd9b8,0x7a24
        DCW 0x86b6,0x28e5, 0x4b9e,0x6747, 0xd958,0x7a06
        DCW 0x8686,0x2856, 0x4b75,0x6764, 0xd8f8,0x79e7
        DCW 0x8656,0x27c7, 0x4b4d,0x6782, 0xd898,0x79c9
        DCW 0x8628,0x2738, 0x4b24,0x67a0, 0xd839,0x79aa
        DCW 0x85fa,0x26a8, 0x4afb,0x67bd, 0xd7d9,0x798a
        DCW 0x85cd,0x2618, 0x4ad3,0x67da, 0xd77a,0x796a
        DCW 0x85a0,0x2588, 0x4aaa,0x67f8, 0xd71b,0x794a
        DCW 0x8574,0x24f8, 0x4a81,0x6815, 0xd6bb,0x792a
        DCW 0x8549,0x2467, 0x4a58,0x6832, 0xd65c,0x790a
        DCW 0x851f,0x23d7, 0x4a2f,0x6850, 0xd5fd,0x78e9
        DCW 0x84f5,0x2346, 0x4a06,0x686d, 0xd59e,0x78c8
        DCW 0x84cc,0x22b5, 0x49dd,0x688a, 0xd53f,0x78a6
        DCW 0x84a3,0x2224, 0x49b4,0x68a7, 0xd4e1,0x7885
        DCW 0x847b,0x2192, 0x498b,0x68c4, 0xd482,0x7863
        DCW 0x8454,0x2101, 0x4962,0x68e0, 0xd424,0x7840
        DCW 0x842d,0x206f, 0x4939,0x68fd, 0xd3c5,0x781e
        DCW 0x8407,0x1fdd, 0x490f,0x691a, 0xd367,0x77fb
        DCW 0x83e2,0x1f4b, 0x48e6,0x6937, 0xd309,0x77d8
        DCW 0x83be,0x1eb8, 0x48bd,0x6953, 0xd2ab,0x77b4
        DCW 0x839a,0x1e26, 0x4893,0x6970, 0xd24d,0x7790
        DCW 0x8377,0x1d93, 0x486a,0x698c, 0xd1ef,0x776c
        DCW 0x8354,0x1d01, 0x4840,0x69a9, 0xd191,0x7748
        DCW 0x8332,0x1c6e, 0x4817,0x69c5, 0xd134,0x7723
        DCW 0x8311,0x1bda, 0x47ed,0x69e1, 0xd0d6,0x76fe
        DCW 0x82f1,0x1b47, 0x47c4,0x69fd, 0xd079,0x76d9
        DCW 0x82d1,0x1ab4, 0x479a,0x6a1a, 0xd01b,0x76b4
        DCW 0x82b2,0x1a20, 0x4770,0x6a36, 0xcfbe,0x768e
        DCW 0x8293,0x198d, 0x4747,0x6a52, 0xcf61,0x7668
        DCW 0x8276,0x18f9, 0x471d,0x6a6e, 0xcf04,0x7642
        DCW 0x8259,0x1865, 0x46f3,0x6a89, 0xcea7,0x761b
        DCW 0x823c,0x17d1, 0x46c9,0x6aa5, 0xce4b,0x75f4
        DCW 0x8220,0x173c, 0x469f,0x6ac1, 0xcdee,0x75cd
        DCW 0x8205,0x16a8, 0x4675,0x6add, 0xcd92,0x75a6
        DCW 0x81eb,0x1614, 0x464b,0x6af8, 0xcd35,0x757e
        DCW 0x81d1,0x157f, 0x4621,0x6b14, 0xccd9,0x7556
        DCW 0x81b8,0x14ea, 0x45f7,0x6b30, 0xcc7d,0x752d
        DCW 0x81a0,0x1455, 0x45cd,0x6b4b, 0xcc21,0x7505
        DCW 0x8188,0x13c1, 0x45a3,0x6b66, 0xcbc5,0x74dc
        DCW 0x8172,0x132b, 0x4579,0x6b82, 0xcb69,0x74b3
        DCW 0x815b,0x1296, 0x454f,0x6b9d, 0xcb0e,0x7489
        DCW 0x8146,0x1201, 0x4524,0x6bb8, 0xcab2,0x7460
        DCW 0x8131,0x116c, 0x44fa,0x6bd3, 0xca57,0x7436
        DCW 0x811d,0x10d6, 0x44d0,0x6bee, 0xc9fc,0x740b
        DCW 0x8109,0x1041, 0x44a5,0x6c09, 0xc9a1,0x73e1
        DCW 0x80f6,0x0fab, 0x447b,0x6c24, 0xc946,0x73b6
        DCW 0x80e4,0x0f15, 0x4450,0x6c3f, 0xc8eb,0x738b
        DCW 0x80d3,0x0e80, 0x4426,0x6c5a, 0xc890,0x735f
        DCW 0x80c2,0x0dea, 0x43fb,0x6c75, 0xc836,0x7334
        DCW 0x80b2,0x0d54, 0x43d1,0x6c8f, 0xc7db,0x7308
        DCW 0x80a3,0x0cbe, 0x43a6,0x6caa, 0xc781,0x72dc
        DCW 0x8094,0x0c28, 0x437b,0x6cc4, 0xc727,0x72af
        DCW 0x8086,0x0b92, 0x4351,0x6cdf, 0xc6cd,0x7282
        DCW 0x8079,0x0afb, 0x4326,0x6cf9, 0xc673,0x7255
        DCW 0x806c,0x0a65, 0x42fb,0x6d14, 0xc619,0x7228
        DCW 0x8060,0x09cf, 0x42d0,0x6d2e, 0xc5c0,0x71fa
        DCW 0x8055,0x0938, 0x42a5,0x6d48, 0xc566,0x71cc
        DCW 0x804b,0x08a2, 0x427a,0x6d62, 0xc50d,0x719e
        DCW 0x8041,0x080c, 0x424f,0x6d7c, 0xc4b4,0x7170
        DCW 0x8038,0x0775, 0x4224,0x6d96, 0xc45b,0x7141
        DCW 0x802f,0x06de, 0x41f9,0x6db0, 0xc402,0x7112
        DCW 0x8027,0x0648, 0x41ce,0x6dca, 0xc3a9,0x70e3
        DCW 0x8020,0x05b1, 0x41a3,0x6de4, 0xc351,0x70b3
        DCW 0x801a,0x051b, 0x4178,0x6dfe, 0xc2f8,0x7083
        DCW 0x8014,0x0484, 0x414d,0x6e17, 0xc2a0,0x7053
        DCW 0x800f,0x03ed, 0x4121,0x6e31, 0xc248,0x7023
        DCW 0x800b,0x0356, 0x40f6,0x6e4a, 0xc1f0,0x6ff2
        DCW 0x8008,0x02c0, 0x40cb,0x6e64, 0xc198,0x6fc2
        DCW 0x8005,0x0229, 0x409f,0x6e7d, 0xc140,0x6f90
        DCW 0x8002,0x0192, 0x4074,0x6e97, 0xc0e9,0x6f5f
        DCW 0x8001,0x00fb, 0x4048,0x6eb0, 0xc091,0x6f2d
        DCW 0x8000,0x0065, 0x401d,0x6ec9, 0xc03a,0x6efb
        DCW 0x8000,0xffce, 0x3ff1,0x6ee2, 0xbfe3,0x6ec9
        DCW 0x8001,0xff37, 0x3fc6,0x6efb, 0xbf8c,0x6e97
        DCW 0x8002,0xfea0, 0x3f9a,0x6f14, 0xbf35,0x6e64
        DCW 0x8004,0xfe09, 0x3f6f,0x6f2d, 0xbedf,0x6e31
        DCW 0x8007,0xfd73, 0x3f43,0x6f46, 0xbe88,0x6dfe
        DCW 0x800a,0xfcdc, 0x3f17,0x6f5f, 0xbe32,0x6dca
        DCW 0x800e,0xfc45, 0x3eec,0x6f78, 0xbddc,0x6d96
        DCW 0x8013,0xfbae, 0x3ec0,0x6f90, 0xbd86,0x6d62
        DCW 0x8018,0xfb18, 0x3e94,0x6fa9, 0xbd30,0x6d2e
        DCW 0x801e,0xfa81, 0x3e68,0x6fc2, 0xbcda,0x6cf9
        DCW 0x8025,0xf9ea, 0x3e3c,0x6fda, 0xbc85,0x6cc4
        DCW 0x802d,0xf954, 0x3e10,0x6ff2, 0xbc2f,0x6c8f
        DCW 0x8035,0xf8bd, 0x3de4,0x700b, 0xbbda,0x6c5a
        DCW 0x803e,0xf827, 0x3db8,0x7023, 0xbb85,0x6c24
        DCW 0x8047,0xf790, 0x3d8c,0x703b, 0xbb30,0x6bee
        DCW 0x8052,0xf6fa, 0x3d60,0x7053, 0xbadc,0x6bb8
        DCW 0x805d,0xf663, 0x3d34,0x706b, 0xba87,0x6b82
        DCW 0x8068,0xf5cd, 0x3d08,0x7083, 0xba33,0x6b4b
        DCW 0x8075,0xf537, 0x3cdc,0x709b, 0xb9df,0x6b14
        DCW 0x8082,0xf4a0, 0x3caf,0x70b3, 0xb98b,0x6add
        DCW 0x808f,0xf40a, 0x3c83,0x70cb, 0xb937,0x6aa5
        DCW 0x809e,0xf374, 0x3c57,0x70e3, 0xb8e3,0x6a6e
        DCW 0x80ad,0xf2de, 0x3c2a,0x70fa, 0xb890,0x6a36
        DCW 0x80bd,0xf248, 0x3bfe,0x7112, 0xb83c,0x69fd
        DCW 0x80cd,0xf1b2, 0x3bd2,0x712a, 0xb7e9,0x69c5
        DCW 0x80de,0xf11c, 0x3ba5,0x7141, 0xb796,0x698c
        DCW 0x80f0,0xf087, 0x3b79,0x7158, 0xb743,0x6953
        DCW 0x8103,0xeff1, 0x3b4c,0x7170, 0xb6f1,0x691a
        DCW 0x8116,0xef5c, 0x3b20,0x7187, 0xb69e,0x68e0
        DCW 0x812a,0xeec6, 0x3af3,0x719e, 0xb64c,0x68a7
        DCW 0x813f,0xee31, 0x3ac6,0x71b5, 0xb5fa,0x686d
        DCW 0x8154,0xed9b, 0x3a9a,0x71cc, 0xb5a8,0x6832
        DCW 0x816a,0xed06, 0x3a6d,0x71e3, 0xb556,0x67f8
        DCW 0x8181,0xec71, 0x3a40,0x71fa, 0xb505,0x67bd
        DCW 0x8198,0xebdc, 0x3a13,0x7211, 0xb4b3,0x6782
        DCW 0x81b0,0xeb47, 0x39e7,0x7228, 0xb462,0x6747
        DCW 0x81c9,0xeab3, 0x39ba,0x723f, 0xb411,0x670b
        DCW 0x81e2,0xea1e, 0x398d,0x7255, 0xb3c0,0x66d0
        DCW 0x81fd,0xe989, 0x3960,0x726c, 0xb36f,0x6693
        DCW 0x8217,0xe8f5, 0x3933,0x7282, 0xb31f,0x6657
        DCW 0x8233,0xe861, 0x3906,0x7299, 0xb2cf,0x661b
        DCW 0x824f,0xe7cd, 0x38d9,0x72af, 0xb27f,0x65de
        DCW 0x826c,0xe739, 0x38ac,0x72c5, 0xb22f,0x65a1
        DCW 0x8289,0xe6a5, 0x387f,0x72dc, 0xb1df,0x6564
        DCW 0x82a8,0xe611, 0x3852,0x72f2, 0xb18f,0x6526
        DCW 0x82c6,0xe57d, 0x3825,0x7308, 0xb140,0x64e9
        DCW 0x82e6,0xe4ea, 0x37f7,0x731e, 0xb0f1,0x64ab
        DCW 0x8306,0xe457, 0x37ca,0x7334, 0xb0a2,0x646c
        DCW 0x8327,0xe3c3, 0x379d,0x734a, 0xb053,0x642e
        DCW 0x8349,0xe330, 0x3770,0x735f, 0xb005,0x63ef
        DCW 0x836b,0xe29e, 0x3742,0x7375, 0xafb6,0x63b0
        DCW 0x838e,0xe20b, 0x3715,0x738b, 0xaf68,0x6371
        DCW 0x83b2,0xe178, 0x36e8,0x73a0, 0xaf1a,0x6332
        DCW 0x83d6,0xe0e6, 0x36ba,0x73b6, 0xaecc,0x62f2
        DCW 0x83fb,0xe054, 0x368d,0x73cb, 0xae7f,0x62b2
        DCW 0x8421,0xdfc2, 0x365f,0x73e1, 0xae31,0x6272
        DCW 0x8447,0xdf30, 0x3632,0x73f6, 0xade4,0x6232
        DCW 0x846e,0xde9e, 0x3604,0x740b, 0xad97,0x61f1
        DCW 0x8496,0xde0d, 0x35d7,0x7421, 0xad4a,0x61b0
        DCW 0x84be,0xdd7c, 0x35a9,0x7436, 0xacfd,0x616f
        DCW 0x84e7,0xdcea, 0x357b,0x744b, 0xacb1,0x612e
        DCW 0x8511,0xdc59, 0x354e,0x7460, 0xac65,0x60ec
        DCW 0x853b,0xdbc9, 0x3520,0x7475, 0xac19,0x60aa
        DCW 0x8566,0xdb38, 0x34f2,0x7489, 0xabcd,0x6068
        DCW 0x8592,0xdaa8, 0x34c4,0x749e, 0xab81,0x6026
        DCW 0x85be,0xda18, 0x3497,0x74b3, 0xab36,0x5fe4
        DCW 0x85eb,0xd988, 0x3469,0x74c7, 0xaaeb,0x5fa1
        DCW 0x8619,0xd8f8, 0x343b,0x74dc, 0xaaa0,0x5f5e
        DCW 0x8647,0xd869, 0x340d,0x74f0, 0xaa55,0x5f1b
        DCW 0x8676,0xd7d9, 0x33df,0x7505, 0xaa0a,0x5ed7
        DCW 0x86a5,0xd74a, 0x33b1,0x7519, 0xa9c0,0x5e94
        DCW 0x86d6,0xd6bb, 0x3383,0x752d, 0xa976,0x5e50
        DCW 0x8707,0xd62d, 0x3355,0x7542, 0xa92c,0x5e0c
        DCW 0x8738,0xd59e, 0x3327,0x7556, 0xa8e2,0x5dc8
        DCW 0x876b,0xd510, 0x32f9,0x756a, 0xa899,0x5d83
        DCW 0x879d,0xd482, 0x32cb,0x757e, 0xa84f,0x5d3e
        DCW 0x87d1,0xd3f4, 0x329d,0x7592, 0xa806,0x5cf9
        DCW 0x8805,0xd367, 0x326e,0x75a6, 0xa7bd,0x5cb4
        DCW 0x883a,0xd2da, 0x3240,0x75b9, 0xa774,0x5c6f
        DCW 0x8870,0xd24d, 0x3212,0x75cd, 0xa72c,0x5c29
        DCW 0x88a6,0xd1c0, 0x31e4,0x75e1, 0xa6e4,0x5be3
        DCW 0x88dd,0xd134, 0x31b5,0x75f4, 0xa69c,0x5b9d
        DCW 0x8914,0xd0a7, 0x3187,0x7608, 0xa654,0x5b57
        DCW 0x894c,0xd01b, 0x3159,0x761b, 0xa60c,0x5b10
        DCW 0x8985,0xcf90, 0x312a,0x762e, 0xa5c5,0x5ac9
        DCW 0x89be,0xcf04, 0x30fc,0x7642, 0xa57e,0x5a82
        DCW 0x89f8,0xce79, 0x30cd,0x7655, 0xa537,0x5a3b
        DCW 0x8a33,0xcdee, 0x309f,0x7668, 0xa4f0,0x59f4
        DCW 0x8a6e,0xcd63, 0x3070,0x767b, 0xa4a9,0x59ac
        DCW 0x8aaa,0xccd9, 0x3042,0x768e, 0xa463,0x5964
        DCW 0x8ae7,0xcc4f, 0x3013,0x76a1, 0xa41d,0x591c
        DCW 0x8b24,0xcbc5, 0x2fe5,0x76b4, 0xa3d7,0x58d4
        DCW 0x8b62,0xcb3c, 0x2fb6,0x76c7, 0xa391,0x588c
        DCW 0x8ba0,0xcab2, 0x2f87,0x76d9, 0xa34c,0x5843
        DCW 0x8bdf,0xca29, 0x2f59,0x76ec, 0xa307,0x57fa
        DCW 0x8c1f,0xc9a1, 0x2f2a,0x76fe, 0xa2c2,0x57b1
        DCW 0x8c60,0xc918, 0x2efb,0x7711, 0xa27d,0x5767
        DCW 0x8ca1,0xc890, 0x2ecc,0x7723, 0xa238,0x571e
        DCW 0x8ce2,0xc809, 0x2e9e,0x7736, 0xa1f4,0x56d4
        DCW 0x8d24,0xc781, 0x2e6f,0x7748, 0xa1b0,0x568a
        DCW 0x8d67,0xc6fa, 0x2e40,0x775a, 0xa16c,0x5640
        DCW 0x8dab,0xc673, 0x2e11,0x776c, 0xa129,0x55f6
        DCW 0x8def,0xc5ed, 0x2de2,0x777e, 0xa0e5,0x55ab
        DCW 0x8e34,0xc566, 0x2db3,0x7790, 0xa0a2,0x5560
        DCW 0x8e79,0xc4e0, 0x2d84,0x77a2, 0xa05f,0x5515
        DCW 0x8ebf,0xc45b, 0x2d55,0x77b4, 0xa01c,0x54ca
        DCW 0x8f06,0xc3d6, 0x2d26,0x77c6, 0x9fda,0x547f
        DCW 0x8f4d,0xc351, 0x2cf7,0x77d8, 0x9f98,0x5433
        DCW 0x8f95,0xc2cc, 0x2cc8,0x77e9, 0x9f56,0x53e7
        DCW 0x8fdd,0xc248, 0x2c99,0x77fb, 0x9f14,0x539b
        DCW 0x9026,0xc1c4, 0x2c6a,0x780c, 0x9ed2,0x534f
        DCW 0x9070,0xc140, 0x2c3b,0x781e, 0x9e91,0x5303
        DCW 0x90ba,0xc0bd, 0x2c0c,0x782f, 0x9e50,0x52b6
        DCW 0x9105,0xc03a, 0x2bdc,0x7840, 0x9e0f,0x5269
        DCW 0x9150,0xbfb8, 0x2bad,0x7851, 0x9dce,0x521c
        DCW 0x919c,0xbf35, 0x2b7e,0x7863, 0x9d8e,0x51cf
        DCW 0x91e9,0xbeb3, 0x2b4f,0x7874, 0x9d4e,0x5181
        DCW 0x9236,0xbe32, 0x2b1f,0x7885, 0x9d0e,0x5134
        DCW 0x9284,0xbdb1, 0x2af0,0x7895, 0x9cce,0x50e6
        DCW 0x92d2,0xbd30, 0x2ac1,0x78a6, 0x9c8f,0x5098
        DCW 0x9321,0xbcaf, 0x2a91,0x78b7, 0x9c50,0x504a
        DCW 0x9371,0xbc2f, 0x2a62,0x78c8, 0x9c11,0x4ffb
        DCW 0x93c1,0xbbb0, 0x2a32,0x78d8, 0x9bd2,0x4fad
        DCW 0x9412,0xbb30, 0x2a03,0x78e9, 0x9b94,0x4f5e
        DCW 0x9463,0xbab1, 0x29d3,0x78f9, 0x9b55,0x4f0f
        DCW 0x94b5,0xba33, 0x29a4,0x790a, 0x9b17,0x4ec0
        DCW 0x9508,0xb9b5, 0x2974,0x791a, 0x9ada,0x4e71
        DCW 0x955b,0xb937, 0x2945,0x792a, 0x9a9c,0x4e21
        DCW 0x95ae,0xb8b9, 0x2915,0x793a, 0x9a5f,0x4dd1
        DCW 0x9603,0xb83c, 0x28e5,0x794a, 0x9a22,0x4d81
        DCW 0x9657,0xb7c0, 0x28b6,0x795b, 0x99e5,0x4d31
        DCW 0x96ad,0xb743, 0x2886,0x796a, 0x99a9,0x4ce1
        DCW 0x9703,0xb6c7, 0x2856,0x797a, 0x996d,0x4c91
        DCW 0x9759,0xb64c, 0x2827,0x798a, 0x9930,0x4c40
        DCW 0x97b0,0xb5d1, 0x27f7,0x799a, 0x98f5,0x4bef
        DCW 0x9808,0xb556, 0x27c7,0x79aa, 0x98b9,0x4b9e
        DCW 0x9860,0xb4dc, 0x2797,0x79b9, 0x987e,0x4b4d
        DCW 0x98b9,0xb462, 0x2768,0x79c9, 0x9843,0x4afb
        DCW 0x9913,0xb3e9, 0x2738,0x79d8, 0x9808,0x4aaa
        DCW 0x996d,0xb36f, 0x2708,0x79e7, 0x97ce,0x4a58
        DCW 0x99c7,0xb2f7, 0x26d8,0x79f7, 0x9793,0x4a06
        DCW 0x9a22,0xb27f, 0x26a8,0x7a06, 0x9759,0x49b4
        DCW 0x9a7e,0xb207, 0x2678,0x7a15, 0x9720,0x4962
        DCW 0x9ada,0xb18f, 0x2648,0x7a24, 0x96e6,0x490f
        DCW 0x9b36,0xb118, 0x2618,0x7a33, 0x96ad,0x48bd
        DCW 0x9b94,0xb0a2, 0x25e8,0x7a42, 0x9674,0x486a
        DCW 0x9bf1,0xb02c, 0x25b8,0x7a51, 0x963b,0x4817
        DCW 0x9c50,0xafb6, 0x2588,0x7a60, 0x9603,0x47c4
        DCW 0x9caf,0xaf41, 0x2558,0x7a6e, 0x95ca,0x4770
        DCW 0x9d0e,0xaecc, 0x2528,0x7a7d, 0x9592,0x471d
        DCW 0x9d6e,0xae58, 0x24f8,0x7a8c, 0x955b,0x46c9
        DCW 0x9dce,0xade4, 0x24c8,0x7a9a, 0x9523,0x4675
        DCW 0x9e2f,0xad70, 0x2498,0x7aa8, 0x94ec,0x4621
        DCW 0x9e91,0xacfd, 0x2467,0x7ab7, 0x94b5,0x45cd
        DCW 0x9ef3,0xac8b, 0x2437,0x7ac5, 0x947e,0x4579
        DCW 0x9f56,0xac19, 0x2407,0x7ad3, 0x9448,0x4524
        DCW 0x9fb9,0xaba7, 0x23d7,0x7ae1, 0x9412,0x44d0
        DCW 0xa01c,0xab36, 0x23a7,0x7aef, 0x93dc,0x447b
        DCW 0xa080,0xaac5, 0x2376,0x7afd, 0x93a6,0x4426
        DCW 0xa0e5,0xaa55, 0x2346,0x7b0b, 0x9371,0x43d1
        DCW 0xa14a,0xa9e5, 0x2316,0x7b19, 0x933c,0x437b
        DCW 0xa1b0,0xa976, 0x22e5,0x7b27, 0x9307,0x4326
        DCW 0xa216,0xa907, 0x22b5,0x7b34, 0x92d2,0x42d0
        DCW 0xa27d,0xa899, 0x2284,0x7b42, 0x929e,0x427a
        DCW 0xa2e4,0xa82b, 0x2254,0x7b50, 0x926a,0x4224
        DCW 0xa34c,0xa7bd, 0x2224,0x7b5d, 0x9236,0x41ce
        DCW 0xa3b4,0xa750, 0x21f3,0x7b6a, 0x9202,0x4178
        DCW 0xa41d,0xa6e4, 0x21c3,0x7b78, 0x91cf,0x4121
        DCW 0xa486,0xa678, 0x2192,0x7b85, 0x919c,0x40cb
        DCW 0xa4f0,0xa60c, 0x2162,0x7b92, 0x9169,0x4074
        DCW 0xa55a,0xa5a1, 0x2131,0x7b9f, 0x9137,0x401d
        DCW 0xa5c5,0xa537, 0x2101,0x7bac, 0x9105,0x3fc6
        DCW 0xa630,0xa4cc, 0x20d0,0x7bb9, 0x90d3,0x3f6f
        DCW 0xa69c,0xa463, 0x209f,0x7bc6, 0x90a1,0x3f17
        DCW 0xa708,0xa3fa, 0x206f,0x7bd3, 0x9070,0x3ec0
        DCW 0xa774,0xa391, 0x203e,0x7bdf, 0x903e,0x3e68
        DCW 0xa7e2,0xa329, 0x200e,0x7bec, 0x900e,0x3e10
        DCW 0xa84f,0xa2c2, 0x1fdd,0x7bf9, 0x8fdd,0x3db8
        DCW 0xa8bd,0xa25b, 0x1fac,0x7c05, 0x8fad,0x3d60
        DCW 0xa92c,0xa1f4, 0x1f7b,0x7c11, 0x8f7d,0x3d08
        DCW 0xa99b,0xa18e, 0x1f4b,0x7c1e, 0x8f4d,0x3caf
        DCW 0xaa0a,0xa129, 0x1f1a,0x7c2a, 0x8f1d,0x3c57
        DCW 0xaa7a,0xa0c4, 0x1ee9,0x7c36, 0x8eee,0x3bfe
        DCW 0xaaeb,0xa05f, 0x1eb8,0x7c42, 0x8ebf,0x3ba5
        DCW 0xab5c,0x9ffb, 0x1e88,0x7c4e, 0x8e90,0x3b4c
        DCW 0xabcd,0x9f98, 0x1e57,0x7c5a, 0x8e62,0x3af3
        DCW 0xac3f,0x9f35, 0x1e26,0x7c66, 0x8e34,0x3a9a
        DCW 0xacb1,0x9ed2, 0x1df5,0x7c72, 0x8e06,0x3a40
        DCW 0xad24,0x9e70, 0x1dc4,0x7c7e, 0x8dd8,0x39e7
        DCW 0xad97,0x9e0f, 0x1d93,0x7c89, 0x8dab,0x398d
        DCW 0xae0b,0x9dae, 0x1d62,0x7c95, 0x8d7e,0x3933
        DCW 0xae7f,0x9d4e, 0x1d31,0x7ca0, 0x8d51,0x38d9
        DCW 0xaef3,0x9cee, 0x1d01,0x7cac, 0x8d24,0x387f
        DCW 0xaf68,0x9c8f, 0x1cd0,0x7cb7, 0x8cf8,0x3825
        DCW 0xafdd,0x9c30, 0x1c9f,0x7cc2, 0x8ccc,0x37ca
        DCW 0xb053,0x9bd2, 0x1c6e,0x7cce, 0x8ca1,0x3770
        DCW 0xb0c9,0x9b75, 0x1c3d,0x7cd9, 0x8c75,0x3715
        DCW 0xb140,0x9b17, 0x1c0c,0x7ce4, 0x8c4a,0x36ba
        DCW 0xb1b7,0x9abb, 0x1bda,0x7cef, 0x8c1f,0x365f
        DCW 0xb22f,0x9a5f, 0x1ba9,0x7cfa, 0x8bf5,0x3604
        DCW 0xb2a7,0x9a04, 0x1b78,0x7d05, 0x8bca,0x35a9
        DCW 0xb31f,0x99a9, 0x1b47,0x7d0f, 0x8ba0,0x354e
        DCW 0xb398,0x994e, 0x1b16,0x7d1a, 0x8b77,0x34f2
        DCW 0xb411,0x98f5, 0x1ae5,0x7d25, 0x8b4d,0x3497
        DCW 0xb48b,0x989c, 0x1ab4,0x7d2f, 0x8b24,0x343b
        DCW 0xb505,0x9843, 0x1a83,0x7d3a, 0x8afb,0x33df
        DCW 0xb57f,0x97eb, 0x1a51,0x7d44, 0x8ad3,0x3383
        DCW 0xb5fa,0x9793, 0x1a20,0x7d4e, 0x8aaa,0x3327
        DCW 0xb675,0x973c, 0x19ef,0x7d58, 0x8a82,0x32cb
        DCW 0xb6f1,0x96e6, 0x19be,0x7d63, 0x8a5a,0x326e
        DCW 0xb76d,0x9690, 0x198d,0x7d6d, 0x8a33,0x3212
        DCW 0xb7e9,0x963b, 0x195b,0x7d77, 0x8a0c,0x31b5
        DCW 0xb866,0x95e6, 0x192a,0x7d81, 0x89e5,0x3159
        DCW 0xb8e3,0x9592, 0x18f9,0x7d8a, 0x89be,0x30fc
        DCW 0xb961,0x953f, 0x18c7,0x7d94, 0x8998,0x309f
        DCW 0xb9df,0x94ec, 0x1896,0x7d9e, 0x8972,0x3042
        DCW 0xba5d,0x949a, 0x1865,0x7da7, 0x894c,0x2fe5
        DCW 0xbadc,0x9448, 0x1833,0x7db1, 0x8927,0x2f87
        DCW 0xbb5b,0x93f7, 0x1802,0x7dba, 0x8902,0x2f2a
        DCW 0xbbda,0x93a6, 0x17d1,0x7dc4, 0x88dd,0x2ecc
        DCW 0xbc5a,0x9356, 0x179f,0x7dcd, 0x88b8,0x2e6f
        DCW 0xbcda,0x9307, 0x176e,0x7dd6, 0x8894,0x2e11
        DCW 0xbd5b,0x92b8, 0x173c,0x7de0, 0x8870,0x2db3
        DCW 0xbddc,0x926a, 0x170b,0x7de9, 0x884c,0x2d55
        DCW 0xbe5d,0x921c, 0x16da,0x7df2, 0x8828,0x2cf7
        DCW 0xbedf,0x91cf, 0x16a8,0x7dfb, 0x8805,0x2c99
        DCW 0xbf61,0x9183, 0x1677,0x7e03, 0x87e2,0x2c3b
        DCW 0xbfe3,0x9137, 0x1645,0x7e0c, 0x87c0,0x2bdc
        DCW 0xc066,0x90ec, 0x1614,0x7e15, 0x879d,0x2b7e
        DCW 0xc0e9,0x90a1, 0x15e2,0x7e1e, 0x877b,0x2b1f
        DCW 0xc16c,0x9057, 0x15b1,0x7e26, 0x875a,0x2ac1
        DCW 0xc1f0,0x900e, 0x157f,0x7e2f, 0x8738,0x2a62
        DCW 0xc274,0x8fc5, 0x154d,0x7e37, 0x8717,0x2a03
        DCW 0xc2f8,0x8f7d, 0x151c,0x7e3f, 0x86f6,0x29a4
        DCW 0xc37d,0x8f35, 0x14ea,0x7e48, 0x86d6,0x2945
        DCW 0xc402,0x8eee, 0x14b9,0x7e50, 0x86b6,0x28e5
        DCW 0xc487,0x8ea8, 0x1487,0x7e58, 0x8696,0x2886
        DCW 0xc50d,0x8e62, 0x1455,0x7e60, 0x8676,0x2827
        DCW 0xc593,0x8e1d, 0x1424,0x7e68, 0x8656,0x27c7
        DCW 0xc619,0x8dd8, 0x13f2,0x7e70, 0x8637,0x2768
        DCW 0xc6a0,0x8d94, 0x13c1,0x7e78, 0x8619,0x2708
        DCW 0xc727,0x8d51, 0x138f,0x7e7f, 0x85fa,0x26a8
        DCW 0xc7ae,0x8d0e, 0x135d,0x7e87, 0x85dc,0x2648
        DCW 0xc836,0x8ccc, 0x132b,0x7e8e, 0x85be,0x25e8
        DCW 0xc8be,0x8c8b, 0x12fa,0x7e96, 0x85a0,0x2588
        DCW 0xc946,0x8c4a, 0x12c8,0x7e9d, 0x8583,0x2528
        DCW 0xc9ce,0x8c0a, 0x1296,0x7ea5, 0x8566,0x24c8
        DCW 0xca57,0x8bca, 0x1265,0x7eac, 0x8549,0x2467
        DCW 0xcae0,0x8b8b, 0x1233,0x7eb3, 0x852d,0x2407
        DCW 0xcb69,0x8b4d, 0x1201,0x7eba, 0x8511,0x23a7
        DCW 0xcbf3,0x8b10, 0x11cf,0x7ec1, 0x84f5,0x2346
        DCW 0xcc7d,0x8ad3, 0x119e,0x7ec8, 0x84d9,0x22e5
        DCW 0xcd07,0x8a96, 0x116c,0x7ecf, 0x84be,0x2284
        DCW 0xcd92,0x8a5a, 0x113a,0x7ed6, 0x84a3,0x2224
        DCW 0xce1c,0x8a1f, 0x1108,0x7edd, 0x8488,0x21c3
        DCW 0xcea7,0x89e5, 0x10d6,0x7ee3, 0x846e,0x2162
        DCW 0xcf33,0x89ab, 0x10a4,0x7eea, 0x8454,0x2101
        DCW 0xcfbe,0x8972, 0x1073,0x7ef0, 0x843a,0x209f
        DCW 0xd04a,0x8939, 0x1041,0x7ef7, 0x8421,0x203e
        DCW 0xd0d6,0x8902, 0x100f,0x7efd, 0x8407,0x1fdd
        DCW 0xd162,0x88ca, 0x0fdd,0x7f03, 0x83ef,0x1f7b
        DCW 0xd1ef,0x8894, 0x0fab,0x7f0a, 0x83d6,0x1f1a
        DCW 0xd27c,0x885e, 0x0f79,0x7f10, 0x83be,0x1eb8
        DCW 0xd309,0x8828, 0x0f47,0x7f16, 0x83a6,0x1e57
        DCW 0xd396,0x87f4, 0x0f15,0x7f1c, 0x838e,0x1df5
        DCW 0xd424,0x87c0, 0x0ee4,0x7f22, 0x8377,0x1d93
        DCW 0xd4b1,0x878c, 0x0eb2,0x7f27, 0x8360,0x1d31
        DCW 0xd53f,0x875a, 0x0e80,0x7f2d, 0x8349,0x1cd0
        DCW 0xd5ce,0x8728, 0x0e4e,0x7f33, 0x8332,0x1c6e
        DCW 0xd65c,0x86f6, 0x0e1c,0x7f38, 0x831c,0x1c0c
        DCW 0xd6eb,0x86c6, 0x0dea,0x7f3e, 0x8306,0x1ba9
        DCW 0xd77a,0x8696, 0x0db8,0x7f43, 0x82f1,0x1b47
        DCW 0xd809,0x8666, 0x0d86,0x7f49, 0x82db,0x1ae5
        DCW 0xd898,0x8637, 0x0d54,0x7f4e, 0x82c6,0x1a83
        DCW 0xd928,0x8609, 0x0d22,0x7f53, 0x82b2,0x1a20
        DCW 0xd9b8,0x85dc, 0x0cf0,0x7f58, 0x829d,0x19be
        DCW 0xda48,0x85af, 0x0cbe,0x7f5d, 0x8289,0x195b
        DCW 0xdad8,0x8583, 0x0c8c,0x7f62, 0x8276,0x18f9
        DCW 0xdb68,0x8558, 0x0c5a,0x7f67, 0x8262,0x1896
        DCW 0xdbf9,0x852d, 0x0c28,0x7f6c, 0x824f,0x1833
        DCW 0xdc8a,0x8503, 0x0bf6,0x7f71, 0x823c,0x17d1
        DCW 0xdd1b,0x84d9, 0x0bc4,0x7f75, 0x822a,0x176e
        DCW 0xddac,0x84b0, 0x0b92,0x7f7a, 0x8217,0x170b
        DCW 0xde3d,0x8488, 0x0b60,0x7f7e, 0x8205,0x16a8
        DCW 0xdecf,0x8461, 0x0b2d,0x7f83, 0x81f4,0x1645
        DCW 0xdf61,0x843a, 0x0afb,0x7f87, 0x81e2,0x15e2
        DCW 0xdff2,0x8414, 0x0ac9,0x7f8b, 0x81d1,0x157f
        DCW 0xe085,0x83ef, 0x0a97,0x7f90, 0x81c1,0x151c
        DCW 0xe117,0x83ca, 0x0a65,0x7f94, 0x81b0,0x14b9
        DCW 0xe1a9,0x83a6, 0x0a33,0x7f98, 0x81a0,0x1455
        DCW 0xe23c,0x8382, 0x0a01,0x7f9c, 0x8190,0x13f2
        DCW 0xe2cf,0x8360, 0x09cf,0x7fa0, 0x8181,0x138f
        DCW 0xe361,0x833e, 0x099d,0x7fa3, 0x8172,0x132b
        DCW 0xe3f4,0x831c, 0x096b,0x7fa7, 0x8163,0x12c8
        DCW 0xe488,0x82fb, 0x0938,0x7fab, 0x8154,0x1265
        DCW 0xe51b,0x82db, 0x0906,0x7fae, 0x8146,0x1201
        DCW 0xe5af,0x82bc, 0x08d4,0x7fb2, 0x8138,0x119e
        DCW 0xe642,0x829d, 0x08a2,0x7fb5, 0x812a,0x113a
        DCW 0xe6d6,0x827f, 0x0870,0x7fb9, 0x811d,0x10d6
        DCW 0xe76a,0x8262, 0x083e,0x7fbc, 0x8110,0x1073
        DCW 0xe7fe,0x8246, 0x080c,0x7fbf, 0x8103,0x100f
        DCW 0xe892,0x822a, 0x07d9,0x7fc2, 0x80f6,0x0fab
        DCW 0xe926,0x820e, 0x07a7,0x7fc5, 0x80ea,0x0f47
        DCW 0xe9bb,0x81f4, 0x0775,0x7fc8, 0x80de,0x0ee4
        DCW 0xea4f,0x81da, 0x0743,0x7fcb, 0x80d3,0x0e80
        DCW 0xeae4,0x81c1, 0x0711,0x7fce, 0x80c8,0x0e1c
        DCW 0xeb79,0x81a8, 0x06de,0x7fd1, 0x80bd,0x0db8
        DCW 0xec0e,0x8190, 0x06ac,0x7fd3, 0x80b2,0x0d54
        DCW 0xeca3,0x8179, 0x067a,0x7fd6, 0x80a8,0x0cf0
        DCW 0xed38,0x8163, 0x0648,0x7fd9, 0x809e,0x0c8c
        DCW 0xedcd,0x814d, 0x0616,0x7fdb, 0x8094,0x0c28
        DCW 0xee62,0x8138, 0x05e3,0x7fdd, 0x808b,0x0bc4
        DCW 0xeef8,0x8123, 0x05b1,0x7fe0, 0x8082,0x0b60
        DCW 0xef8d,0x8110, 0x057f,0x7fe2, 0x8079,0x0afb
        DCW 0xf023,0x80fd, 0x054d,0x7fe4, 0x8070,0x0a97
        DCW 0xf0b9,0x80ea, 0x051b,0x7fe6, 0x8068,0x0a33
        DCW 0xf14e,0x80d9, 0x04e8,0x7fe8, 0x8060,0x09cf
        DCW 0xf1e4,0x80c8, 0x04b6,0x7fea, 0x8059,0x096b
        DCW 0xf27a,0x80b7, 0x0484,0x7fec, 0x8052,0x0906
        DCW 0xf310,0x80a8, 0x0452,0x7fed, 0x804b,0x08a2
        DCW 0xf3a6,0x8099, 0x041f,0x7fef, 0x8044,0x083e
        DCW 0xf43c,0x808b, 0x03ed,0x7ff1, 0x803e,0x07d9
        DCW 0xf4d3,0x807d, 0x03bb,0x7ff2, 0x8038,0x0775
        DCW 0xf569,0x8070, 0x0389,0x7ff4, 0x8032,0x0711
        DCW 0xf5ff,0x8064, 0x0356,0x7ff5, 0x802d,0x06ac
        DCW 0xf695,0x8059, 0x0324,0x7ff6, 0x8027,0x0648
        DCW 0xf72c,0x804e, 0x02f2,0x7ff7, 0x8023,0x05e3
        DCW 0xf7c2,0x8044, 0x02c0,0x7ff8, 0x801e,0x057f
        DCW 0xf859,0x803b, 0x028d,0x7ff9, 0x801a,0x051b
        DCW 0xf8ef,0x8032, 0x025b,0x7ffa, 0x8016,0x04b6
        DCW 0xf986,0x802a, 0x0229,0x7ffb, 0x8013,0x0452
        DCW 0xfa1d,0x8023, 0x01f7,0x7ffc, 0x800f,0x03ed
        DCW 0xfab3,0x801c, 0x01c4,0x7ffd, 0x800c,0x0389
        DCW 0xfb4a,0x8016, 0x0192,0x7ffe, 0x800a,0x0324
        DCW 0xfbe1,0x8011, 0x0160,0x7ffe, 0x8008,0x02c0
        DCW 0xfc77,0x800c, 0x012e,0x7fff, 0x8006,0x025b
        DCW 0xfd0e,0x8009, 0x00fb,0x7fff, 0x8004,0x01f7
        DCW 0xfda5,0x8006, 0x00c9,0x7fff, 0x8002,0x0192
        DCW 0xfe3c,0x8003, 0x0097,0x7fff, 0x8001,0x012e
        DCW 0xfed2,0x8001, 0x0065,0x7fff, 0x8001,0x00c9
        DCW 0xff69,0x8000, 0x0032,0x7fff, 0x8000,0x0065

        END
